@@ -10,7 +10,7 @@
  *
  *  The msXpertSuite project is the successor of the massXpert project. This
  *  project now includes various independent modules:
- *  
+ *
  *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
  *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
@@ -26,7 +26,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
+ *
  * END software license
  */
 
@@ -55,6 +55,21 @@ namespace msXpS
 {
 
 
+	//! Map relating the MassToleranceType to a textual representation
+	QMap<int, QString> massToleranceTypeMap{ {MassToleranceType::MASS_TOLERANCE_NONE, "NONE"},
+		{MassToleranceType::MASS_TOLERANCE_PPM, "PPM"},
+		{MassToleranceType::MASS_TOLERANCE_MZ, "MZ"},
+		{MassToleranceType::MASS_TOLERANCE_AMU, "AMU"},
+		{MassToleranceType::MASS_TOLERANCE_RES, "RES"}};
+
+
+	//! Map relating the BinningType to a textual representation
+	QMap<int, QString> binningTypeMap{ {BinningType::BINNING_TYPE_NONE, "None"},
+		{BinningType::BINNING_TYPE_DATA_BASED, "Data-based"},
+		{BinningType::BINNING_TYPE_ARBITRARY, "Arbitrary"}
+	};
+
+
 	//! Define the character to use as a prefix delimiter in QSettings-based
 	// names.
 	/*!
@@ -101,13 +116,61 @@ namespace msXpS
 	/**@}*/
 
 	//! Regular expression that tracks the m/z,i pairs in text files.
-	QRegularExpression gXyFormatMassDataRegExp = 
+	QRegularExpression gXyFormatMassDataRegExp =
 		QRegularExpression("^(\\d*\\.?\\d+)([^\\d^\\.^-]+)(-?\\d*\\.?\\d*[e-]?\\d*)");
 
 	//! Regular expression that tracks the end of line in text files.
 	QRegularExpression gEndOfLineRegExp = QRegularExpression("^\\s+$") ;
 
 
+	//! Return the average of the double list. Also fill-in the variance and standard deviation if the parameters are non-nullptr.
+	void
+		doubleListStatistics(const QList<double> &list, double *sum, double *average,
+				double *variance, double *stdDeviation,
+				double *smallest, double *greatest)
+		{
+			if(sum == Q_NULLPTR || average == Q_NULLPTR || variance == Q_NULLPTR ||
+					stdDeviation == Q_NULLPTR || smallest == Q_NULLPTR || greatest == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Pointers cannot be nullptr."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			int count = list.size();
+
+			if(count == 0)
+				return ;
+
+			*sum = 0;
+			*smallest = std::numeric_limits<double>::max();
+			*greatest = 0;
+
+			for(int iter = 0; iter < count; ++iter)
+			{
+				double current = list.at(iter);
+				*sum += current;
+
+				if(current < *smallest)
+					*smallest = current;
+
+				if(current > *greatest)
+					*greatest = current;
+			}
+
+			*average = *sum / count;
+
+			double varN = 0;
+
+			for(int iter = 0; iter < count; ++iter)
+			{
+				varN += (list.at(iter) - *average) * (list.at(iter) - *average);
+			}
+
+			*variance = varN / count;
+			*stdDeviation = sqrt(*variance);
+		}
+
+
 	//! Tell if both double values, are equal within the double representation capabilities of the platform.
 	bool
 		almostEqual(double value1, double value2, int decimalPlaces)
@@ -140,23 +203,23 @@ namespace msXpS
 			//QString scaleFactorString = QString("%1").arg(scaleFactor,
 			//0, 'f', 60);
 
-			//qWarning() << "valueDiff:" << valueDiffString << "valueSum:" << valueSumString << 
+			//qWarning() << "valueDiff:" << valueDiffString << "valueSum:" << valueSumString <<
 			//"epsilon:" << epsilonString << "scaleFactor:" << scaleFactorString;
 
-			bool res =  valueDiff < scaleFactor 
+			bool res =  valueDiff < scaleFactor
 				// unless the result is subnormal:
 				|| valueDiff < std::numeric_limits<double>::min();
 
 			//qWarning() << __FILE__ << __LINE__ << __FUNCTION__
 			//<< "returning res:" << res;
 
-			return res;	
+			return res;
 		}
 
 
 	//! Provides a text stream handle to the standard output.
-	/*! 
-	 * Use:   msXpS::qStdOut() << __FILE__ << __LINE__ 
+	/*!
+	 * Use:   msXpS::qStdOut() << __FILE__ << __LINE__
 	 *                         << "text to the standard output,"
 	 *                         << "not the error standard output."
 	 *
@@ -172,7 +235,7 @@ namespace msXpS
 
 
 	//! Remove all space characters from a string.
-	/*! 
+	/*!
 	 * \param text reference to the string
 	 * \return reference to the in-place-modified string
 	 */
@@ -191,7 +254,7 @@ namespace msXpS
 
 
 	//! Produce a binary representation of an int
-	/*! 
+	/*!
 	 * \param value int value
 	 * \return a QString with the value binary representation
 	 */
@@ -253,7 +316,7 @@ namespace msXpS
 
 
 	//! Convert a very long text to a paragraph.
-	/*! 
+	/*!
 	 * When a text string is too long to be displayed in a line of reasonable
 	 * length, insert newline characters at positions calculated to yield a
 	 * paragraph of a given width.
@@ -264,7 +327,7 @@ namespace msXpS
 	 * \param width width in characters of the stanza
 	 * \return a new string with the stanzified text
 	 */
-	QString 
+	QString
 		stanzify(const QString &text, int width)
 		{
 			QString result = text;
@@ -301,7 +364,7 @@ namespace msXpS
 
 
 	//! Convert a very long text to paragraphs.
-	/*! 
+	/*!
 	 * When a text string is too long to be displayed in a line of reasonable
 	 * length, insert newline characters at positions calculated to yield a
 	 * paragraph of a given width.
@@ -312,7 +375,7 @@ namespace msXpS
 	 * \param width width in characters of the stanza
 	 * \return a new string with the stanzified text
 	 */
-	QString 
+	QString
 		stanzifyParagraphs(const QString &text, int width)
 		{
 			QString result;
@@ -366,6 +429,56 @@ namespace msXpS
 	}
 
 
+	//! Return the delta corresponding to m/z \p value and \p ppm part-per-million
+	double
+		ppm(double value, double ppm)
+		{
+
+			return (value * ppm) / 1000000;
+		}
+
+
+	//! Return \p value incremented with the matching \p ppm part.
+	double
+		addPpm(double value, double ppm)
+		{
+
+			return value + (value * ppm) / 1000000;
+		}
+
+
+	//! Return \p value decremented with the matching \p ppm part.
+	double
+		removePpm(double value, double ppm)
+		{
+
+			return value - (value * ppm) / 1000000;
+		}
+
+
+	//! Return the delta corresponding to m/z \p value and resolution \p res
+	double
+		res(double value, double res)
+		{
+			return value / res;
+		}
+
+
+	//! Return \p value incremented with the matching \p res part.
+	double
+		addRes(double value, double res)
+		{
+			return value + (value /res);
+		}
+
+	//! Return \p value decremented with the matching \p res part.
+	double
+		removeRes(double value, double res)
+		{
+			return value - (value /res);
+		}
+
+
 	//! Craft a string representing the address location (for debugging).
 	QString
 		pointerAsString(void *pointer)
