@@ -1,37 +1,39 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
+
 #pragma once
 
 
@@ -41,6 +43,7 @@
 
 
 /////////////////////// Local includes
+#include <globals/globals.hpp>
 #include <libmass/Trace.hpp>
 #include <libmass/DataPoint.hpp>
 
@@ -68,86 +71,52 @@ namespace msXpSlibmass
 		may be construed as mass peaks.
 
 */
-	class MassSpectrum 
-		: 
+	class MassSpectrum
+		:
 			public Trace
 	{
 		friend class MassSpectrumJs;
 
 		private:
 
-		//! Index of the bin that was last modified.
-		/*!
-
-			This value has the same function as \c Trace::m_lastModifIdx but in cases
-			where binning is required.
-
-*/
-		int m_lastBinIdx = 0;
-
 		//! Retention time at which this mass spectrum was acquired.
 		double m_rt = 0.0;
 
 		//! Drift time at which this mass spectrum was acquired.
 		double m_dt = 0.0;
 
-		//! Greatest difference between two consecutive m/z values in the spectrum
-		/*!
-
-			This value is useful when performing alignment of the spectra of an
-			acquisition.
-
-*/
-		double m_greatestStep = qSNaN();
+		//! Type of the binning.
+		msXpS::BinningType m_binningType = msXpS::BinningType::BINNING_TYPE_NONE;
 
 		//! Number of bins in the spectrum.
-		/*! 
-
-			The number of bins is useful for spectral alignment.
-
-*/
 		int m_binCount = 0;
 
-		//! Tells if the spectrum can undergo a binning process
-		bool m_isBinnable = false;	
+		//! Index of the bin that was last modified.
+		int m_lastBinIdx = 0;
 
-		//! The smallest m/z value contained in the spectrum list being processed
-		/*!
+		double m_binSize = 0.005;
+		//<! The bin size for the integrations to MZ.
 
-			This value is useful when setting-up the binning framework in \c this
-			MassSpectrum.  It stores the smallest m/z value of a list of mass spectra.
+		int m_binSizeType = msXpS::MassToleranceType::MASS_TOLERANCE_NONE;
+		//<! The bin size type for the integrations to MZ.
 
-*/
-		double m_binMinMz = 0;
+		//! The smallest m/z value contained in the spectrum list being processed.
+		double m_minMz = std::numeric_limits<double>::max();
 
-		//! The greatest m/z value contained in the spectrum list being processed
-		/*!
+		//! The greatest m/z value contained in the spectrum list being processed.
+		double m_maxMz = std::numeric_limits<double>::min();
 
-			This value is useful when setting-up the binning framework in \c this
-			MassSpectrum.  It stores the greatest m/z value of a list of mass spectra.
-
-*/
-		double m_binMaxMz = 0;
+		//! Tells if the mass spectra should be aligned.
+		bool m_applyMzShift = false;
 
 		//! Value by which the mass spectra need to be shifted when aligned.
-		/*!
-
-			This value holds the m/z delta value by which this mass spectrum needs
-			to be shifted in order to align it properly.
-
-*/
 		double m_mzShift = 0;
 
 		public:
 
-		//! Maximum acceptable value of the bin size average.
-		static double m_maxAvgMzBinSize;
-
-		//! Maximum acceptable value of the bin size standard deviation.
-		static double m_maxBinSizeStdDev;
-
 		MassSpectrum();
 		MassSpectrum(const MassSpectrum &other);
+		MassSpectrum(const Trace &other);
 		MassSpectrum(const QString &title);
 		virtual ~MassSpectrum();
 
@@ -155,12 +124,31 @@ namespace msXpSlibmass
 
 		void reset() override;
 
+
 		double rt() const;
 		double &rrt();
 
 		double dt() const;
 		double &rdt();
 
+		void setMinMz(double value);
+		void setMaxMz(double value);
+
+		void setBinningType(msXpS::BinningType binningType);
+		msXpS::BinningType binningType();
+
+		void setBinSize(double value);
+		double binSize() const;
+
+		void setBinSizeType(int type);
+		int binSizeType() const;
+
+		void applyMzShift(bool applyMzShift);
+		bool applyMzShift() const;
+
+		void setRemoveZeroValDataPoints(bool remove);
+		bool isRemoveZeroValDataPoints();
+
 		// The overriding functions below cannot shadow the base class ones.
 		using Trace::initialize;
 
@@ -177,17 +165,16 @@ namespace msXpSlibmass
 				double dt, double mzStart, double mzEnd);
 
 		int initialize(const QByteArray *keyByteArray,
-				const QByteArray *valByteArray, int compressionType, 
+				const QByteArray *valByteArray, int compressionType,
 				double keyStart = qSNaN(), double keyEnd = qSNaN()) override;
 
 		int initialize(const Trace &trace);
 
+		/************************** BINNING METHODS **************************/
 		bool setupBinnability(const QList<MassSpectrum *> &massSpectra);
-		double fillInBins(QList<double> *binList);
-		double binSizeAvg(const QList<double> &binList);
-		double binSizeStdDev(const QList<double> &binList);
-		bool isBinnable(const QList<double> &binList);
-		void setupBins(const QList<double> &binList);
+		bool setupBinnability(const MassSpectrum &massSpectrum);
+		int populateSpectrumWithArbitraryBins();
+		int populateSpectrumWithDataBasedBins(const MassSpectrum &massSpectrum);
 		int binIndex(double mz);
 
 		double determineMzShift(const MassSpectrum &massSpectrum);
@@ -195,38 +182,39 @@ namespace msXpSlibmass
 
 		// The overriding functions below cannot shadow the base class ones.
 		using Trace::combine;
+		using Trace::combineSlice;
 		using Trace::subtract;
 
+		int combine(const MassSpectrum &other,
+				double mzStart = -1, double mzEnd = -1);
+		int subtract(const MassSpectrum &other,
+				double mzStart = -1, double mzEnd = -1);
+
+		/***** The MonoT overloads are mono-thread versions of the function *****/
+		int combineMonoT(const QList<MassSpectrum *> &massSpectra);
+		int combine(const QList<MassSpectrum *> &massSpectra,
+				double mzStart = -1, double mzEnd = -1);
+		int combineSlice(const QList<MassSpectrum *> &massSpectra, int startIdx, int endIdx,
+				double mzStart = -1, double mzEnd = -1);
+
+		int subtractMonoT(const QList<MassSpectrum *> &massSpectra);
+		int subtract(const QList<MassSpectrum *> &massSpectra,
+				double mzStart = -1, double mzEnd = -1);
+		int subtractSlice(const QList<MassSpectrum *> &massSpectra, int startIdx, int endIdx,
+				double mzStart = -1, double mzEnd = -1);
+
+		int combine(const QList<double> &mzList, const QList<double> &iList,
+				double mzStart = -1, double mzEnd = -1);
+		int subtract(const QList<double> &mzList, const QList<double> &iList,
+				double mzStart = -1, double mzEnd = -1);
+
+		// The combinations with binning.
 		int combineBinned(const DataPoint &dataPoint);
 		int subtractBinned(const DataPoint &dataPoint);
 
-		int combine(const MassSpectrum &other);
-		int subtract(const MassSpectrum &other);
-		int combine(const MassSpectrum &other, double mzStart, double mzEnd);
-		int subtract(const MassSpectrum &other, double mzStart, double mzEnd);
-
-		int combine(const QList<MassSpectrum *> &massSpectra);
-		int subtract(const QList<MassSpectrum *> &massSpectra);
-		int combine(const QList<MassSpectrum *> &massSpectra,
-				double mzStart, double mzEnd);
-		int subtract(const QList<MassSpectrum *> &massSpectra,
-				double mzStart, double mzEnd);
-
-		int combine(const QList<double> &mzList, const QList<double> &iList);
-		int subtract(const QList<double> &mzList, const QList<double> &iList);
-		int combine(const QList<double> &mzList, const QList<double> &iList,
-				double mzStart, double mzEnd);
-		int subtract(const QList<double> &mzList, const QList<double> &iList,
-				double mzStart, double mzEnd);
-
 		double tic();
 		double tic(double mzStart, double mzEnd);
 		static double tic(const QList<MassSpectrum *> &massSpectra);
-
-		static void mzMinMax(const QList<MassSpectrum *> &massSpectra,
-				double *minMz, double *maxMz);
-
-		static bool areSpectraSameLength(const QList<MassSpectrum *> &massSpectra);
 	};
 
 } // namespace msXpSlibmass
