@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
  * END software license
  */
 
 
 
+
 #include <omp.h>
 #include <limits>
 
@@ -206,7 +207,7 @@ namespace msXpSlibmass
 
 
 	//! Set the decimal places member datum value
-	/*! 
+	/*!
 
 		Cannot be less than -1. If less than -1, the member value is set to -1.
 		*/
@@ -307,7 +308,7 @@ namespace msXpSlibmass
 					++dataPointCount;
 				}
 				else
-					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 						<< "Attention, data point at string list item index:" << iter << "is not valid.";
 			}
 
@@ -359,7 +360,7 @@ namespace msXpSlibmass
 		the initializing string, these will be sorted in \c this Trace
 		instance.
 
-		\param keyList list of keys 
+		\param keyList list of keys
 		\param valList list of values
 
 		\return the size of \c this Trace after initialization.
@@ -479,9 +480,9 @@ namespace msXpSlibmass
 	/*!
 
 		This is an overload of initialize() that uses byte arrays of keys and values
-		instead of lists. \p keyStart and \p keyEnd are optional.		
-		
-	 The data in the arrays are first decoded/uncompressed in vectors.
+		instead of lists. \p keyStart and \p keyEnd are optional.
+
+		The data in the arrays are first decoded/uncompressed in vectors.
 
 		Note that, if after decoding/uncompressing of the byte arrays, the obtained
 		data vectors do not have the same size, the program crashes.
@@ -500,7 +501,7 @@ namespace msXpSlibmass
 		*/
 	int
 		Trace::initialize(const QByteArray *keyByteArray,
-				const QByteArray *valByteArray, int compressionType, 
+				const QByteArray *valByteArray, int compressionType,
 				double keyStart, double keyEnd)
 		{
 			if(keyByteArray == Q_NULLPTR || valByteArray == Q_NULLPTR)
@@ -576,6 +577,36 @@ namespace msXpSlibmass
 		}
 
 
+	//! Remove all DataPoint instances having value equal to zero.
+	/*!
+
+		return the number of removed DataPoint instances.
+
+*/
+	int
+		Trace::removeZeroValDataPoints()
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Removing from this Trace all the DataPoint instances that have a value equal to 0.";
+
+			int removedCount = 0;
+
+			int iter = size() -1;
+			while(iter >= 0)
+			{
+				if(at(iter)->m_val == 0)
+				{
+					delete takeAt(iter);
+					++removedCount;
+				}
+
+				--iter;
+			}
+
+			return removedCount;
+		}
+
+
 	//! Compute the sum of all the values of \c this Trace
 	/*!
 
@@ -641,12 +672,22 @@ namespace msXpSlibmass
 
 			double valSum = 0;
 
-			for(int iter = 0; iter < size(); ++iter)
+#pragma omp parallel
 			{
-				double key = at(iter)->m_key;
+				// int maxNumbThreads = omp_get_max_threads();
+				// omp_set_num_threads(10);
+				// int numbThreads = omp_get_num_threads();
+				// qDebug() << __FILE__ << __LINE__
+				// << "Number of threads: " << numbThreads << "over" << maxNumbThreads;
 
-				if(key >= keyStart && key <= keyEnd)
-					valSum += at(iter)->m_val;
+#pragma omp for reduction(+ : valSum)
+				for(int iter = 0; iter < size(); ++iter)
+				{
+					double key = at(iter)->m_key;
+
+					if(key >= keyStart && key <= keyEnd)
+						valSum += at(iter)->m_val;
+				}
 			}
 
 			return valSum;
@@ -680,9 +721,10 @@ namespace msXpSlibmass
 
 		\param dataPoint the DataPoint instance to be combined to \c this Trace
 
-		\return 1 if a new DataPoint instance was inserted in \c this Trace;
-		0 if a preexisting DataPoint instance was found having the same key as
-		that in \p dataPoint (the value was thus incremented).
+		\return 1 if a new DataPoint instance was inserted in \c this Trace; 0 if a
+		preexisting DataPoint instance was found having the same key as that in \p
+		dataPoint (the value was thus incremented), -1 if nothing was performed
+		because the value of the dataPoint was 0.
 
 */
 	int
@@ -709,6 +751,10 @@ namespace msXpSlibmass
 			// index such that the next call can make use of it to accelerate the
 			// search process.
 
+			// If the value is 0, then no reason to do anything
+			if(dataPoint.m_val == 0)
+				return -1;
+
 			double key = dataPoint.key();
 
 			// Check if decimals are specified.
@@ -834,14 +880,19 @@ namespace msXpSlibmass
 
 		\param dataPoint the DataPoint instance to be subtracted from \c this Trace
 
-		\return 1 if a new DataPoint instance was inserted in \c this Trace;
-		0 if a preexisting DataPoint instance was found having the same key as
-		that in \p dataPoint (the value was thus incremented).
+		\return 1 if a new DataPoint instance was inserted in \c this Trace; 0 if a
+		preexisting DataPoint instance was found having the same key as that in \p
+		dataPoint (the value was thus incremented), -1 if nothing was performed
+		because the value of the dataPoint was 0..
 
 */
 	int
 		Trace::subtract(const DataPoint &dataPoint)
 		{
+			// If the value is 0, then no reason to do anything
+			if(dataPoint.m_val == 0)
+				return -1;
+
 			DataPoint localDataPoint(dataPoint);
 
 			localDataPoint.rval() *= -1;
@@ -871,7 +922,15 @@ namespace msXpSlibmass
 			int iter = 0;
 
 			for(iter = 0; iter < trace.size(); ++iter)
-				combine(*trace.at(iter));
+			{
+				// If the value is 0, then no reason to do anything
+				DataPoint *p_dp = trace.at(iter);
+
+				if(p_dp->m_val == 0)
+					continue;
+
+				combine(*p_dp);
+			}
 
 			// Return the number of combined data points.
 			return iter;
@@ -894,7 +953,15 @@ namespace msXpSlibmass
 			int iter = 0;
 
 			for(iter = 0; iter < trace.size(); ++iter)
-				subtract(*trace.at(iter));
+			{
+				// If the value is 0, then no reason to do anything
+				DataPoint *p_dp = trace.at(iter);
+
+				if(p_dp->m_val == 0)
+					continue;
+
+				subtract(*p_dp);
+			}
 
 			// Return the number of subtracted data points.
 			return iter;
@@ -930,6 +997,10 @@ namespace msXpSlibmass
 			{
 				DataPoint *p_dp = trace.at(iter);
 
+				// If the value is 0, then no reason to do anything
+				if(p_dp->m_val == 0)
+					continue;
+
 				if(p_dp->m_key >= keyStart && p_dp->m_key <= keyEnd)
 				{
 					combine(*p_dp);
@@ -972,6 +1043,10 @@ namespace msXpSlibmass
 			{
 				DataPoint *p_dp = trace.at(iter);
 
+				// If the value is 0, then no reason to do anything
+				if(p_dp->m_val == 0)
+					continue;
+
 				if(p_dp->m_key >= keyStart && p_dp->m_key <= keyEnd)
 				{
 					subtract(*p_dp);
@@ -992,7 +1067,7 @@ namespace msXpSlibmass
 		}
 
 
-	//! Combine the Trace instances in the \p traces list into \c this Trace
+	//! Combine the Trace instances in the \p traces list into \c this Trace (serial version)
 	/*!
 
 		This function might be called recursively with multiple different
@@ -1003,7 +1078,72 @@ namespace msXpSlibmass
 		- if the \p traces list is empty, return 0
 
 		- iterate in the \p traces list of Trace instances and for each
-		instance call combine(const Trace &). 
+		instance call combine(const Trace &).
+
+		\param traceList list of Trace instances to be combined to \c this
+		Trace
+
+		\return an int containing the number of combined Trace instances.
+
+		\sa combine(const Trace &)
+		*/
+	int
+		Trace::combineMonoT(const QList<Trace *> &traceList)
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+			//<< "";
+
+			// We receive a list of traces and we need to combine all the
+			// spectra thereof in this trace.
+
+			int traceListSize = traceList.size();
+
+			if(!traceListSize)
+				return 0;
+
+			// count is the number of traces that were effectively combined.
+			int count = 0;
+
+			for(int iter = 0; iter < traceListSize; ++iter)
+			{
+				combine(*traceList.at(iter));
+
+				++count;
+
+#if 0
+				QString fileName = QString("/tmp/unranged-combined-trace-after-%1-combinations.txt")
+					.arg(iter + 1);
+				exportToFile(fileName);
+
+#endif
+			}
+			// qDebug() << __FILE__ << __LINE__
+			// << "Effectively combined " << count
+			// << "Trace instances in this Trace for a total of "
+			// << size() << "data points";
+
+#if 0
+			QString fileName = "/tmp/unranged-combined-trace-after-finishing-combinations.txt";
+			exportToFile(fileName);
+
+#endif
+
+			return count;
+		}
+
+
+	//! Combine the Trace instances in the \p traces list into \c this Trace (parallel version)
+	/*!
+
+		This function might be called recursively with multiple different
+		lists of Trace instances.
+
+		The workings of this function are the following:
+
+		- if the \p traces list is empty, return 0
+
+		- iterate in the \p traces list of Trace instances and for each
+		instance call combine(const Trace &).
 
 		\param traces list of Trace instances to be combined to \c this
 		Trace
@@ -1013,67 +1153,179 @@ namespace msXpSlibmass
 		\sa combine(const Trace &)
 		*/
 	int
-		Trace::combine(const QList<Trace *> &traces)
+		Trace::combine(const QList<Trace *> &traceList)
 		{
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
-			//<< "";
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
 
-			// We receive a list of mass spectra and we need to combine all the
-			// spectra thereof in this trace.
+			int traceListSize = traceList.size();
 
-			if(traces.size() == 0)
+			if(!traceListSize)
 				return 0;
 
-			//qDebug() << __FILE__ << __LINE__
-			//<< "Starting combination of " << traces.size()
-			//<< "mass spectra into this trace";
+			// count is the number of traces that were effectively combined.
+			int count = 0;
 
-			Trace *trace = Q_NULLPTR;
-			int specCount = traces.size();
+			int maxThreads = omp_get_max_threads();
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "Max threads: " << maxThreads;
+
+			if(traceListSize < 10 * maxThreads)
+			{
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Number of traces to combine:" << traceListSize
+					<< "Too small a number of traces, not using the parallel code.";
+
+				for(int iter = 0; iter < traceListSize; ++iter)
+				{
+#if 0
+					exportToFile(QString("/tmp/trace-%1-for-combinations.txt")
+							.arg(iter + 1));
+#endif
+
+					combine(*traceList.at(iter));
+
+					++count;
+				}
+			}
+			else
+				/* of if(traceListSize < 10 * numbThreads) */
+			{
+				omp_set_num_threads(maxThreads);
+
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Number of traces sufficient to use parallel code."
+					<< "Set number of threads: " << maxThreads;
+
+				// Allocate as many combined traces as we are using threads.
+				QList<Trace *> combinedTraceList;
+
+				// Set the various slices of traceList so that each thread knows what
+				// slice to handle.
+				QList<QPair<int, int> *> indexRangeList;
+
+				double specPerThread = traceListSize / maxThreads;
+				// Integer part of the fractional result.
+				double intPart;
+				// We won't use fracPart, but anyway.
+				double fracPart = std::modf(specPerThread, &intPart);
+				Q_UNUSED(fracPart);
+
+				for(int iter = 0; iter < maxThreads; ++iter)
+				{
+					combinedTraceList.append(new Trace());
+
+					if(iter != maxThreads - 1)
+					{
+						if(!iter)
+							// We actually start from 0 = (iter * intPat), with iter = 0.
+							indexRangeList.append(new QPair<int, int>(iter * intPart, (iter + 1) * intPart));
+						else
+							// We need to increment by one index value the startIdx, otherwise
+							// we would combine twice the trace at index (iter * intPart)
+							indexRangeList.append(new QPair<int, int>((iter * intPart) + 1, (iter + 1) * intPart));
+					}
+					else
+					{
+
+						// Now is the time to complete to the fractional part, without
+						// bothering: simply set the lastIdx of QPair to
+						// traceListSize - 1.
+
+						indexRangeList.append(new QPair<int, int>(iter * intPart, traceListSize - 1));
+					}
+				}
+
+				// At this point we have setup all the slices to combine them in their
+				// own combined trace.
+
+#pragma omp parallel for shared(count)
+				for(int iter = 0; iter < maxThreads; ++iter)
+				{
+					Trace *p_combinedTrace = combinedTraceList.at(iter);
+					QPair<int, int> *p_indexRange = indexRangeList.at(iter);
+
+					int combineCount = p_combinedTrace->combineSlice(traceList,
+							p_indexRange->first, p_indexRange->second);
+
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+						<< QString::asprintf("Combined slice %d-%d\n", p_indexRange->first, p_indexRange->second);
+
+					count += combineCount;
+				}
+				// End of
+				// #pragma omp parallel
+
+				// At this point we need to conclude and combine into *this mass
+				// trace all the traces in the list of combine traces. Take
+				// advantage of the loop to also free each combine trace in
+				// turn.
+
+				while(combinedTraceList.size())
+				{
+					Trace *trace = combinedTraceList.takeFirst();
+					combine(*trace);
+					delete trace;
+				}
+
+				// At this point, we need to free all the QPair instances
+				while(indexRangeList.size())
+					delete indexRangeList.takeFirst();
+			}
+			// End of
+			// else /* of if(traceListSize < 10 * maxThreads) */
+
+			// The combination work is finished, however it has been performed.
+
+			if(m_removeZeroValDataPoints)
+			{
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Now removing all 0-m/z data points.";
+
+				removeZeroValDataPoints();
+			}
+#if 0
+			exportToFile(QString("/tmp/combined-trace.txt"));
+#endif
+
+			return count;
+		}
+
+
+	//! Combine the [\p startIdx--\p endIdx] slice of \p traceList into \c this Trace
+	/*!
+		\return the number of combined Trace instances.
+		*/
+	int
+		Trace::combineSlice(const QList<Trace *>&traceList, int startIdx, int endIdx)
+		{
+			if(traceList.isEmpty())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"List of traces to combine is empty."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			int firstIdx = std::min(startIdx, endIdx);
+			int lastIdx = std::max(startIdx, endIdx);
+
+			if(firstIdx < 0)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Index cannot be less than 0."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			if(lastIdx > traceList.size())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Out of bounds error."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
 
 			int count = 0;
 
-			for(int iter = 0; iter < specCount; ++iter)
+			for(int iter = firstIdx; iter < lastIdx; ++iter)
 			{
-				trace = traces.at(iter);
-
-				combine(*trace);
-
+				combine(*(traceList.at(iter)));
 				++count;
-
-#if 0
-				QString fileName = QString("/tmp/unranged-combined-spectrum-after-%1-combinations.txt")
-					.arg(iter + 1);
-
-				QFile file(fileName);
-				file.open(QIODevice::WriteOnly);
-				QTextStream fileStream(&file);
-				QString *text = asText();
-				fileStream << *text;
-				delete text;
-				fileStream.flush();
-				file.close();
-
-#endif
 			}
-			// qDebug() << __FILE__ << __LINE__
-			// << "Effectively combined " << combinedSpectra
-			// << "in this single spectrum for a total of "
-			// << size() << "data points";
-
-#if 0
-			QString fileName = "/tmp/unranged-combined-spectrum-after-finishing-combinations.txt";
-
-			QFile file(fileName);
-			file.open(QIODevice::WriteOnly);
-			QTextStream fileStream(&file);
-			QString *text = asText();
-			fileStream << *text;
-			delete text;
-			fileStream.flush();
-			file.close();
-
-#endif
 
 			return count;
 		}
@@ -1086,59 +1338,230 @@ namespace msXpSlibmass
 
 */
 	int
-		Trace::subtract(const QList<Trace *> &traces)
+		Trace::subtractMonoT(const QList<Trace *> &traceList)
 		{
 			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ ;
 
-			if(traces.size() == 0)
+			int traceListSize = traceList.size();
+
+			if(!traceListSize)
 				return 0;
 
-			Trace *trace = Q_NULLPTR;
-			int specCount = traces.size();
+			// count is the number of traces that were effectively combined.
+			int count = 0;
+
+			for(int iter = 0; iter < traceListSize; ++iter)
+			{
+				subtract(*traceList.at(iter));
+
+				++count;
+			}
+
+			// qDebug() << __FILE__ << __LINE__
+			// << "Effectively combined " << count
+			// << "Trace instances in this Trace for a total of "
+			// << size() << "data points";
+
+			return count;
+		}
+
+
+	//! Subtract the Trace instances in the \p traces list from \c this Trace (parallel version)
+	/*!
+
+		This function might be called recursively with multiple different
+		lists of Trace instances.
+
+		The workings of this function are the following:
+
+		- if the \p traces list is empty, return 0
+
+		- iterate in the \p traces list of Trace instances and for each
+		instance call subtract(const Trace &).
+
+		\param traces list of Trace instances to be subtractd to \c this
+		Trace
+
+		\return an int containing the number of subtractd Trace instances.
+
+		\sa subtract(const Trace &)
+		*/
+	int
+		Trace::subtract(const QList<Trace *> &traceList)
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
+
+			int traceListSize = traceList.size();
+
+			if(!traceListSize)
+				return 0;
+
+			// count is the number of traces that were effectively subtracted.
+			int count = 0;
+
+			int maxThreads = omp_get_max_threads();
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "Max threads: " << maxThreads;
+
+			if(traceListSize < 10 * maxThreads)
+			{
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Number of traces to subtract:" << traceListSize
+					<< "Too small a number of traces, not using the parallel code.";
+
+				for(int iter = 0; iter < traceListSize; ++iter)
+				{
+					Trace *p_trace = traceList.at(iter);
+
+#if 0
+					exportToFile(QString("/tmp/trace-%1-for-combinations.txt")
+							.arg(iter + 1));
+#endif
+
+					subtract(*p_trace);
+
+					++count;
+				}
+			}
+			else
+				/* of if(traceListSize < 10 * numbThreads) */
+			{
+				omp_set_num_threads(maxThreads);
+
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Number of traces sufficient to use parallel code."
+					<< "Set number of threads: " << maxThreads;
+
+				// Allocate as many subtractd traces as we are using threads.
+				QList<Trace *> subtractedTraceList;
+
+				// Set the various slices of traceList so that each thread knows what
+				// slice to handle.
+				QList<QPair<int, int> *> indexRangeList;
+
+				double specPerThread = traceListSize / maxThreads;
+				// Integer part of the fractional result.
+				double intPart;
+				// We won't use fracPart, but anyway.
+				double fracPart = std::modf(specPerThread, &intPart);
+				Q_UNUSED(fracPart);
+
+				for(int iter = 0; iter < maxThreads; ++iter)
+				{
+					subtractedTraceList.append(new Trace());
+
+					if(iter != maxThreads - 1)
+					{
+						if(!iter)
+							// We actually start from 0 = (iter * intPat), with iter = 0.
+							indexRangeList.append(new QPair<int, int>(iter * intPart, (iter + 1) * intPart));
+						else
+							// We need to increment by one index value the startIdx, otherwise
+							// we would subtract twice the trace at index (iter * intPart)
+							indexRangeList.append(new QPair<int, int>((iter * intPart) + 1, (iter + 1) * intPart));
+					}
+					else
+					{
+
+						// Now is the time to complete to the fractional part, without
+						// bothering: simply set the lastIdx of QPair to
+						// traceListSize - 1.
+
+						indexRangeList.append(new QPair<int, int>(iter * intPart, traceListSize - 1));
+					}
+				}
+
+				// At this point we have setup all the slices to subtract them in their
+				// own subtractd trace.
+
+#pragma omp parallel for shared(count)
+				for(int iter = 0; iter < maxThreads; ++iter)
+				{
+					Trace *p_subtractedTrace = subtractedTraceList.at(iter);
+					QPair<int, int> *p_indexRange = indexRangeList.at(iter);
+
+					int subtractCount = p_subtractedTrace->subtractSlice(traceList,
+							p_indexRange->first, p_indexRange->second);
+
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+						<< QString::asprintf("Subtractd slice %d-%d\n", p_indexRange->first, p_indexRange->second);
+
+					count += subtractCount;
+				}
+				// End of
+				// #pragma omp parallel
+
+				// At this point we need to conclude and subtract into *this mass
+				// trace all the traces in the list of subtract traces. Take
+				// advantage of the loop to also free each subtract trace in
+				// turn.
+
+				while(subtractedTraceList.size())
+				{
+					Trace *trace = subtractedTraceList.takeFirst();
+					subtract(*trace);
+					delete trace;
+				}
+
+				// At this point, we need to free all the QPair instances
+				while(indexRangeList.size())
+					delete indexRangeList.takeFirst();
+			}
+			// End of
+			// else /* of if(traceListSize < 10 * maxThreads) */
+
+			// The combination work is finished, however it has been performed.
+
+			if(m_removeZeroValDataPoints)
+			{
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Now removing all 0-m/z data points.";
+
+				removeZeroValDataPoints();
+			}
+#if 0
+			exportToFile(QString("/tmp/subtractd-trace.txt"));
+#endif
+
+			return count;
+		}
+
+
+	//! Subtract the [\p startIdx--\p endIdx] slice of \p traceList from \c this Trace
+	/*!
+		\return the number of subtractd Trace instances.
+		*/
+	int
+		Trace::subtractSlice(const QList<Trace *>&traceList, int startIdx, int endIdx)
+		{
+			if(traceList.isEmpty())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"List of traces to subtract is empty."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			int firstIdx = std::min(startIdx, endIdx);
+			int lastIdx = std::max(startIdx, endIdx);
+
+			if(firstIdx < 0)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Index cannot be less than 0."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			if(lastIdx > traceList.size())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Out of bounds error."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
 
 			int count = 0;
 
-			for(int iter = 0; iter < specCount; ++iter)
+			for(int iter = firstIdx; iter < lastIdx; ++iter)
 			{
-				trace = traces.at(iter);
-
-				subtract(*trace);
-
+				subtract(*(traceList.at(iter)));
 				++count;
-
-#if 0
-				QString fileName = QString("/tmp/unranged-combined-spectrum-after-%1-combinations.txt")
-					.arg(iter + 1);
-
-				QFile file(fileName);
-				file.open(QIODevice::WriteOnly);
-				QTextStream fileStream(&file);
-				QString *text = asText();
-				fileStream << *text;
-				delete text;
-				fileStream.flush();
-				file.close();
-
-#endif
 			}
-			// qDebug() << __FILE__ << __LINE__
-			// << "Effectively combined " << combinedSpectra
-			// << "in this single spectrum for a total of "
-			// << size() << "data points";
-
-#if 0
-			QString fileName = "/tmp/unranged-combined-spectrum-after-finishing-combinations.txt";
-
-			QFile file(fileName);
-			file.open(QIODevice::WriteOnly);
-			QTextStream fileStream(&file);
-			QString *text = asText();
-			fileStream << *text;
-			delete text;
-			fileStream.flush();
-			file.close();
-
-#endif
 
 			return count;
 		}
@@ -1170,7 +1593,7 @@ namespace msXpSlibmass
 	int
 		Trace::combine(const QList<Trace *> &traces, double keyStart, double keyEnd)
 		{
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 			//<< "";
 
 			// We receive a list of mass spectra, that is a list of (list of
@@ -1195,22 +1618,13 @@ namespace msXpSlibmass
 			}
 
 			// qDebug() << __FILE__ << __LINE__
-			// << "Effectively combined " << combinedSpectra
-			// << "in this single spectrum for a total of "
+			// << "Effectively combined " << count
+			// << "in this Trace for a total of "
 			// << size() << "data points";
 
 #if 0
-			QString fileName = "/tmp/mzranged-combined-spectrum-after-finishing-combinations.txt";
-
-			QFile file(fileName);
-			file.open(QIODevice::WriteOnly);
-			QTextStream fileStream(&file);
-			QString *text = asText();
-			fileStream << *text;
-			delete text;
-			fileStream.flush();
-			file.close();
-
+			QString fileName = "/tmp/mzranged-combined-trace-after-finishing-combinations.txt";
+			exportToFile(fileName);
 #endif
 
 			return count;
@@ -1245,17 +1659,8 @@ namespace msXpSlibmass
 			}
 
 #if 0
-			QString fileName = "/tmp/mzranged-combined-spectrum-after-finishing-combinations.txt";
-
-			QFile file(fileName);
-			file.open(QIODevice::WriteOnly);
-			QTextStream fileStream(&file);
-			QString *text = asText();
-			fileStream << *text;
-			delete text;
-			fileStream.flush();
-			file.close();
-
+			QString fileName = "/tmp/mzranged-combined-trace-after-finishing-combinations.txt";
+			exportToFile(fileName);
 #endif
 
 			return count;
@@ -1603,8 +2008,11 @@ namespace msXpSlibmass
 
 			int dataPointCount = keyLista.size();
 
+			// Even if the spectrum is empty, we should return an empty string.
+			QString *p_text = new QString();
+
 			if(dataPointCount == 0)
-				return Q_NULLPTR;
+				return p_text;
 
 			int indexStart = (startIndex <= 0 ? 0 : startIndex);
 			if(indexStart >= dataPointCount - 1)
@@ -1614,21 +2022,19 @@ namespace msXpSlibmass
 			if(indexEnd >= dataPointCount - 1)
 				indexEnd = dataPointCount -1;
 
-			QString *text = new QString();
-
 			for(int iter = indexStart; iter <= indexEnd; ++iter)
 			{
-				text->append(QString("%1 %2\n")
+				p_text->append(QString("%1 %2\n")
 						.arg(keyLista.at(iter), 0, 'f', 10)
 						.arg(valLista.at(iter), 0, 'f', 10));
 			}
 
-			return text;
+			return p_text;
 		}
 
 
 	//! Export \c this Trace data to a file as a xy-formatted string.
-	bool 
+	bool
 		Trace::exportToFile(const QString &fileName)
 		{
 			if(fileName.isEmpty())
@@ -1652,5 +2058,4 @@ namespace msXpSlibmass
 		}
 
 
-
 } // namespace msXpSlibmass
