@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 ///////////////////////////// Qt includes
 
 ///////////////////////////// Local includes
@@ -90,6 +91,7 @@ namespace msXpSmineXpert
 			settings.setValue(QString("%1_windowState").arg(m_desc), saveState());
 			settings.setValue(QString("%1_splitterState").arg(m_desc),
 					mp_splitter->saveState());
+			settings.setValue(QString("%1_visible").arg(m_desc), isVisible());
 		}
 
 
@@ -105,11 +107,14 @@ namespace msXpSmineXpert
 					settings.value(QString("%1_windowState").arg(m_desc)).toByteArray());
 			mp_splitter->restoreState(
 					settings.value(QString("%1_splitterState").arg(m_desc)).toByteArray());
+
+			bool wasVisible = settings.value(QString("%1_visible").arg(m_desc)).toBool();
+			setVisible(wasVisible);
 		}
 
 
 	//! Set the name of this window.
-	void 
+	void
 		AbstractMultiPlotWnd::setName(QString name)
 		{
 			m_name = name;
@@ -117,7 +122,7 @@ namespace msXpSmineXpert
 
 
 	//! Get the name of this window.
-	QString 
+	QString
 		AbstractMultiPlotWnd::name()
 		{
 			return m_name;
@@ -125,7 +130,7 @@ namespace msXpSmineXpert
 
 
 	//! Set the description of this window.
-	void 
+	void
 		AbstractMultiPlotWnd::setDesc(QString desc)
 		{
 			m_desc = desc;
@@ -133,7 +138,7 @@ namespace msXpSmineXpert
 
 
 	//! Get the description of this window.
-	QString 
+	QString
 		AbstractMultiPlotWnd::desc()
 		{
 			return m_desc;
@@ -288,7 +293,7 @@ namespace msXpSmineXpert
 
 
 	//! Set the color to be used to configure the new plot widget to be created.
-	void 
+	void
 		AbstractMultiPlotWnd::setWishedColor(QColor color)
 		{
 			m_wishedColor = color;
@@ -349,7 +354,7 @@ namespace msXpSmineXpert
 		and triggers QApplication::processEvents() to force display of the current
 		progress as indicated by \p currentValue.
 
-		\param msg string containing the initial feedback message. 
+		\param msg string containing the initial feedback message.
 
 		\param startValue initial value of the progress range.
 
@@ -364,15 +369,15 @@ namespace msXpSmineXpert
 		{
 			//qDebug() << __FILE__ << __LINE__
 			//<< "Entering" << __FUNCTION__
-			//<< "with message: " << msg 
-			//<< "and startValue:" << startValue 
-			//<< "and endValue:" << endValue 
+			//<< "with message: " << msg
+			//<< "and startValue:" << startValue
+			//<< "and endValue:" << endValue
 			//<< "and currentValue:" << currentValue
 			//<< "and log type:" << logType;
 
 			if(logType & LogType::LOG_TO_CONSOLE)
 			{
-				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 				//<< "log type to console";
 				MainWindow *mainWindow = static_cast<MainWindow *>(parent());
 				mainWindow->logConsoleMessage(msg);
@@ -429,7 +434,7 @@ namespace msXpSmineXpert
 		{
 			//qDebug() << __FILE__ << __LINE__
 			//<< "Entering" << __FUNCTION__
-			//<< "with message: " << msg 
+			//<< "with message: " << msg
 			//<< "and currentValue:" << currentValue
 			//<< "and log type:" << logType;
 
@@ -741,4 +746,45 @@ namespace msXpSmineXpert
 			emit cancelOperationSignal();
 		}
 
+
+	void
+		AbstractMultiPlotWnd::setMzIntegrationParamsWnd(MzIntegrationParamsWnd *wnd)
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "with params:" << wnd->paramsAsText();
+
+			mp_mzIntegrationParamsWnd = wnd;
+		}
+
+	void
+		AbstractMultiPlotWnd::updateMzIntegrationParams(MzIntegrationParams params, const QColor &color)
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< QString::asprintf("\twith params:\n%s\n", params.asText().toLatin1().data())
+				//<< "and color:" << color;
+
+			mp_mzIntegrationParamsWnd->updateMzIntegrationParams(params, color);
+		}
+
+
+	void
+		AbstractMultiPlotWnd::mzIntegrationParamsChanged(MzIntegrationParams params)
+		{
+			// This function answers to the corresponding signal emitted from the
+			// MzIntegrationParamsWnd.
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "with params:" << params.asText();
+
+			// The last focused plot widget needs to "read" the new params and set
+			// them to its history.
+
+			if(mp_lastFocusedPlotWidget != Q_NULLPTR)
+			{
+				dynamic_cast<TicChromPlotWidget *>(mp_lastFocusedPlotWidget)->setMzIntegrationParams(params);
+			}
+
+		}
+
+
 } // namespace msXpSmineXpert
