@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #pragma once
 
 ///////////////////////////// Qt includes
@@ -52,10 +53,14 @@
 #include <minexpert/gui/AbstractPlotWidget.hpp>
 #include <minexpert/nongui/MassSpecDataSet.hpp>
 
+
 namespace msXpSmineXpert
 {
 
 	class AnalysisPreferences;
+	class MzIntegrationParams;
+	class MzIntegrationParamsWnd;
+
 
 	//! The AbstractMultiPlotWnd class provides a base class for plot windows.
 	/*!
@@ -81,7 +86,7 @@ namespace msXpSmineXpert
 		the multigraph qualificator). The bottom part of the window has as many
 		plots are necessary. These plot widgets can only contain a single graph
 		(thus the monograph qualificator). Each monograph plot widget has its graph
-		replicated in the multigraph plot widget. 
+		replicated in the multigraph plot widget.
 
 		This dual plotting system allows for two views of the plots: one uncluttered
 		monograph view for each plot (bottom, as many plot widgets as necessary) and
@@ -160,6 +165,8 @@ namespace msXpSmineXpert
 		//! Pointer to the AnalysisPreferences instance.
 		AnalysisPreferences *mp_analysisPreferences = Q_NULLPTR;
 
+		//! Widget in which to set the MZ integration parameters
+		MzIntegrationParamsWnd *mp_mzIntegrationParamsWnd = Q_NULLPTR;
 
 		public:
 
@@ -189,6 +196,10 @@ namespace msXpSmineXpert
 		virtual void showMultiGraph(AbstractPlotWidget *widget);
 		virtual void hideMultiGraph(AbstractPlotWidget *widget);
 
+		void setMzIntegrationParamsWnd(MzIntegrationParamsWnd *wnd);
+		void updateMzIntegrationParams(MzIntegrationParams params, const QColor &color = QColor());
+		void mzIntegrationParamsChanged(MzIntegrationParams params);
+
 		void removeMultiGraph(AbstractPlotWidget *widget);
 
 		bool delistPlotWidget(AbstractPlotWidget *plotWidget);
