@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 ///////////////////////////// Qt include
 
 
@@ -40,7 +41,7 @@
 #include <minexpert/gui/AbstractMultiPlotWnd.hpp>
 #include <minexpert/nongui/MassSpecSqlite3Handler.hpp>
 #include <minexpert/nongui/SqlMassDataSlicer.hpp>
-#include <minexpert/gui/SqlMassDataSlicerDlg.hpp>
+#include <minexpert/gui/SqlMassDataSlicerWnd.hpp>
 #include <minexpert/nongui/globals.hpp>
 #include <minexpert/gui/MainWindow.hpp>
 #include <minexpert/gui/SaveToGraphicsFileDlg.hpp>
@@ -50,6 +51,21 @@
 namespace msXpSmineXpert
 {
 
+	/*/js/ Class: AbstractPlotWidget
+	 *
+	 * <comment>This class is not exported to the JavaScript environment, but it
+	 * serves as the base class for all the plot widget subclasses. Its methods
+	 * are thus available to all the objects of the various subclasses, like
+	 * MassSpecPlotWidget, DriftSpecPlotWidget, ColorMapPlotWidget and
+	 * TicChromPlotWidget, collectively referred to as <PlotWidget> below.
+	 *
+	 * When using objects of the various classes above, the object name will thus
+	 * be, for example, ticChromPlotWidget0 or driftSpecPlotWidget1. These objects
+	 * are made available to the scripting environment in the tree view on the
+	 * left part of the scripting window. See the user manual for
+	 * details.</comment>
+	 */
+
 
 	//! Construct a AbstractPlotWidget instance.
 	/*!
@@ -73,24 +89,57 @@ namespace msXpSmineXpert
 		than one graph.
 
 */
-	AbstractPlotWidget::AbstractPlotWidget(QWidget *parent, QWidget *parentWnd,
+	AbstractPlotWidget::AbstractPlotWidget(QWidget *parent,
 			const QString &name, const QString &desc,
 			MassSpecDataSet *massSpecDataSet, const QString &fileName,
 			bool isMultiGraph)
-		: 
+		:
 			QCustomPlot{parent},
-		mp_parentWnd{parentWnd},
+		mp_parentWnd{parent},
 		m_name{name},
 		m_desc{desc},
 		mp_massSpecDataSet{massSpecDataSet},
 		m_fileName{fileName},
 		m_isMultiGraph{isMultiGraph}
 	{
+		//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+		//<< "Creating new plot widget:" << this
+		//<< "with parent window:" << parent;
+
+		if(parent == Q_NULLPTR)
+			qFatal("Fatal error at %s@%d -- %s(). "
+					"Not possible that parent pointer is nullptr."
+					"Program aborted.",
+					__FILE__, __LINE__, __FUNCTION__);
+
 		if(!setupWidget())
 			qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 		setObjectName("plotWidget");
 
+		if(fileName.isEmpty())
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "The file name passed to the plot widget constructor is empty";
+		else
+		{
+			QFileInfo fileInfo(fileName);
+
+			if(!fileInfo.exists())
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "The file name passed as parameter does not exist:" << fileName;
+			else
+			{
+				if(!fileInfo.isAbsolute())
+				{
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+						<< "The file name passed as parameter is not absolute";
+				}
+				else
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+						<< "The file name passed as parameter:" << fileName;
+			}
+		}
+
 		show();
 	}
 
@@ -112,11 +161,32 @@ namespace msXpSmineXpert
 
 		for(int iter = 0; iter < m_yAxisRangeHistory.size(); ++iter)
 			delete m_yAxisRangeHistory.at(iter);
+
+		// Inform the parent window that this is no longer be the last focused widget:
+
+		AbstractMultiPlotWnd *parentWnd = static_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+
+		if(parentWnd == Q_NULLPTR)
+			qFatal("Fatal error at %s@%d -- %s(). "
+					"Not possible that parent pointer is nullptr."
+					"Program aborted.",
+					__FILE__, __LINE__, __FUNCTION__);
+
+
+		if(parentWnd->mp_lastFocusedPlotWidget == this)
+			parentWnd->mp_lastFocusedPlotWidget = Q_NULLPTR;
 	}
 
 
+	QWidget *
+		AbstractPlotWidget::parentWnd()
+		{
+			return mp_parentWnd;
+		}
+
+
 	//! Set the name of the plot widget.
-	void 
+	void
 		AbstractPlotWidget::setName(QString name)
 		{
 			m_name = name;
@@ -124,14 +194,14 @@ namespace msXpSmineXpert
 
 
 	//! Get the name of the plot widget.
-	QString 
+	QString
 		AbstractPlotWidget::name() const
 		{
 			return m_name;
 		}
 
 	//! Set the description of the plot widget.
-	void 
+	void
 		AbstractPlotWidget::setDesc(QString desc)
 		{
 			m_desc = desc;
@@ -139,7 +209,7 @@ namespace msXpSmineXpert
 
 
 	//! Get the description of the plot widget.
-	QString 
+	QString
 		AbstractPlotWidget::desc() const
 		{
 			return m_desc;
@@ -268,23 +338,21 @@ namespace msXpSmineXpert
 
 			mpa_contextMenu = new QMenu(this);
 
-			QAction *action = Q_NULLPTR;
-
 			if(mp_colorMap == Q_NULLPTR)
 			{
 				QMenu *exportMenu = mpa_contextMenu->addMenu("Export");
 
-				action = exportMenu->addAction("&Data");
-				connect(action, &QAction::triggered, this, &AbstractPlotWidget::exportData);
+				mpa_exportDataAction = exportMenu->addAction("&Data");
+				connect(mpa_exportDataAction, &QAction::triggered, this, &AbstractPlotWidget::exportData);
 
-				action = exportMenu->addAction("&Plot as xy file");
-				connect(action, &QAction::triggered, this, &AbstractPlotWidget::exportPlot);
+				mpa_plotXyFileAction = exportMenu->addAction("&Plot as xy file");
+				connect(mpa_plotXyFileAction, &QAction::triggered, this, &AbstractPlotWidget::exportPlot);
 
 				mpa_contextMenu->addSeparator();
 			}
 
-			action = mpa_contextMenu->addAction("&Save plot to graphics file");
-			connect(action, &QAction::triggered, this, &AbstractPlotWidget::savePlotToGraphicsFile);
+			mpa_savePlotGraphics = mpa_contextMenu->addAction("&Save plot to graphics file");
+			connect(mpa_savePlotGraphics, &QAction::triggered, this, &AbstractPlotWidget::savePlotToGraphicsFile);
 
 			return mpa_contextMenu;
 		}
@@ -310,6 +378,12 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.showHistory()
+	 *
+	 * Shows a tooltip with the History of this plot widget.
+	 */
+
 	//! Craft a tooltip and display it with text describing the History of \c this AbstractPlotWidget instance.
 	/*!
 
@@ -335,6 +409,12 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.historyAsText()
+	 *
+	 * Return a textual representation of the history of this plot widget.
+	 */
+
 	//! Craft a textual representation of \c this AbstractPlotWidget History.
 	/*!
 
@@ -354,6 +434,99 @@ namespace msXpSmineXpert
 		}
 
 
+	//! Find a minimal integration range starting at an existing data point
+	/*!
+
+		If the user clicks onto a plot at a location that is not a true data point,
+		get a data range that begins at the preceding data point and that ends at
+		the clicked location point.
+
+*/
+	bool
+		AbstractPlotWidget::findIntegrationLowerRangeForKey(double key, QCPRange *p_range)
+		{
+			if(p_range == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Pointer cannot be nullptr."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			// Given a key double value, we want to know what is the range that will
+			// frame correctly the key double value if that key value is not exactly
+			// the one of a point of the trace.
+
+			// First of all get the keys of the graph.
+
+			QCPGraph *theGraph = graph();
+
+			if(theGraph == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Programming error."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+
+			QCPDataMap *mapData = theGraph->data();
+
+			QList<double> keys = mapData->keys();
+
+			// There is one check to be done: the user might erroneously set the mouse
+			// cursor beyond the last point of the graph. If that is the case, then
+			// upper key needs to be that very point. All we need to do is return the
+			// lower key, that is the pre-last key of the keys list. No need to
+			// iterate in the keys list.
+
+			if(key > keys.last() && keys.size() > 1)
+			{
+				// No need to search for the key in the keys, just get the lower key
+				// immediately.
+				p_range->lower = keys.at(keys.size() - 2);
+				p_range->upper = keys.at(keys.size() - 1);
+
+				return true;
+			}
+
+			// Likewise, if the cursor is set left of the first plot point, then that
+			// will be the lower range point. All we need is to provide the upper
+			// range point as the second point of the plot.
+
+			if(key < keys.first())
+			{
+				if(keys.size() < 1)
+					return false;
+
+				p_range->lower = keys.at(0);
+				p_range->upper = keys.at(1);
+
+				return true;
+			}
+
+			// Finally the generic case where the user point to any point *in* the
+			// graph
+
+			for(int iter = 0; iter < keys.size(); ++iter)
+			{
+				if(keys.at(iter) > key)
+				{
+					if(iter > 0)
+					{
+						p_range->lower = keys.at(iter - 1);
+						p_range->upper = key;
+						return true;
+					}
+					else
+					{
+						p_range->lower = keys.at(0);
+						p_range->upper = key;
+						return true;
+					}
+				}
+			}
+
+			return false;
+		}
+
+
 	//! Clear the history of the axis ranges.
 	/*!
 
@@ -458,8 +631,9 @@ namespace msXpSmineXpert
 			replot();
 		}
 
+
 	//! Insert a keyboard key code in the list of registered key.
-	/*! 
+	/*!
 
 		The \c m_regQtKeyCodeMap map of keyboard keys is a register that stores
 		key codes of keys that have a special meaning while working on the data
@@ -473,8 +647,15 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.showHelpSummary()
+	 *
+	 * Show a tooltip with a help string summarizing the various keyboard key
+	 * combinations.
+	 */
+
 	//! Craft a string explaining the various key combinations for the integrations.
-	void 
+	void
 		AbstractPlotWidget::showHelpSummary()
 		{
 			QString helpText;
@@ -490,7 +671,7 @@ namespace msXpSmineXpert
 
 			QSize widgetSize = size();
 			QPoint windowPos = mp_parentWnd->pos();
-			QPoint toolTipPos = windowPos + QPoint(widgetSize.width() / 2, widgetSize.height() / 2);
+			QPoint toolTipPos = windowPos + QPoint(widgetSize.width() / 4, widgetSize.height() / 4);
 			QToolTip::showText(toolTipPos, helpText, this, QRect());
 		}
 
@@ -560,6 +741,11 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.shouldDestroyPlotWidget()
+	 *
+	 * Start the destruction of this plot widget.
+	 */
 
 	//! Start the process of destroying a plot widget.
 	/*!
@@ -575,8 +761,7 @@ namespace msXpSmineXpert
 			// automatically destroy the corresponding, the matching multigraph
 			// QGraph. This signal is connected to a slot that will handle the removal
 			// of the matching multigraph graph.
-			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-				->removeMultiGraph(this);
+			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->removeMultiGraph(this);
 
 			// This abstract plot widget does not handle its destruction by
 			// itself, it delegates its distruction to the main window of the
@@ -586,42 +771,58 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.toggleMultiGraph()
+	 *
+	 * If the multi-graph plot corresponding to this plot widget is visible, make
+	 * it invisible. And vice versa.
+	 */
+
 	//! Toggle visibility of the corresponding graph in the multigraph plot widget.
 	void
 		AbstractPlotWidget::toggleMultiGraph()
 		{
-			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-				->toggleMultiGraph(this);
+			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->toggleMultiGraph(this);
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.showMultiGraph()
+	 *
+	 * Make the multi-graph plot corresponding to this plot widget visible.
+	 */
+
 	//! Make visible the corresponding graph in the multigraph plot widget.
 	void
 		AbstractPlotWidget::showMultiGraph()
 		{
-			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-				->showMultiGraph(this);
+			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->showMultiGraph(this);
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.hideMultiGraph()
+	 *
+	 * Make the multi-graph plot corresponding to this plot widget invisible.
+	 */
+
 	//! Make invisible the corresponding graph in the multigraph plot widget.
 	void
 		AbstractPlotWidget::hideMultiGraph()
 		{
-			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-				->hideMultiGraph(this);
+			static_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->hideMultiGraph(this);
 		}
 
 
-	//! Change the plotting and decoration color for \c this plot widget.
+	//! Set the plotting and decoration color for \c this plot widget.
 	/*!
 
 		This function is called when the user wants to change the color of the plot
 		widget's graph and decorations (tick labels, axis names).
 
 */
-	void 
-		AbstractPlotWidget::changeColor(const QColor& newColor)
+	void
+		AbstractPlotWidget::setPlottingColor(const QColor& newColor)
 		{
 			if(!newColor.isValid())
 				return;
@@ -636,6 +837,9 @@ namespace msXpSmineXpert
 				return;
 			}
 
+			// We want to have the color handy
+			m_plottingColor = newColor;
+
 			// First this single-graph widget
 			QPen pen;
 
@@ -646,25 +850,88 @@ namespace msXpSmineXpert
 				QCPAxis *xAxis = mp_colorMap->keyAxis();
 
 				pen = xAxis->basePen();
-				pen.setColor(newColor);
+				pen.setColor(m_plottingColor);
 
 				xAxis->setBasePen(pen);
-				xAxis->setLabelColor(newColor);
-				xAxis->setTickLabelColor(newColor);
+				xAxis->setLabelColor(m_plottingColor);
+				xAxis->setTickLabelColor(m_plottingColor);
 
 				QCPAxis *yAxis = mp_colorMap->valueAxis();
 				yAxis->setBasePen(pen);
-				yAxis->setLabelColor(newColor);
-				yAxis->setTickLabelColor(newColor);
+				yAxis->setLabelColor(m_plottingColor);
+				yAxis->setTickLabelColor(m_plottingColor);
 
 				replot();
 
 				// There is not multi-graph replica, just return.
 				return;
-			}	
+			}
 
 			pen = graph()->pen();
-			pen.setColor(newColor);
+			pen.setColor(m_plottingColor);
+			graph()->setPen(pen);
+
+			replot();
+
+			return;
+		}
+
+
+	//! Change the plotting and decoration color for \c this plot widget.
+	/*!
+
+		This function is called when the user wants to change the color of the plot
+		widget's graph and decorations (tick labels, axis names).
+
+*/
+	void
+		AbstractPlotWidget::changePlottingColor(const QColor& newColor)
+		{
+			if(!newColor.isValid())
+				return;
+
+			// This function does not do anything if *this is a multi-graph plot
+			// widget.
+			if(m_isMultiGraph)
+			{
+				qDebug() << __FILE__ << __LINE__
+					<< "Returning because *this plot widget is multi-graph.";
+
+				return;
+			}
+
+			// We want to have the color handy
+			m_plottingColor = newColor;
+
+			// First this single-graph widget
+			QPen pen;
+
+			// If this plot widget is hosting the color map, there is a specific
+			// treatement.
+			if(mp_colorMap != Q_NULLPTR)
+			{
+				QCPAxis *xAxis = mp_colorMap->keyAxis();
+
+				pen = xAxis->basePen();
+				pen.setColor(m_plottingColor);
+
+				xAxis->setBasePen(pen);
+				xAxis->setLabelColor(m_plottingColor);
+				xAxis->setTickLabelColor(m_plottingColor);
+
+				QCPAxis *yAxis = mp_colorMap->valueAxis();
+				yAxis->setBasePen(pen);
+				yAxis->setLabelColor(m_plottingColor);
+				yAxis->setTickLabelColor(m_plottingColor);
+
+				replot();
+
+				// There is not multi-graph replica, just return.
+				return;
+			}
+
+			pen = graph()->pen();
+			pen.setColor(m_plottingColor);
 			graph()->setPen(pen);
 
 			replot();
@@ -676,10 +943,10 @@ namespace msXpSmineXpert
 			QCPGraph *multiGraph = parentWnd->multiGraph(this);
 
 			//qDebug() << __FILE__ << __LINE__
-			//<< "for this:" << this << "found replicaGraph: " << replicaGraph;
+			//<< "for this:" << this << "found multiGraph: " << multiGraph;
 
 			pen = multiGraph->pen();
-			pen.setColor(newColor);
+			pen.setColor(m_plottingColor);
 			multiGraph->setPen(pen);
 			multiGraph->parentPlot()->replot();
 
@@ -688,14 +955,21 @@ namespace msXpSmineXpert
 
 
 	//! Return the pen color of \c this plot widget graph.
-	QColor 
+	QColor
 		AbstractPlotWidget::color() const
 		{
-			QPen pen = graph()->pen();
-			return pen.color();
+			return m_plottingColor;
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.exportData()
+	 *
+	 * Open the data export configuration window. Exporting data allows exporting
+	 * only a subset of the whole data set. It can only be performed if the
+	 * original data were read from a SQLite3 database-formatted mass data file.
+	 */
+
 	//! Export data to one or more files.
 	/*!
 
@@ -705,7 +979,7 @@ namespace msXpSmineXpert
 		can be exported to new files according to a number of criteria defined by
 		the user.
 
-		\sa SqlMassDataSlicer <br> SqlMassDataSlicerDlg.
+		\sa SqlMassDataSlicer <br> SqlMassDataSlicerWnd.
 		*/
 	void
 		AbstractPlotWidget::exportData()
@@ -720,6 +994,11 @@ namespace msXpSmineXpert
 			// slice all these data in chunks of a given size (count is deduced) or
 			// b. export the data in a number of chunks (size is deduced).
 
+			// Note that the window in which the user configures the data export
+			// operation is owned by MainWindow, the program's main window. That
+			// window is available for any plot widget to use, and thus needs to be
+			// updated with the caller widget's data to be useful.
+
 			if(m_isMultiGraph || mp_colorMap != Q_NULLPTR)
 				return;
 
@@ -736,6 +1015,11 @@ namespace msXpSmineXpert
 				return;
 			}
 
+			// Get a pointer to the configuratin window
+
+			MainWindow *p_mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+			SqlMassDataSlicerWnd *p_massDataSlicerWnd = p_mainWindow->mp_sqlMassDataSlicerWnd;
+
 			QCPAxis *xAxis = graph()->keyAxis();
 			double start = xAxis->range().lower;
 			double end = xAxis->range().upper;
@@ -754,31 +1038,46 @@ namespace msXpSmineXpert
 			histItem->newIntegrationRange(arbitraryIntegrationType(), start, end);
 			localHistory.appendHistoryItem(histItem);
 
-			qDebug() << __FILE__ << __LINE__
-				<< "History right before slicing: " << localHistory.asText();
+			// We want to provide the full path name to the file
 
-			// We'll need the handler of the mass data.
-			MassSpecSqlite3Handler sqlHandler(m_fileName);
+			QFileInfo fileInfo(m_fileName);
 
-			// And now allocate the slicer itself.
-			SqlMassDataSlicer dataSlicer(&sqlHandler, localHistory, start, end);
+			// Sanity check
+			if(!fileInfo.exists())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Cannot be that the plot widget has no file name member data set correctly."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
 
-			SqlMassDataSlicerDlg dataSlicerDlg(static_cast<QWidget *>(this),
-					&dataSlicer, "mineXpert");
-			dataSlicerDlg.setSrcFileName(m_fileName);
+			if(!fileInfo.isAbsolute())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"The plot widget's member m_fileName must be an absolute file path name."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
 
-			// When the sql data handler will perform the export, it will emit
-			// progression signals. Make the connecions.
-			connect(&sqlHandler, &MassSpecSqlite3Handler::setupFeedbackSignal,
-					&dataSlicerDlg, &SqlMassDataSlicerDlg::setupFeedback);
-			connect(&sqlHandler, &MassSpecSqlite3Handler::updateFeedbackSignal,
-					&dataSlicerDlg, &SqlMassDataSlicerDlg::updateFeedback);
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "The plot widget's m_fileName:" << m_fileName;
 
-			dataSlicerDlg.exec();
+			p_massDataSlicerWnd->initialize(fileInfo.absoluteFilePath(),
+					start, end, localHistory);
+
+			p_massDataSlicerWnd->activate();
 		}
 
 
-	QString 
+	/*/js/
+	 * <PlotWidget>.asXyText()
+	 *
+	 * Return  a textual representation of \c this plot widget data.
+	 *
+	 * Only the data range currently displayed is processed.
+	 */
+
+	//! Return a textual representation of \c this plot widget.
+	/*!
+	 * Only the currently visible data range is processed.
+	 */
+	QString
 		AbstractPlotWidget::asXyText()
 		{
 			if(m_isMultiGraph)
@@ -826,6 +1125,16 @@ namespace msXpSmineXpert
 
 
 
+	/*/js/
+	 * <PlotWidget>.exportPlot()
+	 *
+	 * Open a file selection dialog window to select a file in which to export
+	 * this plot widget's data.
+	 *
+	 * Unlike the exportData() function, this function only exports the currently
+	 * displayed graph data to a comma-separated value text file.
+	 */
+
 	//! Export plot data to a text file as a simple plot.
 	/*!
 
@@ -873,6 +1182,17 @@ namespace msXpSmineXpert
 
 
 
+	/*/js/
+	 * <PlotWidget>.exportPlotToFile(fileName, rangeStart, rangeEnd)
+	 *
+	 * Export plot data to a text file as a simple plot. Limit the data range to
+	 * export using the numerical parameters.
+	 *
+	 * fileName: <String> holding the file name
+	 * rangeStart: <Number> holding the beginning of the range to be exported
+	 * rangeEnd <Number> holding the end of the range to be exported
+	 */
+
 	//! Export plot data to a text file as a simple plot.
 	/*!
 
@@ -983,11 +1303,17 @@ namespace msXpSmineXpert
 		}
 
 
-	bool 
+	/*/js/
+	 * <PlotWidget>.savePlotToGraphicsFile()
+	 *
+	 * Open the graphics export configuration dialog window.
+	 */
+
+	//! Open the graphics export configuration dialog window.
+	bool
 		AbstractPlotWidget::savePlotToGraphicsFile()
 		{
-			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
-				<< "There.";
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
 
 			SaveToGraphicsFileDlg dlg(this, this, "mineXpert");
 
@@ -998,17 +1324,38 @@ namespace msXpSmineXpert
 		}
 
 
-	bool 
-		AbstractPlotWidget::saveToPdfFile(const QString &fileName, 
-				bool noCosmeticPen, int width, int height, 
+	/*/js/
+	 * <PlotWidget>.saveToPdfFile(fileName, noCosmeticPen, width, height, pdfCreator, title)
+	 *
+	 * Save this plot widget as a PDF file graphics.
+	 *
+	 * fileName: <String> holding the file name
+	 * noCosmeticPen: <Boolean> that tells if pen optimizations should be performed
+	 * width: <Number> holding the size of the graphics file in pixels
+	 * height: <Number> holding the size of the graphics file in pixels
+	 * pdfCreator: <String> holding the name of the creator of the PDF file
+	 * title: <String> holding the title of the graphics file
+	 */
+
+	//! Save this plot widget as a PDF file graphics.
+	bool
+		AbstractPlotWidget::savePlotToPdfFile(const QString &fileName,
+				bool noCosmeticPen, int width, int height,
 				const QString &pdfCreator, const QString &title)
 		{
-			return savePdf(fileName, noCosmeticPen, width, height, 
+			return savePdf(fileName, noCosmeticPen, width, height,
 					pdfCreator, title);
 		}
 
 
-	QList<double> 
+	/*/js/
+	 * <PlotWidget>.keys()
+	 *
+	 * Return an <Array> of <Number> values representing the keys of the data plotted.
+	 */
+
+	//! Return a list of double values representing the keys of the data plotted.
+	QList<double>
 		AbstractPlotWidget::keys()
 		{
 			QList<double> keyList;
@@ -1030,6 +1377,13 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.values()
+	 *
+	 * Return an <Array> of <Number> values representing the values of the data plotted.
+	 */
+
+	//! Return a list of double values representing the values of the data plotted.
 	QList<double>
 		AbstractPlotWidget::values()
 		{
@@ -1059,6 +1413,13 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.trace()
+	 *
+	 * Return a Trace object initialized with the data in this plot widget.
+	 */
+
+	//! Return a Trace instance initialized with the data in this plot widget
 	msXpSlibmass::Trace
 		AbstractPlotWidget::trace()
 		{
@@ -1076,7 +1437,7 @@ namespace msXpSmineXpert
 
 			QMapIterator<double, QCPData>i(*p_data);
 
-			while (i.hasNext()) 
+			while (i.hasNext())
 			{
 				i.next();
 
@@ -1109,8 +1470,17 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.replotWithAxisRangeX(lower, upper)
+	 *
+	 * Replot this plot widget with new key (X-axis) data range.
+	 *
+	 * lower: <Number> holding the start value of the range
+	 * upper: <Number> holding the end value of the range
+	 */
+
 	//! Trigger a graph replot action with new x-axis range.
-	void 
+	void
 		AbstractPlotWidget::replotWithAxisRangeX(double lower, double upper)
 		{
 			xAxis->setRange(lower, upper);
@@ -1118,8 +1488,17 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.replotWithAxisRangeY(lower, upper)
+	 *
+	 * Replot this plot widget with new value (Y-axis) data range.
+	 *
+	 * lower: <Number> holding the start value of the range
+	 * upper: <Number> holding the end value of the range
+	 */
+
 	//! Trigger a graph replot action with new y-axis range.
-	void 
+	void
 		AbstractPlotWidget::replotWithAxisRangeY(double lower, double upper)
 		{
 			yAxis->setRange(lower, upper);
@@ -1495,7 +1874,7 @@ namespace msXpSmineXpert
 		}
 
 	//! Rescale the x/y axes as a result of user interaction.
-	/*! 
+	/*!
 
 		Depending on the keyboard key that is pressed, axis rescaling can be carried
 		over in different ways:
@@ -1706,6 +2085,14 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * <PlotWidget>.getYatX(x)
+	 *
+	 * get the value at key x.
+	 *
+	 * x: <Number> holding the X-axis key for which the value is returned.
+	 */
+
 	//! Get the value corresponding to key \p k.
 	/*!
 
@@ -1717,7 +2104,7 @@ namespace msXpSmineXpert
 		\return double value containing the value corresponding to key \p k.
 
 */
-	double 
+	double
 		AbstractPlotWidget::getYatX(double x)
 		{
 			return getYatX(x, graph());
@@ -1741,21 +2128,6 @@ namespace msXpSmineXpert
 	void
 		AbstractPlotWidget::mouseMoveHandler(QMouseEvent *event)
 		{
-			// First off, set keyboard focus to this widget.
-			this->setFocus();
-
-			// Inform the parent window that this is the last focused widget:
-			AbstractMultiPlotWnd *parentWnd =
-				dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
-			if(parentWnd == Q_NULLPTR)
-				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
-
-			if(parentWnd->mp_lastFocusedPlotWidget != this)
-			{
-				parentWnd->mp_lastFocusedPlotWidget = dynamic_cast<QWidget *>(this);
-				parentWnd->emitRedrawPlotBackground(dynamic_cast<QWidget *>(this));
-			}
-
 			// Whatever happens, we want to store the plot coordinates of the current
 			// mouse cursor position (will be useful of the label feature).
 			QPointF mousePoint = event->localPos();
@@ -1856,7 +2228,7 @@ namespace msXpSmineXpert
 			{
 				m_yRangeMin = tempYmin;
 				m_yRangeMax = tempYmax;
-			}	
+			}
 
 			// We do not need the delta sign anymore:
 			m_yDelta = fabs(m_yDelta);
@@ -1896,8 +2268,7 @@ namespace msXpSmineXpert
 				// that signal will be caught up by all the receivers (plot widgets)
 				// that will update the range synchronously if the user asked for
 				// locking the X and/or Y axis range(s).
-				dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-					->emitReplotSignal(xAxis->range(), yAxis->range());
+				dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->emitReplotSignal(xAxis->range(), yAxis->range());
 			}
 
 			// Now deal with the BUTTON-SPECIFIC CODE.
@@ -2021,10 +2392,34 @@ namespace msXpSmineXpert
 		}
 
 
+	void
+		AbstractPlotWidget::setFocus()
+		{
+			QCustomPlot::setFocus();
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Setting focus to this plot widget:" << this
+			//<< "With parent window:" << mp_parentWnd;
+
+			// Inform the parent window that this is the last focused widget:
+			AbstractMultiPlotWnd *parentWnd = dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+			if(parentWnd == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+
+			if(parentWnd->mp_lastFocusedPlotWidget != this)
+			{
+				parentWnd->mp_lastFocusedPlotWidget = dynamic_cast<QWidget *>(this);
+				parentWnd->emitRedrawPlotBackground(dynamic_cast<QWidget *>(this));
+			}
+		}
+
+
 	//! Record the clicks of the mouse.
 	void
 		AbstractPlotWidget::mousePressHandler(QMouseEvent *event)
 		{
+			setFocus();
+
 			QPointF mousePoint = event->localPos();
 
 			// Let's check if the click is on the axes, either X or Y, because that
@@ -2079,12 +2474,39 @@ namespace msXpSmineXpert
 		}
 
 
+	//! Set the \p params MzIntegrationParams for this plot widget
+	void
+		AbstractPlotWidget::setMzIntegrationParams(MzIntegrationParams params)
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "with params:" << params.asText();
+
+			m_history.setMzIntegrationParams(params);
+		}
+
+
+	//! Get the MzIntegrationParams for this plot widget
+	MzIntegrationParams
+		AbstractPlotWidget::mzIntegrationParams() const
+		{
+			return m_history.mzIntegrationParams();
+		}
+
+
 	//! React to the release of the mouse buttons.
 	void
 		AbstractPlotWidget::mouseReleaseHandler(QMouseEvent *event)
 		{
 			if(!m_isDragging)
 			{
+				// Even if we are not dragging, if one of the registered keys is
+				// pressed, then handle the case.
+				if(m_regQtKeyCodeMap.contains(m_pressedKeyCode))
+				{
+					mouseReleaseHandledEvent(event);
+					return;
+				}
+
 				// We only want to show the context menu if the button is the right
 				// button.
 
@@ -2124,12 +2546,6 @@ namespace msXpSmineXpert
 				m_hStartTracer->setVisible(true);
 			}
 
-			// By definition we are stopping the drag operation by releasing the mouse
-			// button. Whatever that mouse button was pressed before and if there was
-			// one
-			// pressed before.
-			m_isDragging = false;
-
 			// If we were using the "quantum" display for the rescale of the axes
 			// using right-click drag on any axis, then reset the count to 0.
 			m_mouseMoveHandlerSkipCount = 0;
@@ -2172,29 +2588,22 @@ namespace msXpSmineXpert
 
 					if(m_clickWasOnXAxis || m_clickWasOnYAxis)
 					{
-						// We were doing an axis range operation which means we do not
-						// need
-						// to compute any thing, just let the system as is. This is
-						// because
-						// when the user clicks onto an axis and then right-button-drags
-						// the
-						// mouse, then the plot is update real-time, so we do not need
-						// to
-						// replot it, it is already plot fine. The only thing would be
-						// to
-						// change the Y axis range if the X axis was dragged-upon so as
-						// to
+
+						// We were doing an axis range operation which means we do not need
+						// to compute any thing, just let the system as is. This is because
+						// when the user clicks onto an axis and then right-button-drags the
+						// mouse, then the plot is update real-time, so we do not need to
+						// replot it, it is already plot fine. The only thing would be to
+						// change the Y axis range if the X axis was dragged-upon so as to
 						// maximise the zoom on the Y axis (à la mmass, in fact).
 
 						if(m_clickWasOnXAxis)
 						{
-							// We may either leave the scale of the Y axis as is
-							// (default) or
-							// the user may want an automatic scale of the Y axis such
-							// that the
-							// data displayed in the new X axis range are full scale on
-							// the Y
-							// axis. For this, the Shift modifier key should be pressed.
+
+							// We may either leave the scale of the Y axis as is (default) or
+							// the user may want an automatic scale of the Y axis such that
+							// the data displayed in the new X axis range are full scale on
+							// the Y axis. For this, the Shift modifier key should be pressed.
 
 							Qt::KeyboardModifiers modifiers =
 								QGuiApplication::queryKeyboardModifiers();
@@ -2269,8 +2678,7 @@ namespace msXpSmineXpert
 				updateAxisRangeHistory();
 
 				// Do not forget that we may have locked the x and/or y axis...
-				dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-					->emitReplotSignal(xAxis->range(), yAxis->range());
+				dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->emitReplotSignal(xAxis->range(), yAxis->range());
 			}
 			else if(event->button() == Qt::LeftButton)
 			{
@@ -2280,8 +2688,7 @@ namespace msXpSmineXpert
 					// want
 					// to propagate to the other widgets the range change.
 					// Do not forget that we may have locked the x and/or y axis...
-					dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-						->emitReplotSignal(xAxis->range(), yAxis->range());
+					dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->emitReplotSignal(xAxis->range(), yAxis->range());
 				}
 
 				// We were dragging with the left button pressed. Either we were
@@ -2300,8 +2707,7 @@ namespace msXpSmineXpert
 					updateAxisRangeHistory();
 
 					// Do not forget that we may have locked the x and/or y axis...
-					dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-						->emitReplotSignal(xAxis->range(), yAxis->range());
+					dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->emitReplotSignal(xAxis->range(), yAxis->range());
 				}
 				else if(m_selectLine->visible())
 				{
@@ -2315,6 +2721,14 @@ namespace msXpSmineXpert
 				}
 			}
 
+			// By definition we are stopping the drag operation by releasing the mouse
+			// button. Whatever that mouse button was pressed before and if there was
+			// one pressed before.
+			// We cannot set that boolean value to false before this place, because we
+			// call a number of routines above that need to know that dragging was
+			// occurring. Like mouseReleaseHandledEvent(event) for example.
+			m_isDragging = false;
+
 			// Replot because we want to make the graphical items invisible, be them
 			// line or
 			// rectangle.
@@ -2379,8 +2793,7 @@ namespace msXpSmineXpert
 			updateAxisRangeHistory();
 
 			// Do not forget that we may have locked the x and/or y axis...
-			dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)
-				->emitReplotSignal(xAxis->range(), yAxis->range());
+			dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd)->emitReplotSignal(xAxis->range(), yAxis->range());
 
 			event->accept();
 		}
@@ -2410,5 +2823,6 @@ namespace msXpSmineXpert
 		}
 
 
+
 } // namespace msXpSmineXpert
 
