@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #pragma once
 
 /////////////////////// Qt includes
@@ -50,6 +51,7 @@
 #include <minexpert/nongui/History.hpp>
 #include <libmass/Trace.hpp>
 #include <minexpert/nongui/MassSpecDataSet.hpp>
+#include <minexpert/nongui/MzIntegrationParams.hpp>
 
 
 namespace msXpSmineXpert
@@ -71,7 +73,7 @@ namespace msXpSmineXpert
 		protected:
 
 		//! Pointer to the parent object.
-		QWidget *mp_parentWnd = 0;
+		QWidget *mp_parentWnd = Q_NULLPTR;
 
 		//! Name of the plot widget.
 		QString m_name = "NOT_SET";
@@ -115,7 +117,7 @@ namespace msXpSmineXpert
 			data. This pointer allows us to taccess that MassSpecDataSet instance.
 
 			Note that we do now systematically own the data in MassSpecDataSet. The
-			processe leading to that ownership is as follows: 
+			processe leading to that ownership is as follows:
 
 			When a data file is loaded in full (that is, the mass data are stored in
 			the MassSpecDataSet instance) the first operation that is performed on
@@ -123,7 +125,7 @@ namespace msXpSmineXpert
 			display that TIC chromatogram, a TicChromPlotWidget object is
 			instanciated. <em> Only at that moment</em> the TicChromPlotWidget
 			instance that gets the pointer to the MassSpecDataSet instance calls
-			setParent() on that pointer so that it becomes the parent of the data. 
+			setParent() on that pointer so that it becomes the parent of the data.
 
 			This process is designed so that only when a TIC chromatogram plot
 			widget is destroyed the corresponding mass data get destroyed all along.
@@ -163,7 +165,7 @@ namespace msXpSmineXpert
 		bool m_isMultiGraph = false;
 
 		//! Pointer to the mz=f(dt) color map...
-		/*! 
+		/*!
 
 			That color map instance is only allocated when drift time mass
 			spectrometry data are handled.
@@ -171,8 +173,6 @@ namespace msXpSmineXpert
 */
 		QCPColorMap *mp_colorMap = Q_NULLPTR;
 
-		bool m_areDtKeys = true;
-
 		//! History member in which to store all the events...
 		/*!
 
@@ -198,6 +198,10 @@ namespace msXpSmineXpert
 		//! Pointer to the contextual menu.
 		QMenu *mpa_contextMenu = Q_NULLPTR;
 
+		QAction *mpa_exportDataAction = Q_NULLPTR;
+		QAction *mpa_plotXyFileAction = Q_NULLPTR;
+		QAction *mpa_savePlotGraphics = Q_NULLPTR;
+
 		//! Rectangle defining the borders of zoomed-in/out data.
 		QCPItemRect *m_zoomRect = Q_NULLPTR;
 
@@ -205,7 +209,7 @@ namespace msXpSmineXpert
 		QCPItemLine *m_selectLine = Q_NULLPTR;
 
 		//! Text that is displayed on the plotted graph to document it.
-		/*! 
+		/*!
 
 			That text is typically the file path of the data file.
 
@@ -254,7 +258,7 @@ namespace msXpSmineXpert
 		QPen m_pen;
 
 		//! How many mouse move events must be skipped */
-		/*! 
+		/*!
 
 			when the data are so massive that the graph panning becomes sluggish. By
 			default, the value is 10 events to be skipped before accounting one. The
@@ -321,26 +325,34 @@ namespace msXpSmineXpert
 		//! Color used for the background of focused plot.
 		QBrush m_focusedBrush = QBrush(m_focusedColor);
 
+		QColor m_plottingColor = QColor("black");
+
+		bool findIntegrationLowerRangeForKey(double key, QCPRange *p_range);
+
 		public:
 
-		explicit AbstractPlotWidget(QWidget *parent, QWidget *parentWnd, const QString &name,
+		explicit AbstractPlotWidget(QWidget *parent, const QString &name,
 				const QString &desc, MassSpecDataSet *massSpecDataSet, const QString &fileName = QString(),
 				bool isMultiGraph = false);
 
 		virtual ~AbstractPlotWidget();
 
+		QWidget *parentWnd();
+
 		Q_INVOKABLE void setName(QString name);
 		Q_INVOKABLE QString name() const;
 
 		Q_INVOKABLE void setDesc(QString desc);
 		Q_INVOKABLE QString desc() const;
 
+		void setMzIntegrationParams(MzIntegrationParams params);
+		MzIntegrationParams mzIntegrationParams() const;
 
-		double xRangeMin() const {return m_xRangeMin;}
-		double xRangeMax() const {return m_xRangeMax;}
+		Q_INVOKABLE double xRangeMin() const {return m_xRangeMin;}
+		Q_INVOKABLE double xRangeMax() const {return m_xRangeMax;}
 
-		double yRangeMin() const {return m_yRangeMin;}
-		double yRangeMax() const {return m_yRangeMax;}
+		Q_INVOKABLE double yRangeMin() const {return m_yRangeMin;}
+		Q_INVOKABLE double yRangeMax() const {return m_yRangeMax;}
 
 		QCPColorMap *colorMap() const{return mp_colorMap;}
 
@@ -352,11 +364,11 @@ namespace msXpSmineXpert
 		virtual bool setupWidget();
 		virtual QMenu *createContextMenu();
 
-		void showHelpSummary();
+		Q_INVOKABLE void showHelpSummary();
 
 		void clearHistory();
 		void addHistoryItem(HistoryItem *item);
-		void showHistory();
+		Q_INVOKABLE void showHistory();
 		Q_INVOKABLE QString historyAsText();
 
 		void clearAxisRangeHistory();
@@ -392,6 +404,7 @@ namespace msXpSmineXpert
 				QCPAxis::SelectablePart part,
 				QMouseEvent * event);
 
+		virtual void setFocus();
 		virtual void keyPressEvent(QKeyEvent *event);
 		virtual void keyReleaseEvent(QKeyEvent *event);
 
@@ -407,26 +420,34 @@ namespace msXpSmineXpert
 
 		Q_INVOKABLE void shouldDestroyPlotWidget();
 
-		void toggleMultiGraph();
-		void showMultiGraph();
-		void hideMultiGraph();
+		Q_INVOKABLE void toggleMultiGraph();
+		Q_INVOKABLE void showMultiGraph();
+		Q_INVOKABLE void hideMultiGraph();
 
 		QColor color() const;
-		void changeColor(const QColor& newColor);
+		void setPlottingColor(const QColor& newColor);
+		void changePlottingColor(const QColor& newColor);
 
 		Q_INVOKABLE QString asXyText();
 
 		Q_INVOKABLE virtual void exportData();
 		Q_INVOKABLE virtual void exportPlot();
-		Q_INVOKABLE virtual void exportPlotToFile(const QString &fileName, 
+		Q_INVOKABLE virtual void exportPlotToFile(const QString &fileName,
 				double start = qSNaN(), double end = qSNaN());
 
-		virtual bool savePlotToGraphicsFile();
+		Q_INVOKABLE virtual bool savePlotToGraphicsFile();
 
-		Q_INVOKABLE virtual bool saveToPdfFile(const QString &fileName, 
-				bool noCosmeticPen, int width, int height, 
+		Q_INVOKABLE virtual bool savePlotToPdfFile(const QString &fileName,
+				bool noCosmeticPen, int width, int height,
 				const QString &pdfCreator, const QString &title);
 
+
+		//! Return the arbitrary integration type that this plot widget can handle
+		/*!
+			This integration type is useful to craft HistoryItem objects reporting the
+			ranges at any given time in the plot widget, be it of any of the derived
+			classes.
+			*/
 		virtual int arbitraryIntegrationType() = 0;
 
 		Q_INVOKABLE QList<double> keys();
@@ -445,6 +466,11 @@ signals:
 		void toggleMultiGraphSignal(AbstractPlotWidget *widget,
 				bool recursive);
 
+		// This integration is particular and sits in the abstract class because it
+		// can be elicited from any plot widget type (mass, drift, tic).
+
+		void newTicChromatogram(MassSpecDataSet *massSpecDataSet,
+				const QString &msg, const History &history, QColor color);
 	};
 
 
