@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #include <qcustomplot.h>
 
 ///////////////////////////// Qt include
@@ -49,6 +50,15 @@ namespace msXpSmineXpert
 {
 
 
+	/*/js/ Class: ColorMapPlotWidget
+	 * <comment>This class cannot be instantiated with the new operator. Objects
+	 * of this class are made available to the scripting environment under the
+	 * form of object variable names colorMapPlotWidget[index], with [index]
+	 * being 0 for the first object created and being incremented each time a new
+	 * ColorMapPlotWidget is created.</comment>
+	*/
+
+
 	//! Construct a ColorMapPlotWidget instance.
 	/*!
 
@@ -56,8 +66,6 @@ namespace msXpSmineXpert
 
 		\param parent parent widget.
 
-		\param parentWnd parent window.
-
 		\param name name of this window to be used for the decoration (title)
 
 		\param desc description of this window to be used for the plot decoration.
@@ -73,23 +81,22 @@ namespace msXpSmineXpert
 
 */
 	ColorMapPlotWidget::ColorMapPlotWidget(QWidget *parent,
-			AbstractMultiPlotWnd *parentWnd,
 			const QString &name,
 			const QString &desc,
 			MassSpecDataSet *massSpecDataSet,
 			const QString &fileName,
 			bool isMultiGraph)
-		: AbstractPlotWidget{parent, parentWnd, name,
+		: AbstractPlotWidget{parent, name,
 			desc, massSpecDataSet, fileName, isMultiGraph}
 	{
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_R,
-			 	"Press T while right mouse click-drag to integrate to a XIC chromatogram");
-		AbstractPlotWidget::registerQtKeyCode(Qt::Key_S, 
+				"Press T while right mouse click-drag to integrate to a XIC chromatogram");
+		AbstractPlotWidget::registerQtKeyCode(Qt::Key_S,
 				"Press S while right mouse click-drag to integrate to a mass spectrum");
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_D,
-			 	"Press D while right mouse click-drag to integrate to a drift spectrum");
+				"Press D while right mouse click-drag to integrate to a drift spectrum");
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_I,
-			 	"Press I while right mouse click-drag to integrate to a TIC intensity value"); 
+				"Press I while right mouse click-drag to integrate to a TIC intensity value");
 
 		// Give the axes some labels:
 		xAxis->setLabel("drift time (ms)");
@@ -110,12 +117,12 @@ namespace msXpSmineXpert
 	}
 
 
-	//! Return the integration type that yielded the data in this plot.
-	int	
+	//! Return the arbitrary integration type that this plot widget can handle
+	int
 		ColorMapPlotWidget::arbitraryIntegrationType()
 		{
-			return 
-				IntegrationType::DATA_TO_DT | IntegrationType::DATA_TO_MZ;
+			return
+				IntegrationType::DT_TO_XXX | IntegrationType::MZ_TO_XXX;
 		}
 
 
@@ -144,7 +151,7 @@ namespace msXpSmineXpert
 		scale.
 
 */
-	void 
+	void
 		ColorMapPlotWidget::setColorMapPointData(double key, double val, double z)
 		{
 			mp_colorMap->data()->setData(key, val, z);
@@ -166,8 +173,8 @@ namespace msXpSmineXpert
 		valSize) cells of the map.
 
 */
-	void 
-		ColorMapPlotWidget::setColorMapSizeAndRanges(int keySize, int valSize, 
+	void
+		ColorMapPlotWidget::setColorMapSizeAndRanges(int keySize, int valSize,
 				QCPRange keyRange, QCPRange valRange, double fillingValue)
 		{
 			mp_colorMap->data()->setSize(keySize, valSize);
@@ -200,6 +207,9 @@ namespace msXpSmineXpert
 		yAxis->setBasePen(basePen);
 		yAxis->setLabelColor(color);
 		yAxis->setTickLabelColor(color);
+
+		// Finally store that color.
+		m_plottingColor = color;
 	}
 
 
@@ -217,7 +227,7 @@ namespace msXpSmineXpert
 		in the color map, to ensure that the map is redraw with proper scales.
 
 */
-	void 
+	void
 		ColorMapPlotWidget::rescaleColorMapDataRange(bool recalculateDataBounds)
 		{
 			mp_colorMap->rescaleDataRange(recalculateDataBounds);
@@ -281,6 +291,18 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * ColorMapPlotWidget.integrateToRt(dtRangeStart, dtRangeEnd, mzRangeStart, mzRangeEnd)
+	 *
+	 * Starts an integration of mass spectra data to a XIC chromatogram limiting
+	 * the data ranges specified with the numerical arguments.
+	 *
+	 * dtRange(Start | End) drift time range values
+	 * mzRange(Start | End) m/z range values
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Integrate data to a retention time (TIC/XIC) chromatogram
 	/*!
 
@@ -288,28 +310,25 @@ namespace msXpSmineXpert
 		ranges on both the x and y axes.
 
 */
-	void 
-		ColorMapPlotWidget::integrateToRt(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper)
-		{
-			QCPAxis *xAxis = mp_colorMap->keyAxis();
-			QPen basePen = xAxis->basePen();
-			QColor color = basePen.color();
+	void
+		ColorMapPlotWidget::integrateToRt(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd)
 
+		{
 			History localHistory = m_history;
 			HistoryItem *histItem = new HistoryItem;
 
-			histItem->newIntegrationRange(IntegrationType::DT_TO_RT, 
-					xAxisLower, xAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_DT_TO_RT,
+					dtRangeStart, dtRangeEnd);
 
-			histItem->newIntegrationRange(IntegrationType::MZ_TO_RT, 
-					yAxisLower, yAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_MZ_TO_RT,
+					mzRangeStart, mzRangeEnd);
 
 			// Aggreate the new item to the local history copy we made.
 			localHistory.appendHistoryItem(histItem);
 
-			emit newTicChromatogram(mp_massSpecDataSet, 
-					"Calculating extracted ion chromatogram.", localHistory, color);
+			emit newTicChromatogram(mp_massSpecDataSet,
+					"Calculating extracted ion chromatogram.", localHistory, m_plottingColor);
 		}
 
 
@@ -320,28 +339,61 @@ namespace msXpSmineXpert
 		ranges on both the x and y axes.
 
 */
-	void 
-		ColorMapPlotWidget::integrateToMz(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper)
+	/*/js/
+	 * ColorMapPlotWidget.integrateToMz(dtRangeStart, dtRangeEnd, mzRangeStart, mzRangeEnd)
+	 *
+	 * Starts an integration of mass spectra data to a mass spectrum limiting the
+	 * data ranges specified with the numerical arguments.
+	 *
+	 * dtRange(Start | End) drift time range values
+	 * mzRange(Start | End) m/z range values
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+	void
+		ColorMapPlotWidget::integrateToMz(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd)
 		{
-			QCPAxis *xAxis = mp_colorMap->keyAxis();
-			QPen basePen = xAxis->basePen();
-			QColor color = basePen.color();
-
 			History localHistory = m_history;
+
+			// The color map plot widget serves as a starting point for data
+			// exploration exactly as the TIC chromatogram plot widget that is created
+			// right after having loaded mass data. Thus, when integrating to a mass
+			// spectrum, we need to comply with the mz integration parameters that
+			// were set for that TIC chrom widget (so-called root widget for a given
+			// mass data file). Let's get that widget and obtain from it a copy of the
+			// mz integration parameters.
+
+			ColorMapWnd *parentWnd = static_cast<ColorMapWnd *>(mp_parentWnd);
+			MainWindow *mainWindow = static_cast<MainWindow *>(parentWnd->parent());
+
+			// This is merely a holder for a result of the call below
+			DataPlotWidgetRelation *plotWidgetRelation = Q_NULLPTR;
+
+			AbstractPlotWidget *rootPlotWidget =
+				mainWindow->m_dataPlotWidgetRelationer.rootPlotWidget(this, &plotWidgetRelation);
+
+			if(rootPlotWidget == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Cannot be that a color map widget has no root TIC chrom plot widget."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			MzIntegrationParams params = static_cast<TicChromPlotWidget *>(rootPlotWidget)->mzIntegrationParams();
+			localHistory.setMzIntegrationParams(params);
 			HistoryItem *histItem = new HistoryItem;
 
-			histItem->newIntegrationRange(IntegrationType::DT_TO_MZ, 
-					xAxisLower, xAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_DT_TO_MZ,
+					dtRangeStart, dtRangeEnd);
 
-			histItem->newIntegrationRange(IntegrationType::MZ_TO_MZ, 
-					yAxisLower, yAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_MZ_TO_MZ,
+					mzRangeStart, mzRangeEnd);
 
 			// Aggreate the new item to the local history copy we made.
 			localHistory.appendHistoryItem(histItem);
 
 			emit newMassSpectrum(mp_massSpecDataSet,
-					"Calculating mass spectrum.", localHistory, color);
+					"Calculating mass spectrum.", localHistory, m_plottingColor);
 		}
 
 
@@ -352,22 +404,29 @@ namespace msXpSmineXpert
 		ranges on both the x and y axes.
 
 */
-	void 
-		ColorMapPlotWidget::integrateToDt(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper)
+	/*/js/
+	 * ColorMapPlotWidget.integrateToDt(dtRangeStart, dtRangeEnd, mzRangeStart, mzRangeEnd)
+	 *
+	 * Starts an integration of mass spectra data to a drift spectrum limiting the
+	 * data ranges specified with the numerical arguments.
+	 *
+	 * dtRange(Start | End) drift time range values
+	 * mzRange(Start | End) m/z range values
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+	void
+		ColorMapPlotWidget::integrateToDt(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd)
 		{
-			QCPAxis *xAxis = mp_colorMap->keyAxis();
-			QPen basePen = xAxis->basePen();
-			QColor color = basePen.color();
-
 			History localHistory = m_history;
 			HistoryItem *histItem = new HistoryItem;
 
-			histItem->newIntegrationRange(IntegrationType::DT_TO_DT, 
-					xAxisLower, xAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_DT_TO_DT,
+					dtRangeStart, dtRangeEnd);
 
-			histItem->newIntegrationRange(IntegrationType::MZ_TO_DT, 
-					yAxisLower, yAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_MZ_TO_DT,
+					mzRangeStart, mzRangeEnd);
 
 			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 			//<< "histItem:" << histItem->asText();
@@ -376,38 +435,50 @@ namespace msXpSmineXpert
 			localHistory.appendHistoryItem(histItem);
 
 			emit newDriftSpectrum(mp_massSpecDataSet,
-					"Calculating drift spectrum.", localHistory, color);
+					"Calculating drift spectrum.", localHistory, m_plottingColor);
 		}
 
 
-	//! Integrate data to a single TIC intensity value 
+	//! Integrate data to a single TIC intensity value
 	/*!
 
 		The data to be integrated are bordered by the parameters that describe data
 		ranges on both the x and y axes.
 
 		This is not a conventional integration, because it does not produce any
-		graph, it only produces a single TIC intensity value. 
+		graph, it only produces a single TIC intensity value.
 
 */
-	double 
-		ColorMapPlotWidget::integrateToTicIntensity(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper)
+	/*/js/
+	 * ColorMapPlotWidget.integrateToTicIntensity(dtRangeStart, dtRangeEnd, mzRangeStart, mzRangeEnd)
+	 *
+	 * Starts an integration of mass spectra data to a TIC intensity single value
+	 * limiting the data ranges specified with the numerical arguments.
+	 *
+	 * dtRange(Start | End) drift time range values
+	 * mzRange(Start | End) m/z range values
+	 *
+	 * The TIC intensity value is returned
+	 */
+	double
+		ColorMapPlotWidget::integrateToTicIntensity(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd)
 		{
 			History localHistory = m_history;
 			HistoryItem *histItem = new HistoryItem;
 
-			histItem->newIntegrationRange(IntegrationType::DT_TO_TIC_INT, 
-					xAxisLower, xAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_DT_TO_TIC_INT,
+					dtRangeStart, dtRangeEnd);
 
-			histItem->newIntegrationRange(IntegrationType::MZ_TO_TIC_INT, 
-					yAxisLower, yAxisUpper);
+			histItem->newIntegrationRange(IntegrationType::DTMZ_MZ_TO_TIC_INT,
+					mzRangeStart, mzRangeEnd);
 
 			// Aggreate the new item to the local history copy we made.
 			localHistory.appendHistoryItem(histItem);
 
 			ColorMapWnd *parentWnd = static_cast<ColorMapWnd *>(mp_parentWnd);
 
+
 			// Capture immediately the text that is located in the status bar
 			// because if the computation is long that text will have disappeared
 			// and we won't be able to show it in the TIC int text.
@@ -426,10 +497,10 @@ namespace msXpSmineXpert
 
 			if(currentMessage.isEmpty())
 				parentWnd->statusBar()->showMessage(QString("dt range [%1-%2] ; m/z range [%3-%4]: TIC I = %5")
-						.arg(xAxisLower, 0, 'f', 3)
-						.arg(xAxisUpper, 0, 'f', 3)
-						.arg(yAxisLower, 0, 'f', 3)
-						.arg(yAxisUpper, 0, 'f', 3)
+						.arg(dtRangeStart, 0, 'f', 3)
+						.arg(dtRangeEnd, 0, 'f', 3)
+						.arg(mzRangeStart, 0, 'f', 3)
+						.arg(mzRangeEnd, 0, 'f', 3)
 						.arg(m_lastI, 0, 'g', 3));
 			else
 				parentWnd->statusBar()->showMessage(currentMessage +
@@ -439,8 +510,14 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * ColorMapPlotWidget.transposeAxes()
+	 *
+	 * Transpose the axes of this color map plot widget
+	 */
+
 	//! Transpose the axes such that the dt axis takes the place of the m/z axis.
-	void 
+	void
 		ColorMapPlotWidget::transposeAxes()
 		{
 			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" ;
@@ -450,17 +527,17 @@ namespace msXpSmineXpert
 			int keySize = origData->keySize();
 			int valueSize = origData->valueSize();
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "Orig data size:" << keySize << valueSize;
 
 			QCPRange keyRange = origData->keyRange();
 			QCPRange valueRange = origData->valueRange();
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "Value at cell 80,650:" << origData->cell(80,650);
 
 			// Transposed map.
-			QCPColorMapData *newData = 
+			QCPColorMapData *newData =
 				new QCPColorMapData(valueSize, keySize, valueRange, keyRange);
 
 			for(int iter = 0; iter < keySize; ++iter)
@@ -473,7 +550,7 @@ namespace msXpSmineXpert
 				}
 			}
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "New data size:" << newData->keySize() << newData->valueSize();
 
 			// At this point the transposition has been done.
@@ -498,7 +575,7 @@ namespace msXpSmineXpert
 				yAxis->setLabel("drift time (ms)");
 			}
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "Value at cell 80,650:" << origData->cell(80,650);
 
 			//QTimer::singleShot(9000, this,
@@ -507,7 +584,7 @@ namespace msXpSmineXpert
 			// Will take ownership of the newData.
 			mp_colorMap->setData(newData);
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "Value at cell 80,650:" << newData->cell(80,650)
 			//<< "Value at cell 650, 80:" << newData->cell(650,80);
 
@@ -535,10 +612,6 @@ namespace msXpSmineXpert
 				// The user wants to copy the current cursor location to the console
 				// window, such that it remains available after having moved the cursor.
 
-				QCPAxis *xAxis = mp_colorMap->keyAxis();
-				QPen basePen = xAxis->basePen();
-				QColor color = basePen.color();
-
 				QString label;
 
 				label = QString("(%1,%2)\n")
@@ -559,7 +632,7 @@ namespace msXpSmineXpert
 					// Send the label to the console window, so that we can make a
 					// copy paste.
 
-					mainWindow->logConsoleMessage(label, color);
+					mainWindow->logConsoleMessage(label, m_plottingColor);
 				}
 
 				event->accept();
@@ -572,4 +645,134 @@ namespace msXpSmineXpert
 		}
 
 
+	//! Record the clicks of the mouse.
+	void
+		ColorMapPlotWidget::mousePressHandler(QMouseEvent *event)
+		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
+
+			// Let the base class do the base work.
+			AbstractPlotWidget::mousePressHandler(event);
+
+			// Now, we have to update the m/z integration params in the corresponding
+			// window, but using the integration params that are located inside of the
+			// root plot widget's history object. This is because the color map plot
+			// widget actually needs to behave like the TIC chromatogram widget, as
+			// they both are the starting points for the exploration of mass data.
+
+			AbstractMultiPlotWnd *parentWnd = dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+
+			if(parentWnd == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+
+			// We now need to get the root widget's m/z integrations params.
+
+			MainWindow *mainWindow = static_cast<MainWindow *>(parentWnd->parent());
+
+			// The plot widget relation pointer that will be set in the call below.
+			DataPlotWidgetRelation *plotWidgetRelation;
+
+			AbstractPlotWidget *rootPlotWidget =
+				mainWindow->m_dataPlotWidgetRelationer.rootPlotWidget(this, &plotWidgetRelation);
+
+			if(rootPlotWidget == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Cannot be that a color map widget has no root TIC chrom plot widget."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			// We want to maintain a synchrony between both TIC chrom plot widget and
+			// Color Map plot widget. We need to do that, because when we'll update
+			// the m/z integration params in the corresponding window, the various
+			// spin boxes and checkboxes will trigger "ChangedValue" signals that will
+			// automatically update the values in the currently focused plot widget.
+			// If that focused widget is not the root plot widget of the one were are
+			// handling here now, then all the params will be messed up.
+
+			rootPlotWidget->setFocus();
+		}
+
+
+
+	/*/js/
+	 * ColorMapPlotWidget.jsSetMzIntegrationParams(mzIntegrationParams)
+	 *
+	 * Set the MzIntegrationParams object passed as parameter to this color map
+	 * plot widget.
+	 *
+	 * These parameters are taken into account for the integrations to a mass
+	 * spectrum.
+	 */
+	void
+		ColorMapPlotWidget::jsSetMzIntegrationParams()
+		{
+			if(m_isMultiGraph)
+				return;
+
+			MainWindow *mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+			QScriptContext *ctx = mainWindow->mp_scriptingWnd->m_scriptEngine.currentContext();
+
+			int argCount = ctx->argumentCount();
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "argCount:" << argCount;
+
+			if(argCount != 1)
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+
+			QScriptValue arg = ctx->argument(0);
+			QVariant variant = arg.data().toVariant();
+
+			if(!variant.isValid())
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+
+			if(variant.userType() == QMetaType::type("msXpSmineXpert::MzIntegrationParams"))
+			{
+				MzIntegrationParams params(qscriptvalue_cast<msXpSmineXpert::MzIntegrationParams>(arg));
+				AbstractPlotWidget::setMzIntegrationParams(params);
+
+				// Now update the values in the MzIntegrationParamsWnd.
+
+				AbstractMultiPlotWnd *parentWnd = dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+
+				if(parentWnd == Q_NULLPTR)
+					qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+
+				parentWnd->updateMzIntegrationParams(m_history.mzIntegrationParams(),
+						m_plottingColor);
+			}
+			else
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+		}
+
+
+	/*/js/
+	 * ColorMapPlotWidget.jsMzIntegrationParams()
+	 *
+	 * Return the MzIntegrationParams object currently set for this color map
+	 * plot widget.
+	 *
+	 * These parameters are taken into account for the integrations to a mass
+	 * spectrum.
+	 */
+	QScriptValue
+		ColorMapPlotWidget::jsMzIntegrationParams() const
+		{
+			MainWindow *mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+
+			if(m_isMultiGraph)
+				return mainWindow->mp_scriptingWnd->m_scriptEngine.toScriptValue(MzIntegrationParams());
+
+			return mainWindow->mp_scriptingWnd->m_scriptEngine.toScriptValue(m_history.mzIntegrationParams());
+		}
+
+
 } // namespace msXpSmineXpert
