@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #pragma once
 
 
@@ -39,6 +40,7 @@
 #include <QString>
 #include <QWidget>
 #include <QColor>
+#include <QScriptValue>
 
 
 ///////////////////////////// Local includes
@@ -71,18 +73,23 @@ namespace msXpSmineXpert
 		- the value (this is in map jargon the y-axis value, the mz value);
 
 		- the intensity (that is the z value, that is actually color coded in the map).
-	
+
 		The ColorMapPlotWidget may be used as a starting point for the data mining
 		process, as it is possible to trigger integrations starting from it to mass
 		spectrum or to drift spectrum.
 
-	 */
+*/
 	class ColorMapPlotWidget : public AbstractPlotWidget
 	{
 		Q_OBJECT;
 
+		private:
+
+		//! Tells if the keys of the color map are the mobility drift time.
+		bool m_areDtKeys = true;
+
 		public:
-		ColorMapPlotWidget(QWidget *parent, AbstractMultiPlotWnd *parentWnd,
+		ColorMapPlotWidget(QWidget *parent,
 				const QString &name, const QString &desc,
 				MassSpecDataSet *massSpecDataSet = nullptr,
 				const QString & fileName = QString(),
@@ -106,15 +113,19 @@ namespace msXpSmineXpert
 
 		void mouseReleaseHandledEvent(QMouseEvent *event);
 		void keyReleaseEvent(QKeyEvent *event);
+		void mousePressHandler(QMouseEvent *event);
 
-		Q_INVOKABLE void integrateToMz(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper);
-		Q_INVOKABLE void integrateToDt(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper);
-		Q_INVOKABLE void integrateToRt(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper);
-		Q_INVOKABLE double integrateToTicIntensity(double xAxisLower, double xAxisUpper,
-				double yAxisLower, double yAxisUpper);
+		Q_INVOKABLE void jsSetMzIntegrationParams();
+		Q_INVOKABLE QScriptValue jsMzIntegrationParams() const;
+
+		Q_INVOKABLE void integrateToMz(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd);
+		Q_INVOKABLE void integrateToDt(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd);
+		Q_INVOKABLE void integrateToRt(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd);
+		Q_INVOKABLE double integrateToTicIntensity(double dtRangeStart, double dtRangeEnd,
+				double mzRangeStart, double mzRangeEnd);
 
 		Q_INVOKABLE void transposeAxes();
 
