@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include <qmath.h>
 #include <omp.h>
 
@@ -63,14 +64,13 @@ namespace msXpSmineXpert
 			"Color-map" /*m_desc*/}
 	{
 		setupWindow();
-
-		show();
 	}
 
 
 	//! Destruct \c this ColorMapWnd instance.
 	ColorMapWnd::~ColorMapWnd()
 	{
+		writeSettings();
 	}
 
 
@@ -105,6 +105,8 @@ namespace msXpSmineXpert
 						__FILE__, __LINE__, __FUNCTION__);
 
 			mp_toolbar->addAction(mp_transposeAxesAct);
+
+			readSettings();
 		}
 
 
@@ -168,22 +170,33 @@ namespace msXpSmineXpert
 			// Fill-in the dt/mass spectrum hash. This is the hardest computing part
 			// of the whole color map creation task.
 
-			int res = integrator.fillInDtMzHash(&dtValues, &mzCells, 
-					&firstKey, &lastKey, 
+			int res = integrator.fillInDtMzHash(&dtValues, &mzCells,
+					&firstKey, &lastKey,
 					&firstVal, &lastVal);
 
+			// Hide the cancel button, the computation is done now.
+			mp_statusBarCancelPushButton->setVisible(false);
+
 			if(res == 0)
 			{
 				// This is not necessarily an error, the loaded file might not be an
 				// ion mobility mass spectrometry experiment.
 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "Loaded file is not of an ion mobility mass spectrometry experiment.";
+
 				*ok = true;
+
 				return Q_NULLPTR;
 			}
 
 			if(res == -1)
 			{
 				*ok = false;
+
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Error while filling in the dt/mz hash.";
+
 				return Q_NULLPTR;
 			}
 
@@ -192,16 +205,13 @@ namespace msXpSmineXpert
 			//<< "firstKey:" << firstKey << "lastKey:" << lastKey
 			//<< "firstVal:" << firstVal << "lastVal:" << lastVal;
 
-			ColorMapPlotWidget *widget = 
-				new ColorMapPlotWidget(this, this, m_name, m_desc, 
+			ColorMapPlotWidget *widget =
+				new ColorMapPlotWidget(this, m_name, m_desc,
 						massSpecDataSet, massSpecDataSet->fileName());
 
 			widget->setMinimumSize(400, 150);
 			mp_scrollAreaVBoxLayout->addWidget(widget);
 
-			// Hide the cancel button.
-			mp_statusBarCancelPushButton->setVisible(false);
-
 			// We may have gotten here because the operation was cancelled. In any case,
 			// reset the boolean value.
 			m_isOperationCancelled = false;
@@ -217,7 +227,7 @@ namespace msXpSmineXpert
 			// was cancelled then parsedDtValues is less than dtListSize).
 
 			widget->setColorMapSizeAndRanges(dtValues, mzCells,
-					QCPRange(firstKey, lastKey), QCPRange(firstVal, lastVal), 
+					QCPRange(firstKey, lastKey), QCPRange(firstVal, lastVal),
 					0 /* fill with 0 */);
 
 			// The version below will create as many cells in the two axes as there
@@ -228,20 +238,17 @@ namespace msXpSmineXpert
 
 			// Let's see if one color was set as the desired color, otherwise use
 			// nothing.
+
+			QColor localColor = Qt::black;
+
 			if(m_wishedColor.isValid())
-			{
+				localColor = m_wishedColor;
 
-				// I quit coloring the background of the widget because that was too
-				// heavy on the eyes.  
-				//
-				// widget->setBackground(QBrush(m_wishedColor));
-
-				widget->setDecoratingColor(m_wishedColor);
-			}
+			widget->setPlottingColor(localColor);
 
 			QHashIterator<double, msXpSlibmass::MassSpectrum *> iterator(integrator.doubleMassSpecHash());
 
-			while(iterator.hasNext()) 
+			while(iterator.hasNext())
 			{
 				iterator.next();
 
@@ -302,10 +309,10 @@ namespace msXpSmineXpert
 			// "massSpec".
 
 			QString propName =
-				QString("colorMapPlot%1").arg(ColorMapWnd::widgetCounter);
-			QString alias = "lastColorMapPlot";
+				QString("colorMapPlotWidget%1").arg(ColorMapWnd::widgetCounter);
+			QString alias = "lastColorMapPlotWidget";
 
-			QString explanation = QString("Make available a color map plot under name \"%1\" "
+			QString explanation = QString("Make available a color map plot widget under name \"%1\" "
 					"with alias \"%2\".\n").arg(propName).arg(alias);
 
 			// Make the new plot widget globally available under the crafted property
@@ -314,7 +321,7 @@ namespace msXpSmineXpert
 			// tree widget as a child item of this.
 
 			QScriptValue scriptValue;
-			mainWindow->mp_scriptingWnd->publishQObject(widget, this /* parent object */, 
+			mainWindow->mp_scriptingWnd->publishQObject(widget, this /* parent object */,
 					propName, alias, explanation, QString() /* help string */, scriptValue);
 
 			++ColorMapWnd::widgetCounter;
