@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 ///////////////////////////// Qt include
 
 
@@ -48,6 +49,15 @@ namespace msXpSmineXpert
 {
 
 
+	/*/js/ Class: DriftSpecPlotWidget
+	 * <comment>This class cannot be instantiated with the new operator. Objects
+	 * of this class are made available to the scripting environment under the
+	 * form of object variable names driftSpecPlotWidget[index], with [index]
+	 * being 0 for the first object created and being incremented each time a new
+	 * DriftSpecPlotWidget is created.</comment>
+	 */
+
+
 	//! Construct an initialized DriftSpecPlotWidget instance.
 	/*!
 
@@ -56,8 +66,6 @@ namespace msXpSmineXpert
 
 		\param parent parent widget.
 
-		\param parentWnd parent window.
-
 		\param name name of the module.
 
 		\param desc description of the wiget type.
@@ -71,22 +79,21 @@ namespace msXpSmineXpert
 
 */
 	DriftSpecPlotWidget::DriftSpecPlotWidget(QWidget *parent,
-			AbstractMultiPlotWnd *parentWnd,
 			const QString &name,
 			const QString &desc,
 			MassSpecDataSet *massSpecDataSet,
 			const QString &fileName,
 			bool isMultiGraph)
-		: AbstractPlotWidget{parent, parentWnd, name,
+		: AbstractPlotWidget{parent, name,
 			desc, massSpecDataSet, fileName, isMultiGraph}
 	{
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_R,
-			 	"Press T while right mouse click-drag to integrate to a XIC chromatogram");
-		AbstractPlotWidget::registerQtKeyCode(Qt::Key_S, 
+				"Press T while right mouse click-drag to integrate to a XIC chromatogram");
+		AbstractPlotWidget::registerQtKeyCode(Qt::Key_S,
 				"Press S while right mouse click-drag to integrate to a mass spectrum");
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_I,
-			 	"Press I while right mouse click-drag to integrate to a TIC intensity value");
-		
+				"Press I while right mouse click-drag to integrate to a TIC intensity value");
+
 		// Give the axes some labels:
 		xAxis->setLabel("drift time (ms)");
 		yAxis->setLabel("counts");
@@ -99,16 +106,11 @@ namespace msXpSmineXpert
 	}
 
 
-	//! Return the drift spectrum-specific arbitrary integration type.
-	/*!
-
-		When performing a number of integrations, the default integration type is returned.
-
-*/
-	int	
+	//! Return the arbitrary integration type that this plot widget can handle
+	int
 		DriftSpecPlotWidget::arbitraryIntegrationType()
 		{
-			return IntegrationType::ANY_TO_DT;
+			return IntegrationType::DT_TO_XXX;
 		}
 
 
@@ -360,10 +362,10 @@ namespace msXpSmineXpert
 		DriftSpecPlotWidget::mouseReleaseHandledEvent(QMouseEvent *event)
 		{
 			if(m_isMultiGraph)
-			// All the actions below can only be performed if the plot widget has a
-			// knowledge of the mass spec data set. Thus, if *this plot widget is a
-			// multi-graph plot widget we need to return immediately.
-			return;
+				// All the actions below can only be performed if the plot widget has a
+				// knowledge of the mass spec data set. Thus, if *this plot widget is a
+				// multi-graph plot widget we need to return immediately.
+				return;
 
 			if(m_pressedKeyCode == Qt::Key_S && event->button() == Qt::RightButton)
 			{
@@ -382,15 +384,27 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * DriftSpecPlotWidget.integrateToMz(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a mass spectrum limiting the
+	 * data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Integrate data to a mass spectrum with drift time range [\p lower -- \p upper ].
-	void 
+	void
 		DriftSpecPlotWidget::integrateToMz(double lower, double upper)
 		{
 			// Get the pointer to the x-axis (key axis).
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -416,15 +430,27 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * DriftSpecPlotWidget.integrateToRt(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a XIC chromatogram limiting the
+	 * data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Integrate data to a TIC chromatogram (XIC) with drift time range [\p lower -- \p upper ].
-	void 
+	void
 		DriftSpecPlotWidget::integrateToRt(double lower, double upper)
 		{
 			// Get the pointer to the x-axis (key axis).
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -450,11 +476,23 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * DriftSpecPlotWidget.integrateToTicIntensity(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a TIC intensity single value
+	 * limiting the data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * The TIC intensity value is returned
+	 */
+
 	//! Integrate data to a single TIC intensity value with drift time range [\p lower -- \p upper ].
-	double	
+	double
 		DriftSpecPlotWidget::integrateToTicIntensity(double lower, double upper)
 		{
-if(m_isMultiGraph)
+			if(m_isMultiGraph)
 				// We cannot perform the action, because this plot widget has not mass
 				// spec data set pointer, it is a graph-only plot widget to display all
 				// the graphs overlaid.
@@ -464,7 +502,7 @@ if(m_isMultiGraph)
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -525,7 +563,7 @@ if(m_isMultiGraph)
 			// Sanity check. It is not possible that this plot widget is not
 			// multi-graph and that at the same time, the mass spec data set pointer
 			// is nullptr.
-			
+
 			if(!m_isMultiGraph && mp_massSpecDataSet == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d -- %s(). "
 						"Programming error."
@@ -556,6 +594,16 @@ if(m_isMultiGraph)
 		}
 
 
+	/*/js/
+	 * DriftSpecPlotWidget.newPlot(trace)
+	 *
+	 * Create a new DriftSpecPlotWidget using <Trace> trace for the initialization
+	 * of the data.
+	 *
+	 * This function creates a new plot widget to display the <Trace> data.
+	 */
+
+	//! Create a new plot. This function is useful in the scripting environment.
 	void
 		DriftSpecPlotWidget::newPlot()
 		{
@@ -569,7 +617,7 @@ if(m_isMultiGraph)
 			QScriptContext *ctx = mainWindow->mp_scriptingWnd->m_scriptEngine.currentContext();
 
 			int argCount = ctx->argumentCount();
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "argCount:" << argCount;
 
 			if(argCount != 1)
@@ -581,7 +629,7 @@ if(m_isMultiGraph)
 			QScriptValue arg = ctx->argument(0);
 			QVariant variant = arg.data().toVariant();
 
-			if(!variant.isValid())	
+			if(!variant.isValid())
 			{
 				ctx->throwError("Syntax error: function newPlot() takes one Trace object as argument.");
 				return;
@@ -598,7 +646,7 @@ if(m_isMultiGraph)
 				QColor color = pen.color();
 
 				static_cast<TicChromWnd *>(mp_parentWnd)->newPlot(this, mp_massSpecDataSet,
-						QVector<double>::fromList(trace.keyList()), 
+						QVector<double>::fromList(trace.keyList()),
 						QVector<double>::fromList(trace.valList()),
 						"Trace from scripting environment.", m_history, color);
 			}
@@ -612,7 +660,7 @@ if(m_isMultiGraph)
 
 
 	//! Key release event handler.
-	/*! 
+	/*!
 
 		Depending on the type of plot widget (multigraph or not), the actions
 		triggered in this function will differ.
@@ -1187,4 +1235,67 @@ if(m_isMultiGraph)
 		}
 
 
+	void
+		DriftSpecPlotWidget::jsSetMzIntegrationParams()
+		{
+			if(m_isMultiGraph)
+				return;
+
+			MainWindow *mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+			QScriptContext *ctx = mainWindow->mp_scriptingWnd->m_scriptEngine.currentContext();
+
+			int argCount = ctx->argumentCount();
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "argCount:" << argCount;
+
+			if(argCount != 1)
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+
+			QScriptValue arg = ctx->argument(0);
+			QVariant variant = arg.data().toVariant();
+
+			if(!variant.isValid())
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+
+			if(variant.userType() == QMetaType::type("msXpSmineXpert::MzIntegrationParams"))
+			{
+				MzIntegrationParams params(qscriptvalue_cast<msXpSmineXpert::MzIntegrationParams>(arg));
+				AbstractPlotWidget::setMzIntegrationParams(params);
+
+				// Now update the values in the MzIntegrationParamsWnd.
+
+				AbstractMultiPlotWnd *parentWnd = dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+
+				if(parentWnd == Q_NULLPTR)
+					qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+
+				parentWnd->updateMzIntegrationParams(m_history.mzIntegrationParams(),
+						m_plottingColor);
+			}
+			else
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+		}
+
+
+	QScriptValue
+		DriftSpecPlotWidget::jsMzIntegrationParams() const
+		{
+			MainWindow *mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+
+			if(m_isMultiGraph)
+				return mainWindow->mp_scriptingWnd->m_scriptEngine.toScriptValue(MzIntegrationParams());
+
+			return mainWindow->mp_scriptingWnd->m_scriptEngine.toScriptValue(m_history.mzIntegrationParams());
+		}
+
+
 } // namespace msXpSmineXpert
