@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include <qmath.h>
 #include <omp.h>
 
@@ -68,13 +69,12 @@ namespace msXpSmineXpert
 			"Drift-spectrum" /*m_desc*/}
 	{
 		initialize();
-
-		show();
 	}
 
 	//! Destruct \c this DriftSpecWnd instance.
 	DriftSpecWnd::~DriftSpecWnd()
 	{
+		writeSettings();
 	}
 
 
@@ -83,6 +83,8 @@ namespace msXpSmineXpert
 		DriftSpecWnd::initialize()
 		{
 			initializePlotRegion();
+
+			readSettings();
 		}
 
 
@@ -99,7 +101,7 @@ namespace msXpSmineXpert
 			// true parameter in the function call below).
 
 			mp_multiGraphPlotWidget =
-				new DriftSpecPlotWidget(this, this, m_name, m_desc, Q_NULLPTR /* MassSpecDataSet * */, 
+				new DriftSpecPlotWidget(this, m_name, m_desc, Q_NULLPTR /* MassSpecDataSet * */,
 						QString() /* fileName */, true /* isMultiGraph */);
 
 			mp_multiGraphPlotWidget->setObjectName("multiGraphPlot");
@@ -181,8 +183,8 @@ namespace msXpSmineXpert
 			else
 				fileName = "NOT_SET";
 
-			DriftSpecPlotWidget *widget = 
-				new DriftSpecPlotWidget(this, this, m_name, desc, 
+			DriftSpecPlotWidget *widget =
+				new DriftSpecPlotWidget(this, m_name, desc,
 						massSpecDataSet, fileName, isMultiGraph);
 
 			widget->setObjectName("monoGraphPlot");
@@ -194,10 +196,10 @@ namespace msXpSmineXpert
 			// "driftSpecPlot".
 
 			QString propName =
-				QString("driftSpecPlot%1").arg(DriftSpecWnd::widgetCounter);
-			QString alias = "lastDriftSpectrumPlot";
+				QString("driftSpecPlotWidget%1").arg(DriftSpecWnd::widgetCounter);
+			QString alias = "lastDriftSpectrumPlotWidget";
 
-			QString explanation = QString("Make available a drift spectrum plot under name \"%1\" "
+			QString explanation = QString("Make available a drift spectrum plot widget under name \"%1\" "
 					"with alias \"%2\".\n").arg(propName).arg(alias);
 
 			// Make the new plot widget globally available under the crafted property
@@ -237,14 +239,13 @@ namespace msXpSmineXpert
 
 			// Let's see if one color was set as the desired color, otherwise use
 			// black.
-			QPen currentPen = widget->graph()->pen();
 
-			if(!m_wishedColor.isValid())
-				currentPen.setColor(Qt::black);
-			else
-				currentPen.setColor(m_wishedColor);
+			QColor localColor = Qt::black;
 
-			widget->graph()->setPen(currentPen);
+			if(m_wishedColor.isValid())
+				localColor = m_wishedColor;
+
+			widget->setPlottingColor(localColor);
 
 			widget->rescaleAxes(true /*only enlarge*/);
 			widget->replot();
@@ -257,7 +258,9 @@ namespace msXpSmineXpert
 
 			QCPGraph *newGraph = mp_multiGraphPlotWidget->addGraph();
 			newGraph->setData(keyVector, valVector);
-			newGraph->setPen(currentPen);
+
+			// Duplicate the same widget pen for the multigraph graph.
+			newGraph->setPen(widget->graph()->pen());
 			mp_multiGraphPlotWidget->rescaleAxes(true /*only enlarge*/);
 			mp_multiGraphPlotWidget->replot();
 
@@ -331,9 +334,9 @@ namespace msXpSmineXpert
 			//qDebug() << "At this point the integration is finished. Desetup the feedback.";
 
 			//De-setup the feedback.
-			QTimer::singleShot(2000, this, [=]() 
-					{ 
-					setupFeedback("", -1, -1, -1, LogType::LOG_TO_STATUS_BAR); 
+			QTimer::singleShot(2000, this, [=]()
+					{
+					setupFeedback("", -1, -1, -1, LogType::LOG_TO_STATUS_BAR);
 					}
 					);
 
@@ -342,7 +345,7 @@ namespace msXpSmineXpert
 
 			if(!res)
 			{
-				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 					<< "The integration to Dt has created no drift spectrum.";
 
 				return;
