@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #pragma once
 
 
@@ -51,18 +52,18 @@
 #include <minexpert/gui/AbstractPlotWidget.hpp>
 
 #include <minexpert/gui/ConsoleWnd.hpp>
-#include <libmassgui/ScriptingWnd.hpp>
+#include <minexpert/gui/ScriptingWnd.hpp>
 #include <minexpert/gui/TicChromWnd.hpp>
 #include <minexpert/gui/DriftSpecWnd.hpp>
 #include <minexpert/gui/MassSpecWnd.hpp>
 #include <minexpert/gui/ColorMapWnd.hpp>
-
+#include <minexpert/gui/XicExtractionWnd.hpp>
+#include <minexpert/gui/MzIntegrationParamsWnd.hpp>
 #include <minexpert/nongui/MassSpecDataSet.hpp>
 #include <minexpert/gui/OpenSpectraDlg.hpp>
-
 #include <minexpert/nongui/MassSpecDataFileLoaderSqlite3.hpp>
-
 #include <minexpert/gui/DataPlotWidgetRelation.hpp>
+#include <minexpert/gui/SqlMassDataSlicerWnd.hpp>
 
 
 namespace msXpSmineXpert
@@ -80,7 +81,6 @@ namespace msXpSmineXpert
 
 		public:
 
-
 			//! DataPlotWidgetRelationer instance responsible for maintaining plot widget relations.
 			DataPlotWidgetRelationer m_dataPlotWidgetRelationer;
 
@@ -103,14 +103,24 @@ namespace msXpSmineXpert
 			ConsoleWnd *mp_consoleWnd = Q_NULLPTR;
 
 			//! Scripting window where all the scripting activity is performed.
-			msXpSlibmassgui::ScriptingWnd *mp_scriptingWnd = Q_NULLPTR;
+			ScriptingWnd *mp_scriptingWnd = Q_NULLPTR;
+
+			//! XIC extraction configuration window.
+			XicExtractionWnd* mp_xicExtractionWnd = Q_NULLPTR;
+
+			//! Mz integration configuration window
+			MzIntegrationParamsWnd *mp_mzIntegrationParamsWnd = Q_NULLPTR;
+
+			//! Sql data slicer configuration window.
+			SqlMassDataSlicerWnd *mp_sqlMassDataSlicerWnd = Q_NULLPTR;
+
 
 			// Dialog window displaying all the currently opened data files in a list widget.
 			/*!
 
 				The dialog window has a drop down menu allowing to take actions against
 				the items in the list widget, like hiding, showing, closing.
-				
+
 				*/
 			OpenSpectraDlg *mp_openSpectraDlg = Q_NULLPTR;
 
@@ -142,6 +152,7 @@ namespace msXpSmineXpert
 			QMenu *m_fileMenu;
 			QMenu *m_plotMenu;
 			QMenu *m_windowsMenu;
+			QMenu *m_dataProcessingMenu;
 			QMenu *m_helpMenu;
 
 			QAction *m_openFullMsFileAct;
@@ -160,13 +171,20 @@ namespace msXpSmineXpert
 			QAction *m_showOpenSpectraDlgAct;
 			QAction *m_showConsoleWndAct;
 			QAction *m_showScriptingWndAct;
+			QAction *m_showXicExtractionWndAct;
+			QAction *m_showMzIntegrationParamsWndAct;
+			QAction *m_showSqlMassDataSlicerWndAct;
+
 			QAction *m_showAllWindowsAct;
 			QAction *m_saveWorkspaceAct;
-			
+
+
 			// Construction/destruction
 			MainWindow(const QString &moduleName);
 			~MainWindow();
 
+			const OpenSpectraDlg *openSpectraDlg() const;
+
 			void createMenusAndActions();
 
 			void writeSettings();
@@ -175,9 +193,11 @@ namespace msXpSmineXpert
 			Q_INVOKABLE void about();
 			Q_INVOKABLE void quit();
 
+			void concludeFileLoading();
+
 			void seedInitialPlots(MassSpecDataSet *massSpecDataSet);
 
-			void clearPlots();
+			Q_INVOKABLE void clearPlots();
 
 			void closeEvent(QCloseEvent *event);
 
@@ -204,7 +224,7 @@ namespace msXpSmineXpert
 			void statusBarMessage(QString msg);
 			void statusBarProgress(int value);
 
-			Q_INVOKABLE void createMassSpectrumClipboard();
+			Q_INVOKABLE void createMassSpectrumFromClipboard();
 
 			Q_INVOKABLE void openMassSpectrometryFile(const QString &fileName = QString(), bool isStreamed = false);
 			Q_INVOKABLE void openMassSpectrometryFiles(const QStringList &fileList,
@@ -219,6 +239,8 @@ namespace msXpSmineXpert
 
 			void makeLoaderConnections(MassSpecDataFileLoader *loader);
 
+			void xicExtraction();
+
 			void analysisPrefDlg();
 			void showTicChromWnd();
 			void showMassSpecWnd();
@@ -227,6 +249,10 @@ namespace msXpSmineXpert
 			void showOpenMassSpectraDlg();
 			void showConsoleWnd();
 			void showScriptingWnd();
+			void showXicExtractionWnd();
+			void showMzIntegrationParamsWnd();
+			void showSqlMassDataSlicerWnd();
+
 			void showAllWindows();
 			void saveWorkspace();
 
