@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 ///////////////////////////// Qt include
 
 
@@ -47,6 +48,14 @@
 namespace msXpSmineXpert
 {
 
+	/*/js/ Class: MassSpecPlotWidget
+	 * <comment>This class cannot be instantiated with the new operator. Objects
+	 * of this class are made available to the scripting environment under the
+	 * form of object variable names massSpecPlotWidget[index], with [index]
+	 * being 0 for the first object created and being incremented each time a new
+	 * MassSpecPlotWidget is created.</comment>
+	 */
+
 
 	//! Construct an initialized MassSpecPlotWidget instance.
 	/*!
@@ -56,8 +65,6 @@ namespace msXpSmineXpert
 
 		\param parent parent widget.
 
-		\param parentWnd parent window.
-
 		\param name name of the module.
 
 		\param desc description of the wiget type.
@@ -71,22 +78,21 @@ namespace msXpSmineXpert
 
 */
 	MassSpecPlotWidget::MassSpecPlotWidget(QWidget * parent,
-			AbstractMultiPlotWnd *parentWnd,
 			const QString & name,
 			const QString & desc,
 			MassSpecDataSet * massSpecDataSet,
 			const QString & fileName,
 			bool isMultiGraph)
-		: 
-			AbstractPlotWidget{parent, parentWnd, name,
+		:
+			AbstractPlotWidget{parent, name,
 				desc, massSpecDataSet, fileName, isMultiGraph}
 	{
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_R,
-			 	"Press T while right mouse click-drag to integrate to a XIC chromatogram");
+				"Press R while right mouse click-drag to integrate to a XIC chromatogram");
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_D,
-			 	"Press D while right mouse click-drag to integrate to a drift spectrum");
+				"Press D while right mouse click-drag to integrate to a drift spectrum");
 		AbstractPlotWidget::registerQtKeyCode(Qt::Key_I,
-			 	"Press I while right mouse click-drag to integrate to a TIC intensity value");
+				"Press I while right mouse click-drag to integrate to a TIC intensity value");
 
 		// Give the axes some labels:
 		xAxis->setLabel("m/z");
@@ -100,6 +106,21 @@ namespace msXpSmineXpert
 	}
 
 
+	QMenu *
+		MassSpecPlotWidget::createContextMenu()
+		{
+			AbstractPlotWidget::createContextMenu();
+
+			// In the mass spectrum plot widget, exporting the data does not make
+			// sense because we cannot filter the mass spectra for a valid m/z range.
+			// So we need to hide the corresponding menu item.
+
+			mpa_exportDataAction->setVisible(false);
+
+			return mpa_contextMenu;
+		}
+
+
 	//! Deconvolute the mass peaks into charge and molecular mass.
 	bool
 		MassSpecPlotWidget::deconvolute()
@@ -115,8 +136,14 @@ namespace msXpSmineXpert
 
 			// Note that m_xDelta is always fabs().
 
-			if(m_xDelta <= 1)
+			if(m_xDelta >= 0.9 && m_xDelta <= 1.1)
+			{
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					//<< "m_xDelta:" << m_xDelta
+					//<< "trying isotope-based deconvolution.";
+
 				return deconvoluteIsotopicCluster();
+			}
 
 			// If not deconvoluting on the basis of the isotopic cluster, then:
 
@@ -125,6 +152,9 @@ namespace msXpSmineXpert
 			MassSpecWnd *wnd = dynamic_cast<MassSpecWnd *>(mp_parentWnd);
 			int span = wnd->chargeSpan();
 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "span is:" << span;
+
 			return deconvoluteChargedState(span);
 		}
 
@@ -225,9 +255,9 @@ namespace msXpSmineXpert
 			// then we consider that the peaks do not belong to the same charge state
 			// envelope.
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
-			//<< "Charge:" << chargeIntPart
-			//<< "Charge fractional part: " << chargeFracPart;
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+				//<< "Charge:" << chargeIntPart
+				//<< "Charge fractional part: " << chargeFracPart;
 
 			if(chargeFracPart >= (1 - m_chargeFracPartTolerance /* that is >= 0.01 */) &&
 					chargeFracPart <= m_chargeFracPartTolerance /* that is <= 0.99 */)
@@ -238,8 +268,8 @@ namespace msXpSmineXpert
 				m_lastMass = qSNaN();
 
 				//qDebug() << __FILE__ << __LINE__
-				//<< "Not a charge state family peak,"
-				//<< "returning from deconvoluteChargeState";
+					//<< "Not a charge state family peak,"
+					//<< "returning from deconvoluteChargeState";
 
 				return false;
 			}
@@ -279,10 +309,10 @@ namespace msXpSmineXpert
 			m_lastMass = (curMz * m_lastCharge) - (m_lastCharge * m_protonMass);
 
 			//qDebug() << __FILE__ << __LINE__
-			//<< "startMz:" << QString("%1").arg(startMz, 0, 'f', 6)
-			//<< "m_lastMz (curMz):" << QString("%1").arg(m_lastMz, 0, 'f', 6)
-			//<< "m_lastMass:" << QString("%1").arg(m_lastMass, 0, 'f', 6)
-			//<< "m_lastCharge:" << QString("%1").arg(m_lastCharge);
+				//<< "startMz:" << QString("%1").arg(startMz, 0, 'f', 6)
+				//<< "m_lastMz (curMz):" << QString("%1").arg(m_lastMz, 0, 'f', 6)
+				//<< "m_lastMass:" << QString("%1").arg(m_lastMass, 0, 'f', 6)
+				//<< "m_lastCharge:" << QString("%1").arg(m_lastCharge);
 
 			return true;
 		}
@@ -325,7 +355,7 @@ namespace msXpSmineXpert
 				return false;
 			}
 
-			// Update the start and end range values for user in another function.
+			// Update the start and end range values for use in another function.
 			m_keyRangeStart = m_xRangeMin;
 			m_keyRangeEnd = m_xRangeMax;
 
@@ -333,6 +363,13 @@ namespace msXpSmineXpert
 			double chargeIntPart;
 			double chargeFracPart = modf(chargeTemp, &chargeIntPart);
 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "m_xDelta:" << m_xDelta
+			//<< "chargeTemp:" << chargeTemp
+			//<< "chargeIntPart:" << chargeIntPart
+			//<< "chargeFracPart:" << chargeFracPart
+			//<< "m_chargeFracPartTolerance:" << m_chargeFracPartTolerance;
+
 			if(chargeFracPart >= (1 - m_chargeFracPartTolerance) &&
 					chargeFracPart <= m_chargeFracPartTolerance)
 			{
@@ -351,10 +388,20 @@ namespace msXpSmineXpert
 			}
 
 			if(chargeFracPart > m_chargeFracPartTolerance)
+			{
 				m_lastCharge = chargeIntPart + 1;
+
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "chargeFracPart > m_chargeFracPartTolerance -> m_lastCharge = " << m_lastCharge;
+			}
 			else
+			{
 				m_lastCharge = chargeIntPart;
 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "chargeFracPart <=  m_chargeFracPartTolerance -> m_lastCharge = " << m_lastCharge;
+			}
+
 			// Now that we have the charge in the form of an int, we can compute the
 			// Mr of the lightest isotopic cluster peak (the one that has the lowest x
 			// value). That value is stored in m_xRangeMin.
@@ -651,6 +698,9 @@ namespace msXpSmineXpert
 				// of the corresponding isotopic cluster is being measure if there is
 				// the m_xDeltaText item associated to the plot.
 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< QDateTime().currentDateTime() << "now trying a deconvolution";
+
 				if(!deconvolute())
 					return;
 
@@ -710,15 +760,27 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * MassSpecPlotWidget.integrateToDt(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a drift spectrum limiting the
+	 * data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Perform a ranged [\p lower -- \p upper] data integration to a drift spectrum.
 	void
 		MassSpecPlotWidget::integrateToDt(double lower, double upper)
-		{	
+		{
 			// Get the pointer to the x-axis (key axis).
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -739,10 +801,22 @@ namespace msXpSmineXpert
 			localHistory.appendHistoryItem(histItem);
 
 			emit newDriftSpectrum(mp_massSpecDataSet, "Calculating drift spectrum.",
-					localHistory, graph()->pen().color());
+					localHistory, m_plottingColor);
 		}
 
 
+	/*/js/
+	 * MassSpecPlotWidget.integrateToRt(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a XIC chromatogram limiting the
+	 * data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Perform a ranged [\p lower -- \p upper] data integration to a TIC chromatogram.
 	void
 		MassSpecPlotWidget::integrateToRt(double lower, double upper)
@@ -751,7 +825,7 @@ namespace msXpSmineXpert
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -775,12 +849,24 @@ namespace msXpSmineXpert
 			// if this plot widget is NOT multi-graph. So, even if mp_massSpecDataSet
 			// is nullptr, there is no risk.
 			emit newTicChromatogram(mp_massSpecDataSet, "Calculating extracted ion chromatogram.",
-					localHistory, graph()->pen().color());
+					localHistory, m_plottingColor);
 		}
 
 
+	/*/js/
+	 * MassSpecPlotWidget.integrateToTicIntensity(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a TIC intensity single value
+	 * limiting the data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * The TIC intensity value is returned
+	 */
+
 	//! Perform a ranged [\p lower -- \p upper] data integration to a single TIC intensity value.
-	double	
+	double
 		MassSpecPlotWidget::integrateToTicIntensity(double lower, double upper)
 		{
 			if(m_isMultiGraph)
@@ -793,7 +879,7 @@ namespace msXpSmineXpert
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -886,7 +972,7 @@ namespace msXpSmineXpert
 
 
 	//! Return a MassSpectrum crafted from the plot data.
-	msXpSlibmass::MassSpectrum 
+	msXpSlibmass::MassSpectrum
 		MassSpecPlotWidget::massSpectrum(double lower, double upper)
 		{
 			msXpSlibmass::MassSpectrum massSpectrum;
@@ -929,6 +1015,16 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * MassSpecPlotWidget.newPlot(trace)
+	 *
+	 * Create a new MassSpecPlotWidget using <Trace> trace for the initialization
+	 * of the data.
+	 *
+	 * This function creates a new plot widget to display the <Trace> data.
+	 */
+
+	//! Create a new plot. This function is useful in the scripting environment.
 	void
 		MassSpecPlotWidget::newPlot()
 		{
@@ -942,7 +1038,7 @@ namespace msXpSmineXpert
 			QScriptContext *ctx = mainWindow->mp_scriptingWnd->m_scriptEngine.currentContext();
 
 			int argCount = ctx->argumentCount();
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "argCount:" << argCount;
 
 			if(argCount != 1)
@@ -954,7 +1050,7 @@ namespace msXpSmineXpert
 			QScriptValue arg = ctx->argument(0);
 			QVariant variant = arg.data().toVariant();
 
-			if(!variant.isValid())	
+			if(!variant.isValid())
 			{
 				ctx->throwError("Syntax error: function newPlot() takes one Trace object as argument.");
 				return;
@@ -964,16 +1060,10 @@ namespace msXpSmineXpert
 			{
 				msXpSlibmass::Trace trace(qscriptvalue_cast<msXpSlibmass::Trace>(arg));
 
-				// Get the color of the graph, so that we can print the stanza in
-				// the console window with the proper color.
-
-				QPen pen = graph()->pen();
-				QColor color = pen.color();
-
 				static_cast<MassSpecWnd *>(mp_parentWnd)->newPlot(this, mp_massSpecDataSet,
-						QVector<double>::fromList(trace.keyList()), 
+						QVector<double>::fromList(trace.keyList()),
 						QVector<double>::fromList(trace.valList()),
-						"Trace from scripting environment.", m_history, color);
+						"Trace from scripting environment.", m_history, m_plottingColor);
 			}
 			else
 			{
@@ -985,7 +1075,7 @@ namespace msXpSmineXpert
 
 
 	//! Key release event handler.
-	/*! 
+	/*!
 
 		Depending on the type of plot widget (multigraph or not), the actions
 		triggered in this function will differ.
@@ -1037,12 +1127,6 @@ namespace msXpSmineXpert
 						if(!iterGraph->visible())
 							continue;
 
-						// Get the color of the graph, so that we can print the stanza
-						// in
-						// the console window with the proper color.
-						QPen curPen = iterGraph->pen();
-						QColor curColor = curPen.color();
-
 						// Get a handle to the conventional plot widget of which this
 						// multi-graph plot widget's graph is a replica.
 						AbstractPlotWidget *massSpecPlotWidget =
@@ -1099,7 +1183,7 @@ namespace msXpSmineXpert
 								// qDebug() << __FILE__ << __LINE__
 								// << "Stanza: " << stanza;
 
-								emit recordAnalysisStanza(stanza, curColor);
+								emit recordAnalysisStanza(stanza, m_plottingColor);
 							}
 						}
 					}
@@ -1127,17 +1211,11 @@ namespace msXpSmineXpert
 					}
 					else
 					{
-						// Send the stanza to the console window, so that we can make a
-						// copy
+
+						// Send the stanza to the console window, so that we can make a copy
 						// paste.
 
-						// Get the color of the graph, so that we can print the stanza
-						// in
-						// the console window with the proper color.
-						QPen curPen = graph()->pen();
-						QColor curColor = curPen.color();
-
-						emit recordAnalysisStanza(stanza, curColor);
+						emit recordAnalysisStanza(stanza, m_plottingColor);
 					}
 				}
 
@@ -1446,11 +1524,6 @@ namespace msXpSmineXpert
 						if(!iterGraph->visible())
 							continue;
 
-						// Get the color of the graph, so that we can print the label in
-						// the console window with the proper color.
-						QPen curPen = iterGraph->pen();
-						QColor curColor = curPen.color();
-
 						// Get a handle to the conventional plot widget of which this
 						// multi-graph plot widget's graph is a replica.
 
@@ -1464,7 +1537,7 @@ namespace msXpSmineXpert
 
 						DataPlotWidgetRelation *p_rootRelation = Q_NULLPTR;
 						AbstractPlotWidget *ticChromPlotWidget =
-							mainWindow->m_dataPlotWidgetRelationer.rootPlotWidget(massSpecPlotWidget, 
+							mainWindow->m_dataPlotWidgetRelationer.rootPlotWidget(massSpecPlotWidget,
 									&p_rootRelation);
 
 						if(ticChromPlotWidget == nullptr)
@@ -1504,7 +1577,7 @@ namespace msXpSmineXpert
 								// Send the label to the console window, so that we can
 								// make a copy
 								// paste.
-								mainWindow->logConsoleMessage(label, curColor);
+								mainWindow->logConsoleMessage(label, m_plottingColor);
 							}
 						}
 					}
@@ -1536,13 +1609,7 @@ namespace msXpSmineXpert
 						// copy
 						// paste.
 
-						// Get the color of the graph, so that we can print the stanza
-						// in
-						// the console window with the proper color.
-						QPen curPen = graph()->pen();
-						QColor curColor = curPen.color();
-
-						mainWindow->logConsoleMessage(label, curColor);
+						mainWindow->logConsoleMessage(label, m_plottingColor);
 					}
 				}
 
@@ -1557,14 +1624,13 @@ namespace msXpSmineXpert
 		}
 
 
-	//! Return the arbitrary integration type that this plot widget can handle (IntegrationType::ANY_TO_MZ).
+	//! Return the arbitrary integration type that this plot widget can handle
 	int
 		MassSpecPlotWidget::arbitraryIntegrationType()
 		{
-			// FIXME
-			// Should not that be ????? 
-			return IntegrationType::ANY_TO_MZ;
+			return IntegrationType::MZ_TO_XXX;
 		}
 
 
+
 } // namespace msXpSmineXpert
