@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include <qmath.h>
 #include <omp.h>
 
@@ -70,14 +71,13 @@ namespace msXpSmineXpert
 			"Mass-spectrum" /*m_desc*/}
 	{
 		initialize();
-
-		show();
 	}
 
 
 	//! Destruct \c this MassSpecWnd instance.
 	MassSpecWnd::~MassSpecWnd()
 	{
+		writeSettings();
 	}
 
 
@@ -101,6 +101,8 @@ namespace msXpSmineXpert
 			// This widget will be appended to the others already set up in the base
 			// class.
 			statusBar()->addPermanentWidget(mp_chargeSpanSpinBox);
+
+			readSettings();
 		}
 
 
@@ -117,7 +119,7 @@ namespace msXpSmineXpert
 			// the true parameter in the function call below).
 
 			mp_multiGraphPlotWidget =
-				new MassSpecPlotWidget(this, this, m_name, m_desc, 
+				new MassSpecPlotWidget(this, m_name, m_desc,
 						Q_NULLPTR /* MassSpecDataSet * */, QString() /* fileName */,
 						true /* isMultiGraph */);
 
@@ -167,7 +169,7 @@ namespace msXpSmineXpert
 
 
 	//! Return the charge span value from the corresponding spin box widget.
-	int 
+	int
 		MassSpecWnd::chargeSpan() const
 		{
 			return mp_chargeSpanSpinBox->value();
@@ -191,9 +193,9 @@ namespace msXpSmineXpert
 
 		\return the allocated plot widget.
 
-*/	
+*/
 	MassSpecPlotWidget *
-		MassSpecWnd::addPlotWidget(const QVector<double> &keyVector, const QVector<double> &valVector, 
+		MassSpecWnd::addPlotWidget(const QVector<double> &keyVector, const QVector<double> &valVector,
 				const QString &desc,
 				MassSpecDataSet *massSpecDataSet, bool isMultiGraph)
 		{
@@ -207,7 +209,7 @@ namespace msXpSmineXpert
 				fileName = "NOT_SET";
 
 			MassSpecPlotWidget *widget = new MassSpecPlotWidget(
-					this, this, m_name, desc, massSpecDataSet, fileName, isMultiGraph);
+					this, m_name, desc, massSpecDataSet, fileName, isMultiGraph);
 
 			widget->setObjectName("monoGraphPlot");
 
@@ -219,10 +221,10 @@ namespace msXpSmineXpert
 			// "massSpec".
 
 			QString propName =
-				QString("massSpecPlot%1").arg(MassSpecWnd::widgetCounter);
-			QString alias = "lastMassSpecPlot";
+				QString("massSpecPlotWidget%1").arg(MassSpecWnd::widgetCounter);
+			QString alias = "lastMassSpecPlotWidget";
 
-			QString explanation = QString("Make available a mass spectrum plot under name \"%1\" "
+			QString explanation = QString("Make available a mass spectrum plot widget under name \"%1\" "
 					"with alias \"%2\".\n").arg(propName).arg(alias);
 
 			// Make the new plot widget globally available under the crafted property
@@ -231,7 +233,7 @@ namespace msXpSmineXpert
 			// tree widget as a child item of this.
 
 			QScriptValue scriptValue;
-			mainWindow->mp_scriptingWnd->publishQObject(widget, this /* parent object */, 
+			mainWindow->mp_scriptingWnd->publishQObject(widget, this /* parent object */,
 					propName, alias, explanation, QString() /* help string */, scriptValue);
 
 			++MassSpecWnd::widgetCounter;
@@ -246,14 +248,13 @@ namespace msXpSmineXpert
 
 			// Let's see if one color was set as the desired color, otherwise use
 			// black.
-			QPen currentPen = widget->graph()->pen();
 
-			if(!m_wishedColor.isValid())
-				currentPen.setColor(Qt::black);
-			else
-				currentPen.setColor(m_wishedColor);
+			QColor localColor = Qt::black;
 
-			widget->graph()->setPen(currentPen);
+			if(m_wishedColor.isValid())
+				localColor = m_wishedColor;
+
+			widget->setPlottingColor(localColor);
 
 			widget->rescaleAxes(true /*only enlarge*/);
 			widget->replot();
@@ -266,7 +267,9 @@ namespace msXpSmineXpert
 
 			QCPGraph *newGraph = mp_multiGraphPlotWidget->addGraph();
 			newGraph->setData(keyVector, valVector);
-			newGraph->setPen(currentPen);
+
+			// Duplicate the same widget pen for the multigraph graph.
+			newGraph->setPen(widget->graph()->pen());
 			mp_multiGraphPlotWidget->rescaleAxes(true /*only enlarge*/);
 			mp_multiGraphPlotWidget->replot();
 
@@ -314,6 +317,9 @@ namespace msXpSmineXpert
 
 			m_wishedColor = color;
 
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "m_wishedColor:" << color;
+
 			// Make sure the cancel button is visible.
 			mp_statusBarCancelPushButton->setVisible(true);
 
@@ -339,9 +345,9 @@ namespace msXpSmineXpert
 			//qDebug() << "At this point the integration is finished. Desetup the feedback.";
 
 			//De-setup the feedback.
-			QTimer::singleShot(2000, this, [=]() 
-					{ 
-					setupFeedback("", -1, -1, -1, LogType::LOG_TO_STATUS_BAR); 
+			QTimer::singleShot(2000, this, [=]()
+					{
+					setupFeedback("", -1, -1, -1, LogType::LOG_TO_STATUS_BAR);
 					}
 					);
 
