@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 /////////////////////// Qt includes
 #include <QObject>
 #include <QDebug>
@@ -56,7 +57,7 @@ namespace msXpSmineXpert
 	//! Construct an OpenSpectraDlg instance.
 	OpenSpectraDlg::OpenSpectraDlg(QWidget *parent) : QDialog{parent}
 	{
-		if(parent == nullptr)
+		if(parent == Q_NULLPTR)
 			qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 		m_ui.setupUi(this);
@@ -87,6 +88,7 @@ namespace msXpSmineXpert
 		{
 			QSettings settings;
 			settings.setValue("OpenSpectraDlg_geometry", saveGeometry());
+			settings.setValue("OpenSpectraDlg_visible", isVisible());
 		}
 
 
@@ -96,6 +98,9 @@ namespace msXpSmineXpert
 		{
 			QSettings settings;
 			restoreGeometry(settings.value("OpenSpectraDlg_geometry").toByteArray());
+
+			bool wasVisible = settings.value("OpenSpectraDlg_visible").toBool();
+			setVisible(wasVisible);
 		}
 
 
@@ -133,7 +138,7 @@ namespace msXpSmineXpert
 	QListWidgetItem *
 		OpenSpectraDlg::addOpenSpectrum(MassSpecDataSet *dataSet, QColor color)
 		{
-			if(dataSet == nullptr)
+			if(dataSet == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			// Create an icon that is simply a rectangle filled with color.
@@ -161,7 +166,7 @@ namespace msXpSmineXpert
 			// destroyed.
 			AbstractPlotWidget *widget =
 				dynamic_cast<AbstractPlotWidget *>(dataSet->parent());
-			if(widget == nullptr)
+			if(widget == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			m_listItemTicPlotMap.insert(item, widget);
@@ -209,7 +214,7 @@ namespace msXpSmineXpert
 					return;
 
 				//qDebug() << __FILE__ << __LINE__ << "selected item count"
-					//<< selectedItemList.size();
+				//<< selectedItemList.size();
 
 				for(int iter = 0; iter < selectedItemList.size(); ++iter)
 				{
@@ -330,7 +335,7 @@ namespace msXpSmineXpert
 
 				// Let the user select the color.
 
-				QColor newColor = QColorDialog::getColor(Qt::white, nullptr, 
+				QColor newColor = QColorDialog::getColor(Qt::white, Q_NULLPTR,
 						QString("Please select a new color for the plot"));
 
 				if(!newColor.isValid())
@@ -340,13 +345,13 @@ namespace msXpSmineXpert
 
 				MainWindow *mainWindow = static_cast<MainWindow *>(parent());
 
-				QColor *useColor = nullptr;
+				QColor *useColor = Q_NULLPTR;
 
 				// Let's see if the color is already used.
 
 				if(mainWindow->isPlottingColorUsed(newColor))
 				{
-					QMessageBox::StandardButton ret = 
+					QMessageBox::StandardButton ret =
 						QMessageBox::warning
 						(this, "mineXpert - Open spectra",
 						 "This color is already used. Go on ?",
@@ -365,7 +370,7 @@ namespace msXpSmineXpert
 
 				const DataPlotWidgetRelationer &relationer = mainWindow->m_dataPlotWidgetRelationer;
 				QList<DataPlotWidgetRelation *> relationList;
-				AbstractPlotWidget *recoloringPlotWidget = nullptr;
+				AbstractPlotWidget *recoloringPlotWidget = Q_NULLPTR;
 
 				relationer.getRelationsByOriginRecursive(plotWidget, &relationList);
 
@@ -379,21 +384,18 @@ namespace msXpSmineXpert
 					// A DataPlotWidgetRelation relates plot widgets together.
 
 					recoloringPlotWidget = const_cast<AbstractPlotWidget *>(relation->originPlotWidget());
-					if(recoloringPlotWidget != nullptr)
-						recoloringPlotWidget->changeColor(newColor);
+					if(recoloringPlotWidget != Q_NULLPTR)
+						recoloringPlotWidget->changePlottingColor(newColor);
 
 					recoloringPlotWidget = const_cast<AbstractPlotWidget *>(relation->targetPlotWidget());
-					if(recoloringPlotWidget != nullptr)
-						recoloringPlotWidget->changeColor(newColor);
+					if(recoloringPlotWidget != Q_NULLPTR)
+						recoloringPlotWidget->changePlottingColor(newColor);
 				}
 
 				// And now finish by recoloring the color vignette of the corresponding
 				// item in the QListWidget and the matching file name text.
 				QListWidgetItem *item = selectedItemList.first();
 
-				qDebug() << __FILE__ << __LINE__
-					<< "here";
-
 				// Create an icon that is simply a rectangle filled with color.
 				QPixmap rectangle(15, 15);
 				rectangle.fill(newColor);
@@ -447,24 +449,10 @@ namespace msXpSmineXpert
 			}
 			else if(text == "Close all but selected")
 			{
-				int itemCount = m_ui.openSpectraListWidget->count();
 
 				// We cannot remove the widget inside the for loop below because when we
 				// destroy a widget, the QListWidgetItem is destroyed.
-				QList<AbstractPlotWidget *> plotWidgetList;
-
-				for(int iter = 0; iter < itemCount; ++iter)
-				{
-					QListWidgetItem *curItem = m_ui.openSpectraListWidget->item(iter);
-
-					if(!curItem->isSelected())
-					{
-						AbstractPlotWidget *widget =
-							m_listItemTicPlotMap.value(curItem);
-
-						plotWidgetList.append(widget);
-					}
-				}
+				QList<AbstractPlotWidget *> plotWidgetList = unselectedPlotWidgets();
 
 				// Now that we have a list of plot widgets to remove we can iterate in
 				// the list and destroy the widgets sequentially.
@@ -490,12 +478,12 @@ namespace msXpSmineXpert
 	bool
 		OpenSpectraDlg::isPlotWidgetSelected(AbstractPlotWidget *widget)
 		{
-			if(widget == nullptr)
+			if(widget == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			QListWidgetItem *item = m_listItemTicPlotMap.key(widget);
 
-			if(item == nullptr)
+			if(item == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d -- %s. "
 						"Cannot be that obtained pointer is nullptr. Programming error."
 						"Program aborted.",
@@ -537,17 +525,17 @@ namespace msXpSmineXpert
 
 			QList<QListWidgetItem *> itemList =
 				m_ui.openSpectraListWidget->selectedItems();
-			QListWidgetItem *item = nullptr;
+			QListWidgetItem *item = Q_NULLPTR;
 
 			if(itemList.size() == 0)
 			{
-				widget = nullptr;
+				widget = Q_NULLPTR;
 				return -1;
 			}
 
 			item = itemList.at(0);
 
-			if(widget != nullptr)
+			if(widget != Q_NULLPTR)
 				widget = m_listItemTicPlotMap.value(item);
 
 			return m_ui.openSpectraListWidget->row(item);
@@ -570,12 +558,12 @@ namespace msXpSmineXpert
 			QList<QListWidgetItem *> itemList =
 				m_ui.openSpectraListWidget->selectedItems();
 
-			AbstractPlotWidget *widget = nullptr;
-			QListWidgetItem * item = nullptr;
+			AbstractPlotWidget *widget = Q_NULLPTR;
+			QListWidgetItem * item = Q_NULLPTR;
 
 			if(itemList.size() == 0)
 			{
-				if(index != nullptr)
+				if(index != Q_NULLPTR)
 					*index = -1;
 
 				return widget;
@@ -583,7 +571,7 @@ namespace msXpSmineXpert
 
 			item = itemList.at(0);
 
-			if(index != nullptr)
+			if(index != Q_NULLPTR)
 				*index = m_ui.openSpectraListWidget->row(item);
 
 			return m_listItemTicPlotMap.value(item);
@@ -595,10 +583,10 @@ namespace msXpSmineXpert
 		OpenSpectraDlg::plotWidgetAt(int index)
 		{
 			if(index < 0)
-				return nullptr;
+				return Q_NULLPTR;
 
 			if(index >= m_ui.openSpectraListWidget->count())
-				return nullptr;
+				return Q_NULLPTR;
 
 			// Get the plot widget pointer on the basis of the list widet item at
 			// index.
@@ -620,10 +608,10 @@ namespace msXpSmineXpert
 		OpenSpectraDlg::selectPlotWidget(int index)
 		{
 			if(index < 0)
-				return nullptr;
+				return Q_NULLPTR;
 
 			if(index >= m_ui.openSpectraListWidget->count())
-				return nullptr;
+				return Q_NULLPTR;
 
 			// Get the plot widget pointer on the basis of the list widet item at
 			// index.
@@ -646,7 +634,7 @@ namespace msXpSmineXpert
 	int
 		OpenSpectraDlg::selectPlotWidget(AbstractPlotWidget *widget)
 		{
-			if(widget == nullptr)
+			if(widget == Q_NULLPTR)
 				return -1;
 
 			QListWidgetItem *item = m_listItemTicPlotMap.key(widget);
@@ -667,10 +655,10 @@ namespace msXpSmineXpert
 		OpenSpectraDlg::togglePlotWidget(int index)
 		{
 			if(index < 0)
-				return nullptr;
+				return Q_NULLPTR;
 
 			if(index >= m_ui.openSpectraListWidget->count())
-				return nullptr;
+				return Q_NULLPTR;
 
 			// Get the plot widget pointer on the basis of the list widet item at
 			// index.
@@ -693,7 +681,7 @@ namespace msXpSmineXpert
 	int
 		OpenSpectraDlg::togglePlotWidget(AbstractPlotWidget *widget)
 		{
-			if(widget == nullptr)
+			if(widget == Q_NULLPTR)
 				return -1;
 
 			QListWidgetItem *item = m_listItemTicPlotMap.key(widget);
@@ -735,12 +723,12 @@ namespace msXpSmineXpert
 
 		The plot widget corresponding to item at \p index is returned.
 
-		*/
+*/
 	AbstractPlotWidget *
 		OpenSpectraDlg::deselectAllPlotWidgetsButOne(int index)
 		{
-			AbstractPlotWidget *widget = nullptr;
-			QListWidgetItem * item = nullptr;
+			AbstractPlotWidget *widget = Q_NULLPTR;
+			QListWidgetItem * item = Q_NULLPTR;
 
 			for(int iter = 0; iter < m_ui.openSpectraListWidget->count(); ++iter)
 			{
@@ -765,14 +753,14 @@ namespace msXpSmineXpert
 
 		The index of the list widget item corresponding to \p widget is returned.
 
-		*/
+*/
 	int
 		OpenSpectraDlg::deselectAllPlotWidgetsButOne(AbstractPlotWidget *widget)
 		{
-			if(widget == nullptr)
+			if(widget == Q_NULLPTR)
 				return -1;
 
-			AbstractPlotWidget *curWidget = nullptr;
+			AbstractPlotWidget *curWidget = Q_NULLPTR;
 			int itemIndex = -1;
 
 			for(int iter = 0; iter < m_ui.openSpectraListWidget->count(); ++iter)
@@ -793,8 +781,51 @@ namespace msXpSmineXpert
 		}
 
 
+	QList<AbstractPlotWidget *>
+		OpenSpectraDlg::selectedPlotWidgets() const
+		{
+			int itemCount = m_ui.openSpectraListWidget->count();
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "itemCount =" << itemCount;
+
+			QList<AbstractPlotWidget *> plotWidgetList;
+
+			for(int iter = 0; iter < itemCount; ++iter)
+			{
+				QListWidgetItem *curItem = m_ui.openSpectraListWidget->item(iter);
+
+				if(curItem->isSelected())
+					plotWidgetList.append(m_listItemTicPlotMap.value(curItem));
+			}
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Plot widget list size:" << plotWidgetList.size();
+
+			return plotWidgetList;
+		}
+
+
+	QList<AbstractPlotWidget *>
+		OpenSpectraDlg::unselectedPlotWidgets() const
+		{
+			int itemCount = m_ui.openSpectraListWidget->count();
+			QList<AbstractPlotWidget *> plotWidgetList;
+
+			for(int iter = 0; iter < itemCount; ++iter)
+			{
+				QListWidgetItem *curItem = m_ui.openSpectraListWidget->item(iter);
+
+				if(!curItem->isSelected())
+					plotWidgetList.append(m_listItemTicPlotMap.value(curItem));
+			}
+
+			return plotWidgetList;
+		}
+
+
 	//! Handle keyboard key release events.
-	/*! 
+	/*!
 
 		- T toggle visibility of the multigraph plot graph corresponding to selected
 		list widget items.
@@ -805,7 +836,7 @@ namespace msXpSmineXpert
 		- S show the multigraph plot graph corresponding to selected list widget
 		items.
 
-		*/
+*/
 	void
 		OpenSpectraDlg::keyReleaseEvent(QKeyEvent *event)
 		{
