@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 ///////////////////////////// Qt include
 
 
@@ -50,6 +51,15 @@ namespace msXpSmineXpert
 {
 
 
+	/*/js/ Class: TicChromPlotWidget
+	 * <comment>This class cannot be instantiated with the new operator. Objects
+	 * of this class are made available to the scripting environment under the
+	 * form of object variable names ticChromPlotWidget[index], with [index]
+	 * being 0 for the first object created and being incremented each time a new
+	 * TicChromPlotWidget is created.</comment>
+	 */
+
+
 	//! Construct an initialized TicChromPlotWidget instance.
 	/*!
 
@@ -58,8 +68,6 @@ namespace msXpSmineXpert
 
 		\param parent parent widget.
 
-		\param parentWnd parent window.
-
 		\param name name of the module.
 
 		\param desc description of the wiget type.
@@ -71,20 +79,20 @@ namespace msXpSmineXpert
 		\param isMultiGraph tells if the widget to be instanciated is for displaying
 		multiple graphs.
 
-*/ 
-	TicChromPlotWidget::TicChromPlotWidget(QWidget *parent, QWidget *parentWnd,
+*/
+	TicChromPlotWidget::TicChromPlotWidget(QWidget *parent,
 			const QString &name, const QString &desc,
 			MassSpecDataSet *massSpecDataSet, const QString &fileName,
 			bool isMultiGraph)
-		: AbstractPlotWidget{parent, parentWnd, 
-			name, desc, 
+		: AbstractPlotWidget{parent,
+			name, desc,
 			massSpecDataSet, fileName, isMultiGraph}
 	{
-		AbstractPlotWidget::registerQtKeyCode(Qt::Key_S, 
+		registerQtKeyCode(Qt::Key_S,
 				"Press S while right mouse click-drag to integrate to a mass spectrum");
-		AbstractPlotWidget::registerQtKeyCode(Qt::Key_D,
+		registerQtKeyCode(Qt::Key_D,
 				"Press D while right mouse click-drag to integrate to a drift spectrum");
-		AbstractPlotWidget::registerQtKeyCode(Qt::Key_I,
+		registerQtKeyCode(Qt::Key_I,
 				"Press I while right mouse click-drag to integrate to a TIC intensity value");
 
 		// Give the axes some labels:
@@ -100,6 +108,28 @@ namespace msXpSmineXpert
 	}
 
 
+	QMenu *
+		TicChromPlotWidget::createContextMenu()
+		{
+			AbstractPlotWidget::createContextMenu();
+
+			// We can only compute statistics on the basis of the current plot range
+			// when the whole mass data set is in memory. Otherwise that would be too
+			// lengthy.
+			if(!mp_massSpecDataSet->isStreamed())
+			{
+				mpa_contextMenu->addSeparator();
+				QAction *action = mpa_contextMenu->addAction("&Statistics");
+				connect(action,
+						&QAction::triggered,
+						this,
+						&TicChromPlotWidget::statistics);
+			}
+
+			return mpa_contextMenu;
+		}
+
+
 	//! Craft a string representing the data of the peak that was analysed.
 	/*!
 
@@ -165,7 +195,7 @@ namespace msXpSmineXpert
 			// m_dataFormatStringSpecifHash.
 
 			DataFormatStringSpecif *specif =
-				analPrefs->m_dataFormatStringSpecifHash.value(FormatType::DRIFT_SPEC);
+				analPrefs->m_dataFormatStringSpecifHash.value(FormatType::TIC_CHROM);
 
 			if(specif == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
@@ -289,6 +319,13 @@ namespace msXpSmineXpert
 						prevChar = ' ';
 						continue;
 					}
+					if(curChar == 'I')
+					{
+						stanza += QString("%1").arg(m_lastI, 0, 'g', 3);
+
+						prevChar = ' ';
+						continue;
+					}
 
 					// At this point the '%' is not followed by any special character
 					// above, so we skip them both from the text. If the '%' is to be
@@ -312,6 +349,24 @@ namespace msXpSmineXpert
 		}
 
 
+	void
+		TicChromPlotWidget::setFocus()
+		{
+			AbstractPlotWidget::setFocus();
+
+			AbstractMultiPlotWnd *parentWnd = dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+
+			if(parentWnd == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+
+			parentWnd->updateMzIntegrationParams(m_history.mzIntegrationParams(),
+					m_plottingColor);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "params:" << m_history.mzIntegrationParams().asText();
+		}
+
+
 	//! Handler for the mouse release event.
 	/*!
 
@@ -324,8 +379,8 @@ namespace msXpSmineXpert
 		TicChromPlotWidget::mouseReleaseHandledEvent(QMouseEvent *event)
 		{
 
-			// This function is called by 
-			// AbstractPlotWidget::mouseReleaseHandler(QMouseEvent *event) when 
+			// This function is called by
+			// AbstractPlotWidget::mouseReleaseHandler(QMouseEvent *event) when
 			// the key that is pressed is listed in the m_regQtKeyCodeList.
 
 			if(m_isMultiGraph)
@@ -336,7 +391,28 @@ namespace msXpSmineXpert
 
 			if(m_pressedKeyCode == Qt::Key_S && event->button() == Qt::RightButton)
 			{
-				integrateToMz(xRangeMin(), xRangeMax());
+				if(!m_isDragging)
+				{
+					// The user simply clicked on the plot with the right mouse button
+					// while maintaining the S key pressed. Integrate to mz only the
+					// current single retention time. But we need to make sure that we get
+					// into the pseudo range the read data point left of the clicked
+					// location.
+
+					QCPRange keyRange;
+
+					bool res = findIntegrationLowerRangeForKey(m_lastMousedPlotPoint.x(), &keyRange);
+
+					if(!res)
+						qFatal("Fatal error at %s@%d -- %s(). "
+								"Programming error."
+								"Program aborted.",
+								__FILE__, __LINE__, __FUNCTION__);
+
+					integrateToMz(keyRange.lower, keyRange.upper);
+				}
+				else
+					integrateToMz(xRangeMin(), xRangeMax());
 			}
 
 			if(m_pressedKeyCode == Qt::Key_D && event->button() == Qt::RightButton)
@@ -351,6 +427,18 @@ namespace msXpSmineXpert
 		}
 
 
+	/*/js/
+	 * TicChromPlotWidget.integrateToMz(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a mass spectrum limiting the
+	 * data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Perform a ranged [\p lower -- \p upper] data integration to a mass spectrum.
 	void
 		TicChromPlotWidget::integrateToMz(double lower, double upper)
@@ -359,7 +447,7 @@ namespace msXpSmineXpert
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -372,7 +460,15 @@ namespace msXpSmineXpert
 				rangeEnd = xAxis->range().upper;
 			}
 
+			// The history has by definition the bin specifications, which we'll use
+			// to perform the integrations. This is only true when integrating to
+			// MZ, because the result will be a mass spectrum plot. When integrating
+			// to DT or TICintensity, then no problem: we are working all along with
+			// TIC values and TIC values do not need bins to be properly displayed
+			// into a mass spectrum.
+
 			History localHistory = m_history;
+
 			HistoryItem *histItem = new HistoryItem;
 			histItem->newIntegrationRange(IntegrationType::RT_TO_MZ, rangeStart, rangeEnd);
 
@@ -380,19 +476,31 @@ namespace msXpSmineXpert
 			localHistory.appendHistoryItem(histItem);
 
 			emit newMassSpectrum(mp_massSpecDataSet, "Calculating mass spectrum.",
-					localHistory, graph()->pen().color());
+					localHistory, m_plottingColor);
 		}
 
 
+	/*/js/
+	 * TicChromPlotWidget.integrateToDt(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a drift spectrum limiting the
+	 * data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * This function creates a new plot widget to display the obtained results.
+	 */
+
 	//! Perform a ranged [\p lower -- \p upper] data integration to a drift spectrum.
 	void
 		TicChromPlotWidget::integrateToDt(double lower, double upper)
-		{	
+		{
 			// Get the pointer to the x-axis (key axis).
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -412,10 +520,22 @@ namespace msXpSmineXpert
 			localHistory.appendHistoryItem(histItem);
 
 			emit newDriftSpectrum(mp_massSpecDataSet, "Calculating drift spectrum.",
-					localHistory, graph()->pen().color());
+					localHistory, m_plottingColor);
 		}
 
 
+	/*/js/
+	 * TicChromPlotWidget.integrateToTicIntensity(lower, upper)
+	 *
+	 * Starts an integration of mass spectra data to a TIC intensity single value
+	 * limiting the data range specified with the numerical arguments.
+	 *
+	 * lower, upper: retention time values limiting the integration. If no values
+	 * are provided, there is no limitation to the integration.
+	 *
+	 * The TIC intensity value is returned
+	 */
+
 	//! Perform a ranged [\p lower -- \p upper] data integration to a single TIC intensity value.
 	double
 		TicChromPlotWidget::integrateToTicIntensity(double lower, double upper)
@@ -430,7 +550,7 @@ namespace msXpSmineXpert
 
 			QCPAxis *xAxis = graph()->keyAxis();
 
-			double rangeStart = lower; 
+			double rangeStart = lower;
 			if(qIsNaN(rangeStart))
 			{
 				rangeStart = xAxis->range().lower;
@@ -522,6 +642,105 @@ namespace msXpSmineXpert
 		}
 
 
+
+	//! Perform a ranged [\p lower -- \p upper] data integration to a XIC chromatogram.
+	/*
+
+		 This process is not equivalent to an integration to RT because we do not take
+		 into account the History of the current plot widget. Indeed, the integration
+		 is performed without any filtering from the initial mass spectrometry data.
+
+*/
+	void
+		TicChromPlotWidget::integrateToXic(double lower, double upper)
+		{
+
+			// The history list needs to be empty because we do not want any filtering of
+			// the data apart from the two parameters to this function. However, we
+			// want to keep the mass spectral data integration parameters.
+
+			History localHistory = m_history;
+			localHistory.freeList();
+
+			HistoryItem *histItem = new HistoryItem;
+			histItem->newIntegrationRange(IntegrationType::FILE_TO_XIC, lower, upper);
+
+			// Aggreate the new item to the local history copy we made.
+			localHistory.appendHistoryItem(histItem);
+
+			// The signal is emitted, but the connection to the receiver exists only
+			// if this plot widget is NOT multi-graph. So, even if mp_massSpecDataSet
+			// is nullptr, there is no risk.
+			emit newTicChromatogram(mp_massSpecDataSet, "Calculating extracted ion chromatogram.",
+					localHistory, m_plottingColor);
+		}
+
+
+	//! Compute mass spectra statistics on the spectra in the range.
+	void
+		TicChromPlotWidget::statistics()
+		{
+			if(m_isMultiGraph)
+				// We cannot perform the action, because this plot widget has not mass
+				// spec data set pointer, it is a graph-only plot widget to display all
+				// the graphs overlaid.
+				return ;
+
+			// Get the pointer to the x-axis (key axis).
+
+			QCPAxis *xAxis = graph()->keyAxis();
+
+			double rangeStart = xAxis->range().lower;
+
+			double rangeEnd = xAxis->range().upper;
+
+			History localHistory = m_history;
+			HistoryItem *histItem = new HistoryItem;
+			histItem->newIntegrationRange(IntegrationType::RT_TO_ANY, rangeStart, rangeEnd);
+			localHistory.appendHistoryItem(histItem);
+
+			QApplication::setOverrideCursor(QCursor(Qt::BusyCursor));
+
+			// Sanity check. It is not possible that this plot widget is not
+			// multi-graph and that at the same time, the mass spec data set pointer
+			// is nullptr.
+
+			if(mp_massSpecDataSet == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Programming error."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			// Make sure we compute the statistics according to the parameters in
+			// localHistory.
+			MassSpecDataStats statistics = mp_massSpecDataSet->makeStatistics(localHistory);
+
+			QString text = statistics.asText();
+
+			QApplication::restoreOverrideCursor();
+
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "Computed statistics\n" << text;
+
+			TicChromWnd *parentWnd = static_cast<TicChromWnd *>(mp_parentWnd);
+			MainWindow *mainWindow = static_cast<MainWindow *>(parentWnd->parent());
+
+			mainWindow->logConsoleMessage("\n------------------------------------------------\n");
+			mainWindow->logConsoleMessage(text, m_plottingColor);
+			mainWindow->logConsoleMessage("------------------------------------------------\n");
+		}
+
+
+	/*/js/
+	 * TicChromPlotWidget.newPlot(trace)
+	 *
+	 * Create a new TicChromPlotWidget using <Trace> trace for the initialization
+	 * of the data.
+	 *
+	 * This function creates a new plot widget to display the <Trace> data.
+	 */
+
+	//! Create a new plot. This function is useful in the scripting environment.
 	void
 		TicChromPlotWidget::newPlot()
 		{
@@ -535,7 +754,7 @@ namespace msXpSmineXpert
 			QScriptContext *ctx = mainWindow->mp_scriptingWnd->m_scriptEngine.currentContext();
 
 			int argCount = ctx->argumentCount();
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "argCount:" << argCount;
 
 			if(argCount != 1)
@@ -547,7 +766,7 @@ namespace msXpSmineXpert
 			QScriptValue arg = ctx->argument(0);
 			QVariant variant = arg.data().toVariant();
 
-			if(!variant.isValid())	
+			if(!variant.isValid())
 			{
 				ctx->throwError("Syntax error: function newPlot() takes one Trace object as argument.");
 				return;
@@ -560,13 +779,10 @@ namespace msXpSmineXpert
 				// Get the color of the graph, so that we can print the stanza in
 				// the console window with the proper color.
 
-				QPen pen = graph()->pen();
-				QColor color = pen.color();
-
 				static_cast<TicChromWnd *>(mp_parentWnd)->newPlot(this, mp_massSpecDataSet,
-						QVector<double>::fromList(trace.keyList()), 
+						QVector<double>::fromList(trace.keyList()),
 						QVector<double>::fromList(trace.valList()),
-						"Trace from scripting environment.", m_history, color);
+						"Trace from scripting environment.", m_history, m_plottingColor);
 			}
 			else
 			{
@@ -576,8 +792,9 @@ namespace msXpSmineXpert
 		}
 
 
+
 	//! Key release event handler.
-	/*! 
+	/*!
 
 		Depending on the type of plot widget (multigraph or not), the actions
 		triggered in this function will differ.
@@ -630,12 +847,6 @@ namespace msXpSmineXpert
 						if(!iterGraph->visible())
 							continue;
 
-						// Get the color of the graph, so that we can print the stanza in
-						// the console window with the proper color.
-
-						QPen curPen = iterGraph->pen();
-						QColor curColor = curPen.color();
-
 						// Get a handle to the conventional plot widget of which this
 						// multi-graph plot widget's graph is a replica.
 
@@ -680,7 +891,7 @@ namespace msXpSmineXpert
 							}
 							else
 							{
-								emit recordAnalysisStanza(stanza, curColor);
+								emit recordAnalysisStanza(stanza, m_plottingColor);
 							}
 						}
 					}
@@ -708,17 +919,11 @@ namespace msXpSmineXpert
 					}
 					else
 					{
-						// Send the stanza to the console window, so that we can make a
-						// copy
+
+						// Send the stanza to the console window, so that we can make a copy
 						// paste.
 
-						// Get the color of the graph, so that we can print the stanza
-						// in
-						// the console window with the proper color.
-						QPen curPen = graph()->pen();
-						QColor curColor = curPen.color();
-
-						emit recordAnalysisStanza(stanza, curColor);
+						emit recordAnalysisStanza(stanza, m_plottingColor);
 					}
 				}
 
@@ -765,10 +970,6 @@ namespace msXpSmineXpert
 						if(!iterGraph->visible())
 							continue;
 
-						// Get the color of the graph, so that we can print the label in
-						// the console window with the proper color.
-						QPen curPen = iterGraph->pen();
-						QColor curColor = curPen.color();
 
 						// Get a handle to the conventional plot widget of which this
 						// multi-graph plot widget's graph is a replica.
@@ -795,8 +996,8 @@ namespace msXpSmineXpert
 						if(mainWindow->mp_openSpectraDlg->isPlotWidgetSelected(
 									ticChromPlotWidget))
 						{
-							// We can craft the label text to send to the console
-							// window, and
+
+							// We can craft the label text to send to the console window, and
 							// that, with the proper color.
 
 							label = QString("(%1,%2)\n")
@@ -815,7 +1016,7 @@ namespace msXpSmineXpert
 								// Send the label to the console window, so that we can
 								// make a copy
 								// paste.
-								mainWindow->logConsoleMessage(label, curColor);
+								mainWindow->logConsoleMessage(label, m_plottingColor);
 							}
 						}
 					}
@@ -842,17 +1043,11 @@ namespace msXpSmineXpert
 					}
 					else
 					{
-						// Send the label to the console window, so that we can make a
-						// copy
+
+						// Send the label to the console window, so that we can make a copy
 						// paste.
 
-						// Get the color of the graph, so that we can print the stanza
-						// in
-						// the console window with the proper color.
-						QPen curPen = graph()->pen();
-						QColor curColor = curPen.color();
-
-						mainWindow->logConsoleMessage(label, curColor);
+						mainWindow->logConsoleMessage(label, m_plottingColor);
 					}
 				}
 
@@ -868,13 +1063,84 @@ namespace msXpSmineXpert
 		}
 
 
-	//! Return the arbitrary integration type that this plot widget can handle (IntegrationType::ANY_TO_RT).
-	int 
-		TicChromPlotWidget::arbitraryIntegrationType()
+	void
+		TicChromPlotWidget::mousePressHandler(QMouseEvent *event)
 		{
-			return IntegrationType::ANY_TO_RT;
+			AbstractPlotWidget::mousePressHandler(event);
 		}
 
-} 
 
+	void
+		TicChromPlotWidget::jsSetMzIntegrationParams()
+		{
+			if(m_isMultiGraph)
+				return;
+
+			MainWindow *mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+			QScriptContext *ctx = mainWindow->mp_scriptingWnd->m_scriptEngine.currentContext();
+
+			int argCount = ctx->argumentCount();
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "argCount:" << argCount;
+
+			if(argCount != 1)
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+
+			QScriptValue arg = ctx->argument(0);
+			QVariant variant = arg.data().toVariant();
+
+			if(!variant.isValid())
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+
+			if(variant.userType() == QMetaType::type("msXpSmineXpert::MzIntegrationParams"))
+			{
+				MzIntegrationParams params(qscriptvalue_cast<msXpSmineXpert::MzIntegrationParams>(arg));
+				AbstractPlotWidget::setMzIntegrationParams(params);
+
+				// Now update the values in the MzIntegrationParamsWnd.
+
+				AbstractMultiPlotWnd *parentWnd = dynamic_cast<AbstractMultiPlotWnd *>(mp_parentWnd);
+
+				if(parentWnd == Q_NULLPTR)
+					qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+
+				parentWnd->updateMzIntegrationParams(m_history.mzIntegrationParams(),
+						m_plottingColor);
+			}
+			else
+			{
+				ctx->throwError("Syntax error: function setMzIntegrationParams takes one MzIntegrationParams object as argument.");
+				return;
+			}
+		}
+
+
+	QScriptValue
+		TicChromPlotWidget::jsMzIntegrationParams() const
+		{
+			MainWindow *mainWindow = static_cast<MainWindow *>(mp_parentWnd->parent());
+
+			if(m_isMultiGraph)
+				return mainWindow->mp_scriptingWnd->m_scriptEngine.toScriptValue(MzIntegrationParams());
+
+			return mainWindow->mp_scriptingWnd->m_scriptEngine.toScriptValue(m_history.mzIntegrationParams());
+		}
+
+
+	//! Return the arbitrary integration type that this plot widget can handle
+	int
+		TicChromPlotWidget::arbitraryIntegrationType()
+		{
+			return IntegrationType::RT_TO_XXX;
+		}
+
+
+
+}
 // namespace msXpSmineXpert
