@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 /////////////////////// Qt includes
 #include <QPushButton>
 
@@ -46,6 +47,7 @@
 #include <minexpert/nongui/MassSpecDataFileLoaderDx.hpp>
 #include <minexpert/nongui/MassSpecDataFileLoaderMs1.hpp>
 #include <minexpert/nongui/MassDataIntegrator.hpp>
+#include <minexpert/nongui/MassSpecDataStats.hpp>
 
 
 #include <omp.h>
@@ -81,6 +83,7 @@ namespace msXpSmineXpert
 	//! Destruct \c this TicChromWnd instance.
 	TicChromWnd::~TicChromWnd()
 	{
+		writeSettings();
 	}
 
 
@@ -89,8 +92,7 @@ namespace msXpSmineXpert
 		TicChromWnd::initialize()
 		{
 			initializePlotRegion();
-
-			show();
+			readSettings();
 		}
 
 
@@ -107,11 +109,15 @@ namespace msXpSmineXpert
 			// the true parameter in the function call below.
 
 			mp_multiGraphPlotWidget =
-				new TicChromPlotWidget(this, this, m_name, m_desc, Q_NULLPTR /* MassSpecDataSet * */,
+				new TicChromPlotWidget(this, m_name, m_desc, Q_NULLPTR /* MassSpecDataSet * */,
 						QString() /* fileName */, true /* isMultiGraph */);
 			mp_multiGraphPlotWidget->setObjectName("multiGraphPlot");
 
 
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "Created new multigraph TIC plot widget:"
+				<< mp_multiGraphPlotWidget << "window:" << this;
+
 			// We need to remove the single graph that was created upon creation of
 			// the QCustomPlot widget because we'll later add graphs, and we want to
 			// have a proper correlation between the index of the graphs and their
@@ -172,12 +178,12 @@ namespace msXpSmineXpert
 
 		\return the allocated plot widget.
 
-*/	
+*/
 	TicChromPlotWidget *
 		TicChromWnd::addPlotWidget(const QVector<double> &keyVector,
-				const QVector<double> &valVector, 
+				const QVector<double> &valVector,
 				const QString &desc,
-				MassSpecDataSet *massSpecDataSet, 
+				MassSpecDataSet *massSpecDataSet,
 				bool isMultiGraph)
 		{
 			MainWindow *mainWindow = static_cast<MainWindow *>(parent());
@@ -187,8 +193,8 @@ namespace msXpSmineXpert
 			if(massSpecDataSet != Q_NULLPTR)
 				fileName = massSpecDataSet->fileName();
 
-			TicChromPlotWidget *widget = 
-				new TicChromPlotWidget(this, this, m_name, desc, 
+			TicChromPlotWidget *widget =
+				new TicChromPlotWidget(this, m_name, desc,
 						massSpecDataSet, fileName, isMultiGraph);
 
 			widget->setObjectName("monoGraphPlot");
@@ -201,11 +207,11 @@ namespace msXpSmineXpert
 			// "ticChrom".
 
 			QString propName =
-				QString("ticChromPlot%1").arg(TicChromWnd::widgetCounter);
-			QString alias = "lastTicChromPlot";
+				QString("ticChromPlotWidget%1").arg(TicChromWnd::widgetCounter);
+			QString alias = "lastTicChromPlotWidget";
 
-			QString explanation = 
-				QString("Make available a TIC chromatogram plot under name \"%1\" "
+			QString explanation =
+				QString("Make available a TIC chromatogram plot widget under name \"%1\" "
 						"with alias \"%2\".\n").arg(propName).arg(alias);
 
 			// Make the new plot widget globally available under the crafted property
@@ -214,11 +220,11 @@ namespace msXpSmineXpert
 			// tree widget as a child item of this.
 
 			QScriptValue scriptValue;
-			mainWindow->mp_scriptingWnd->publishQObject(widget, 
-					this /* parent object */, 
+			mainWindow->mp_scriptingWnd->publishQObject(widget,
+					this /* parent object */,
 					propName, alias, explanation, QString() /* help string */,
 					scriptValue,
-					QScriptEngine::QtOwnership, 
+					QScriptEngine::QtOwnership,
 					QScriptEngine::QObjectWrapOptions());
 
 			++TicChromWnd::widgetCounter;
@@ -233,14 +239,13 @@ namespace msXpSmineXpert
 
 			// Let's see if one color was set as the desired color, otherwise use
 			// black.
-			QPen currentPen = widget->graph()->pen();
 
-			if(!m_wishedColor.isValid())
-				currentPen.setColor(Qt::black);
-			else
-				currentPen.setColor(m_wishedColor);
+			QColor localColor = Qt::black;
 
-			widget->graph()->setPen(currentPen);
+			if(m_wishedColor.isValid())
+				localColor = m_wishedColor;
+
+			widget->setPlottingColor(localColor);
 
 			widget->rescaleAxes(true /*only enlarge*/);
 			widget->replot();
@@ -251,9 +256,12 @@ namespace msXpSmineXpert
 			// At this point, make sure that the same data are represented on the
 			// multi-graph plot widget.
 
+			QPen currentPen = widget->graph()->pen();
 			QCPGraph *newGraph = mp_multiGraphPlotWidget->addGraph();
 			newGraph->setData(keyVector, valVector);
-			newGraph->setPen(currentPen);
+
+			// Duplicate the same widget pen for the multigraph graph.
+			newGraph->setPen(widget->graph()->pen());
 			mp_multiGraphPlotWidget->rescaleAxes(true /*only enlarge*/);
 			mp_multiGraphPlotWidget->replot();
 
@@ -281,9 +289,19 @@ namespace msXpSmineXpert
 
 
 	AbstractPlotWidget *
-		TicChromWnd::initialTic(MassSpecDataSet *massSpecDataSet)
+		TicChromWnd::initialTic(MassSpecDataSet *p_massSpecDataSet)
 		{
-			MassDataIntegrator integrator(massSpecDataSet);
+
+			if(p_massSpecDataSet == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Pointer cannot be nullptr."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			// The mass data integration processes are handled by the
+			// MassDataIntegrator object.
+
+			MassDataIntegrator integrator(p_massSpecDataSet);
 
 			integrator.initialTic();
 
@@ -297,27 +315,61 @@ namespace msXpSmineXpert
 			// call below).
 
 			AbstractPlotWidget *widget = addPlotWidget(
-					integrator.keyVector(), integrator.valVector(), 
-					massSpecDataSet->fileName(), massSpecDataSet);
+					integrator.keyVector(), integrator.valVector(),
+					p_massSpecDataSet->fileName(), p_massSpecDataSet);
 
 			// For this specific TIC chrom widget, that is created for storing the
 			// initial mass spec data set as read from file, we set it as parent of
 			// the mass spec data set, so that when the widget is destroyed, the mass
 			// spec data set also.
-			massSpecDataSet->setParent(widget);
+			p_massSpecDataSet->setParent(widget);
 
-			// At this point, immediately craft a new HistoryItem instance to append
-			// to the widget's History.
+			// Before updating the history of the widget, update the history with the
+			// two critical data bits that will be conditioning the way future
+			// integrations are performed: the smallestStep value that goes along the
+			// msXpS::MassToleranceType::MASS_TOLERANCE_MZ.
+
+			MassSpecDataStats stats = p_massSpecDataSet->statistics();
+
+			// Use the statistics to setup the initial mz integration params to the
+			// history of the plot widget. Note that we set the binning type to
+			// arbitrary with the bin size set to twice the smallest m/z step observed
+			// throughout all the data points of all the spectra of the file.
+			MzIntegrationParams params(BinningType::BINNING_TYPE_ARBITRARY,
+
+					// this was the first version, but that value is not robuts: many
+					// times it happens to be 0, so the 0.001 default value would be set.
+					// That m_smallestStep value is too low for most occasions.
+					// m_smallestStep,
+
+					// The m_smallestStepAvg is the most robust.
+					stats.m_smallestStepAvg,
+					msXpS::MassToleranceType::MASS_TOLERANCE_MZ,
+					false /* apply mz shift */,
+					false /* remove 0-val data points */);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "stats.m_smallestStepAvg =" << stats.m_smallestStepAvg;
+
+			localHistory.setMzIntegrationParams(params);
+
+			// At this point, make sure that the widget has the new version of the
+			// history.
 
 			widget->rhistory().copyHistory(localHistory, true /*remove duplicates*/);
 
+			// Log the history to the console.
+			MainWindow *mainWindow = static_cast<MainWindow *>(parent());
+			mainWindow->logConsoleMessage(widget->history().innermostRangesAsText());
+
 			//qDebug() << __FILE__ << __LINE__
 			//<< "After initialTic was computed, history:"
 			//<< widget->history().asText();
 
-			// Log the history to the console.
-			MainWindow *mainWindow = static_cast<MainWindow *>(parent());
-			mainWindow->logConsoleMessage(widget->history().innermostRangesAsText());
+			mainWindow->logConsoleMessage(stats.asText(), m_wishedColor);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Statistics: " << p_massSpecDataSet->statisticsAsText();
 
 			// The plot widget reacts to the Qt::Key_Delete key by calling a function
 			// that emits a set of signals, of which the destroyPlotWidget signal
@@ -339,12 +391,16 @@ namespace msXpSmineXpert
 			// chromatogram plot.
 
 			connect(static_cast<TicChromPlotWidget *>(widget),
-					&TicChromPlotWidget::newMassSpectrum,
-					mainWindow->mp_massSpecWnd, &MassSpecWnd::newMassSpectrum);
+					&TicChromPlotWidget::newMassSpectrum, mainWindow->mp_massSpecWnd,
+					&MassSpecWnd::newMassSpectrum);
 
 			connect(static_cast<TicChromPlotWidget *>(widget),
-					&TicChromPlotWidget::newDriftSpectrum,
-					mainWindow->mp_driftSpecWnd, &DriftSpecWnd::newDriftSpectrum);
+					&TicChromPlotWidget::newDriftSpectrum, mainWindow->mp_driftSpecWnd,
+					&DriftSpecWnd::newDriftSpectrum);
+
+			connect(static_cast<TicChromPlotWidget *>(widget),
+					&TicChromPlotWidget::newTicChromatogram, this,
+					&TicChromWnd::newTicChromatogram);
 
 			// Ensure that there is a connection between this plot widget and this
 			// containing window, such that if there is a requirement to replot, the
@@ -357,6 +413,11 @@ namespace msXpSmineXpert
 			connect(this, &AbstractMultiPlotWnd::redrawPlotBackground, widget,
 					&AbstractPlotWidget::redrawPlotBackground);
 
+			// Set this newly created widget to be the last-focused widget.
+			widget->setFocus();
+			mp_lastFocusedPlotWidget = widget;
+			updateMzIntegrationParams(widget->history().mzIntegrationParams());
+
 			return widget;
 		}
 
@@ -378,6 +439,9 @@ namespace msXpSmineXpert
 		TicChromWnd::newTicChromatogram(MassSpecDataSet *massSpecDataSet,
 				const QString &msg, History history, QColor color)
 		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Entering";
+
 			if(massSpecDataSet == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
@@ -408,9 +472,9 @@ namespace msXpSmineXpert
 			//qDebug() << "At this point the integration is finished. Desetup the feedback.";
 
 			//De-setup the feedback.
-			QTimer::singleShot(2000, this, [=]() 
-					{ 
-					setupFeedback("", -1, -1, -1, LogType::LOG_TO_STATUS_BAR); 
+			QTimer::singleShot(2000, this, [=]()
+					{
+					setupFeedback("", -1, -1, -1, LogType::LOG_TO_STATUS_BAR);
 					}
 					);
 
@@ -476,6 +540,9 @@ namespace msXpSmineXpert
 			connect(widget, &TicChromPlotWidget::newDriftSpectrum,
 					mainWindow->mp_driftSpecWnd, &DriftSpecWnd::newDriftSpectrum);
 
+			connect(widget, &TicChromPlotWidget::newTicChromatogram,
+					this, &TicChromWnd::newTicChromatogram);
+
 			AbstractPlotWidget *senderPlotWidget =
 				dynamic_cast<AbstractPlotWidget *>(QObject::sender());
 
@@ -485,6 +552,9 @@ namespace msXpSmineXpert
 			mainWindow->m_dataPlotWidgetRelationer.newRelation(
 					massSpecDataSet, senderPlotWidget, widget,
 					Q_NULLPTR /* open spectra dialog list widget item */);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Exiting";
 		}
 
 
@@ -561,7 +631,6 @@ namespace msXpSmineXpert
 		}
 
 
-
 	//! Toggle the visibility of the multigraph graph and the target widgets.
 	/*!
 
@@ -569,7 +638,7 @@ namespace msXpSmineXpert
 		corresponding to the signal emitter plot widget. This overridden function
 		does more, not only does it toggle the visibility of the multigraph graph
 		corresponding to the sender plot widget, but does the same for all the
-		target plot widgets. 
+		target plot widgets.
 
 		This is useful in the case where there are many opened spectra in the
 		program, thus many initial (top) TIC chromatogram plot widget that may have
