@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #pragma once
 
 
@@ -54,7 +55,7 @@ namespace msXpSmineXpert
 	//! The TicChromWnd class provides a window to display TIC chromatograms.
 	/*!
 
-		The window will host two main parts: 
+		The window will host two main parts:
 
 		- the upper part will host a single plot widget where the graphs
 		corresponding to all the TIC chromatograms will be plotted overlaid.
@@ -86,6 +87,8 @@ namespace msXpSmineXpert
 */
 			static int widgetCounter;
 
+			QPushButton *mp_mzIntegrationParamsButton = Q_NULLPTR;
+
 		public:
 
 			TicChromWnd(QWidget *parent);
@@ -97,10 +100,10 @@ namespace msXpSmineXpert
 			Q_INVOKABLE void hide(){QMainWindow::hide();};
 			Q_INVOKABLE void show(){QMainWindow::show();};
 
-			TicChromPlotWidget *addPlotWidget(const QVector<double> &keyVector, 
+			TicChromPlotWidget *addPlotWidget(const QVector<double> &keyVector,
 					const QVector<double> &valVector,
-					const QString &desc = QString(), 
-					MassSpecDataSet *massSpecDataSet = Q_NULLPTR, 
+					const QString &desc = QString(),
+					MassSpecDataSet *massSpecDataSet = Q_NULLPTR,
 					bool isMultiGraph = false);
 
 			AbstractPlotWidget *initialTic(MassSpecDataSet *massSpecDataSet);
