@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 /////////////////////// Std includes
 #include <iostream>
 
@@ -68,7 +69,7 @@ namespace msXpSmineXpert
 	}
 
 
-	//! Destruct \c this CommandLineRunner instance 
+	//! Destruct \c this CommandLineRunner instance
 	CommandLineRunner::~CommandLineRunner()
 	{
 	}
@@ -100,7 +101,7 @@ namespace msXpSmineXpert
 				if(fileInfo.exists())
 					existingFiles.append(m_currentInputFileName);
 				else
-					msXpS::qStdOut() << __FILE__ << __LINE__ << __FUNCTION__ 
+					msXpS::qStdOut() << __FILE__ << __LINE__ << __FUNCTION__
 						<< "File" << m_currentInputFileName << "not found";
 			}
 
@@ -230,16 +231,16 @@ namespace msXpSmineXpert
 		\param existingFiles pointer to a string list in which to store the file
 		names of the existing files actually found on disk.
 
-		\param errors pointer to a string in which to describe the errors encountered. 
+		\param errors pointer to a string in which to describe the errors encountered.
 
 		\return true if the program can be run through its GUInterface; false
 		otherwise.
 
 */
-	bool 
-		CommandLineRunner::validateCommandLine(QStringList *existingFiles, 
-				QStringList *missingFiles, 
-				QString *errors, 
+	bool
+		CommandLineRunner::validateCommandLine(QStringList *existingFiles,
+				QStringList *missingFiles,
+				QString *errors,
 				bool *shouldRunGui)
 		{
 
@@ -319,7 +320,7 @@ namespace msXpSmineXpert
 						return false;
 					}
 
-					//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+					//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 					//<< "Script file exists indeed.";
 
 					*shouldRunGui = true;
@@ -365,16 +366,10 @@ namespace msXpSmineXpert
 		For example, if a file format conversion is asked for, that conversion is
 		run without even starting the Qt application execution loop.
 
-		\param existingFiles pointer to a string list in which to store the file
-		names of files effectively found on disk. May not be nullptr.
-
 		\param errors pointer to a string in which to store all the errors
 		encountered during the analysis of the command line. May not be nullptr.
 
-		\param shouldRunGui pointer to a boolean value in which to tell if the
-		program should run the graphical user interface. May not be nullptr.
-
-		\return 
+		\return
 
 		- true if the GUI should be run right away
 		- true if a script file name was provided
@@ -425,7 +420,7 @@ namespace msXpSmineXpert
 		\sa FileFormatConverter.
 
 */
-	bool 
+	bool
 		CommandLineRunner::runConversion(QString *errors)
 		{
 			bool noErrorAtAll = true;
@@ -455,8 +450,8 @@ namespace msXpSmineXpert
 
 					// The errors string will be used later by the calling function
 
-					//cout << "\n" <<  __FILE__ << __LINE__ << __FUNCTION__ 
-					//<< "-- Conversion for file:" << loader.inFileName().toStdString() 
+					//cout << "\n" <<  __FILE__ << __LINE__ << __FUNCTION__
+					//<< "-- Conversion for file:" << loader.inFileName().toStdString()
 					//<< "failure with erros:\n\n"
 					//<< errors->toStdString() << endl;
 
@@ -488,7 +483,7 @@ namespace msXpSmineXpert
 		\sa MassSpecDataFileLoaderPwiz.
 
 */
-	bool 
+	bool
 		CommandLineRunner::runExportToXyFiles(QString *errors)
 		{
 			bool noErrorAtAll = true;
@@ -507,8 +502,8 @@ namespace msXpSmineXpert
 			{
 				QString curFileName = m_inFileNames.at(iter);
 
-				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
-					//<< "Current file to export name:" << curFileName;
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+				//<< "Current file to export name:" << curFileName;
 
 				MassSpecDataFileLoaderPwiz loader;
 
@@ -520,7 +515,7 @@ namespace msXpSmineXpert
 				if(loader.readSpectrumCount() == 0)
 				{
 					errors->append(QString("The file %1 to export contains no data:\n")
-						 .arg(curFileName));
+							.arg(curFileName));
 
 					noErrorAtAll = false;
 
