@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include <qmath.h>
 #include <omp.h>
 
@@ -46,12 +47,27 @@
 #include <minexpert/nongui/MassSpecDataFileLoaderDx.hpp>
 #include <minexpert/nongui/MassSpecDataFileLoaderMs1.hpp>
 #include <minexpert/nongui/MassSpecDataFileLoaderSqlite3.hpp>
+#include <minexpert/nongui/SavGolFilter.hpp>
 
 
 namespace msXpSmineXpert
 {
 
 
+	//! Construct an initialized MassDataIntegrator instance.
+	/*!
+
+		\param massSpecDataSet MassSpecDataSet instance holding the mass data on
+		which to perform the integrations.
+
+*/
+	MassDataIntegrator::MassDataIntegrator(MassSpecDataSet *massSpecDataSet)
+		:
+			mp_massSpecDataSet{massSpecDataSet}
+	{
+	}
+
+
 	//! Construct an initialized MassDataIntegrator instance.
 	/*!
 
@@ -71,21 +87,7 @@ namespace msXpSmineXpert
 	}
 
 
-	//! Construct an initialized MassDataIntegrator instance.
-	/*!
-
-		\param massSpecDataSet MassSpecDataSet instance holding the mass data on
-		which to perform the integrations.
-
-*/
-	MassDataIntegrator::MassDataIntegrator(MassSpecDataSet *massSpecDataSet)
-		:
-			mp_massSpecDataSet{massSpecDataSet}
-	{
-	}
-
-
-	//! Destroy \c this MassDataIntegrator instance.	
+	//! Destroy \c this MassDataIntegrator instance.
 	MassDataIntegrator::~MassDataIntegrator()
 	{
 	}
@@ -108,7 +110,7 @@ namespace msXpSmineXpert
 
 
 	//! Return a copy of \c m_history
-	History 
+	History
 		MassDataIntegrator::history() const
 		{
 			return m_history;
@@ -130,7 +132,7 @@ namespace msXpSmineXpert
 
 
 	//! Compute the total ion current intensity value according to \c m_history.
-	double 
+	double
 		MassDataIntegrator::ticIntensity() const
 		{
 			return m_ticIntensity;
@@ -142,13 +144,13 @@ namespace msXpSmineXpert
 		MassDataIntegrator::integrate()
 		{
 
-			// Very complex process here. 
+			// Very complex process here.
 			//
 			// We need to get the most recent history item from m_history.
 
 			const HistoryItem *newestHistoryItem = m_history.newestHistoryItem();
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 			//<< "Newest history item:" << newestHistoryItem->asText();
 
 			// Now we need to know what is the integration type (or what are the
@@ -160,7 +162,7 @@ namespace msXpSmineXpert
 				qFatal("Fatal error at %s@%d -- %s. "
 						"Cannot be that the history item  has not a single integration type."
 						"Program aborted.",
-						__FILE__, __LINE__, __FUNCTION__);	
+						__FILE__, __LINE__, __FUNCTION__);
 
 			// The list might contain more than one integration range, but whatever the
 			// number of these ranges, the IntegrationType needs to point towards the
@@ -173,8 +175,12 @@ namespace msXpSmineXpert
 
 			int integrationType = integrationTypeList.first();
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
-			//<< "integration type:" << integrationType;
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "size:" << HistoryItem::m_integrationTypeDescriptionMap.size();
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+				//<< "integration type:" << integrationType
+				//<< "string:" << HistoryItem::m_integrationTypeDescriptionMap.value(integrationType)->brief;
 
 			if(integrationType & ANY_TO_RT)
 				return integrateToRt();
@@ -200,7 +206,7 @@ namespace msXpSmineXpert
 		When mass spectrometry data are loaded from a mass data file, one of the
 		first computation that is performed is the production of a TIC chromatogram
 		representing the mass data as a function of the acquisition time (retention
-		time). This is what this function does. 
+		time). This is what this function does.
 
 		\param massSpecDataSet pointer to the MassSpecDataSet instance where the
 		data were initially stored during the data file loading process.
@@ -231,7 +237,7 @@ namespace msXpSmineXpert
 			// We now have access to the smallest/greatest retention time values.
 
 			HistoryItem *histItem = new HistoryItem;
-			histItem->newIntegrationRange(IntegrationType::DATA_TO_RT,
+			histItem->newIntegrationRange(IntegrationType::FILE_TO_RT,
 					rtList.first(), rtList.last());
 			m_history.appendHistoryItem(histItem);
 
@@ -271,8 +277,8 @@ namespace msXpSmineXpert
 
 
 	int
-		MassDataIntegrator::fillInDtMzHash(int* p_dtValueCount, int*p_mzCells, 
-				double *p_firstKey, double *p_lastKey, 
+		MassDataIntegrator::fillInDtMzHash(int* p_dtValueCount, int*p_mzCells,
+				double *p_firstKey, double *p_lastKey,
 				double *p_firstVal, double *p_lastVal)
 		{
 			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
@@ -308,7 +314,7 @@ namespace msXpSmineXpert
 
 				int dtListSize = dtList.size();
 
-				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 				//<< "The list of drift time values has " << dtListSize << "items";
 
 				if(dtListSize == 0)
@@ -356,32 +362,20 @@ namespace msXpSmineXpert
 
 					QList<msXpSlibmass::MassSpectrum *> massSpectra = mp_massSpecDataSet->dtHash().values(dt);
 
-					int spectrumCount = massSpectra.size();
-
-#if 0
-					// That should not occur, because we do not store empty spectra when
-					// loading files.
-
-					// It might be that there are no spectra for a given dt value.
-					if(spectrumCount == 0)
-					{
-						qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
-							<< "For dt value" << dt << "there are no mass spectra.";
-
-						// Essential to skip this round  because we allocate a combined mass
-						// spectrum below and we then access its members... Crash.
-
-						continue;
-					}
-#endif 
-
 					msXpSlibmass::MassSpectrum *combinedMassSpectrum = new msXpSlibmass::MassSpectrum;
+					// We compute "rough" spectra here, with type m/z resolution.
+					combinedMassSpectrum->rdecimalPlaces() = 0;
+					// We need to specify that we do not want any binning here.
+					combinedMassSpectrum->setBinningType(msXpS::BinningType::BINNING_TYPE_NONE);
 
-					for(int jter = 0; jter < spectrumCount; ++jter)
-					{
-						combinedMassSpectrum->rdecimalPlaces() = 0;
-						combinedMassSpectrum->combine(*massSpectra.at(jter));
-					}
+					// Older version
+					//int spectrumCount = massSpectra.size();
+					//for(int jter = 0; jter < spectrumCount; ++jter)
+					//{
+					//combinedMassSpectrum->combine(*massSpectra.at(jter));
+					//}
+
+					combinedMassSpectrum->combine(massSpectra);
 
 					int massSpectrumSize = combinedMassSpectrum->size();
 
@@ -404,7 +398,7 @@ namespace msXpSmineXpert
 					// fill-in the colormap.
 					m_doubleMassSpecHash.insert(dt, combinedMassSpectrum);
 
-					//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+					//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 					//<< "The dtMsHash has " << m_doubleMassSpecHash.size() << "items";
 
 					emit updateFeedbackSignal(QString("dt: %1").arg(dt, 0, 'f', 6),
@@ -437,14 +431,14 @@ namespace msXpSmineXpert
 			}
 			else
 			{
-				streamFillInDtMzHash(p_dtValueCount, p_mzCells, 
-						p_firstKey, p_lastKey, 
+				streamFillInDtMzHash(p_dtValueCount, p_mzCells,
+						p_firstKey, p_lastKey,
 						p_firstVal, p_lastVal);
 
 				// We need to set that variable value because that is returned later.
 				parsedDtValues = *p_dtValueCount;
 
-				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 				//<< "The dtMsHash has " << m_doubleMassSpecHash.size() << "items."
 				//<< "That is (in/con)firmed with parsed dt values:" << parsedDtValues;
 			}
@@ -458,10 +452,10 @@ namespace msXpSmineXpert
 			// Because we are documenting the mz = f(dt) colormap, we need two
 			// integration ranges: the MZ range and the DT range.
 
-			histItem->newIntegrationRange(IntegrationType::DATA_TO_DT,
+			histItem->newIntegrationRange(IntegrationType::FILE_TO_DT,
 					*p_firstKey, *p_lastKey);
 
-			histItem->newIntegrationRange(IntegrationType::DATA_TO_MZ,
+			histItem->newIntegrationRange(IntegrationType::FILE_TO_MZ,
 					*p_firstVal, *p_lastVal);
 
 			m_history.appendHistoryItem(histItem);
@@ -471,8 +465,8 @@ namespace msXpSmineXpert
 
 
 	int
-		MassDataIntegrator::streamFillInDtMzHash(int *p_dtValueCount, int*p_mzCells, 
-				double *p_firstKey, double *p_lastKey, 
+		MassDataIntegrator::streamFillInDtMzHash(int *p_dtValueCount, int*p_mzCells,
+				double *p_firstKey, double *p_lastKey,
 				double *p_firstVal, double *p_lastVal)
 		{
 			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
@@ -500,13 +494,14 @@ namespace msXpSmineXpert
 
 			int dtListSize = dtList.size();
 
-			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()" 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
 			//<< "The list of drift time values has " << dtListSize << "items";
 
 			if(dtListSize == 0)
 			{
 				// This is not intrinsically an error. It could be simply that the data
 				// are not for a mobility mass spectrometry experiment.
+				*p_dtValueCount = dtListSize;
 				return 0;
 			}
 
@@ -534,9 +529,12 @@ namespace msXpSmineXpert
 
 
 	//! Integrate to a XIC (extracted ion chromatogram) according to \c m_history.
-	bool 
+	bool
 		MassDataIntegrator::integrateToRt()
 		{
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Entering.";
+
 			if(mp_massSpecDataSet->isStreamed())
 				return streamIntegrateToRt();
 
@@ -551,7 +549,7 @@ namespace msXpSmineXpert
 				qFatal("Fatal error at %s@%d -- %s. "
 						"Cannot be that the history item  has not a single integration type."
 						"Program aborted.",
-						__FILE__, __LINE__, __FUNCTION__);	
+						__FILE__, __LINE__, __FUNCTION__);
 
 			int integrationType = integrationTypes.first();
 
@@ -579,10 +577,11 @@ namespace msXpSmineXpert
 			mzIntegration = m_history.innermostMzRange(&mzStart, &mzEnd);
 			dtIntegration = m_history.innermostDtRange(&dtStart, &dtEnd);
 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
 			//qDebug("rtIntegration: [%.6f-%.6f] \n"
 			//"mzIntegratin: [%.6f-%.6f] \n"
-			//"dtIntegration: [%.6f-%.6f] \n", 
-			//rtStart, rtEnd, 
+			//"dtIntegration: [%.6f-%.6f] \n",
+			//rtStart, rtEnd,
 			//mzStart, mzEnd,
 			//dtStart, dtEnd);
 
@@ -680,6 +679,8 @@ namespace msXpSmineXpert
 					rtTicMap.insert(rt, newTic);
 				}
 				// End of for(int kter = 0; kter < massSpectra.size(); ++kter)
+				// End of
+				// #pragma omp parallel for
 
 #pragma omp critical
 				{
@@ -732,12 +733,15 @@ namespace msXpSmineXpert
 
 			m_isOperationCancelled = false;
 
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Exiting.";
+
 			return true;
 		}
 
 
 	//! Integrate (streamed mode) to a XIC (extracted ion chromatogram) according to \c m_history.
-	bool	
+	bool
 		MassDataIntegrator::streamIntegrateToRt()
 		{
 			m_keyVector.clear();
@@ -754,12 +758,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -767,7 +771,7 @@ namespace msXpSmineXpert
 						&MassSpecDataFileLoaderSqlite3::cancelOperation);
 
 				int res = loader.streamedIntegration(
-						m_history, IntegrationType::ANY_TO_RT, &m_keyVector, &m_valVector);
+						m_history, IntegrationType::XXX_TO_RT, &m_keyVector, &m_valVector);
 
 				if(res == -1)
 					qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
@@ -779,12 +783,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -792,7 +796,7 @@ namespace msXpSmineXpert
 						&loader, &MassSpecDataFileLoaderPwiz::cancelOperation);
 
 				int res = loader.streamedIntegration(
-						m_history, IntegrationType::ANY_TO_RT, &m_keyVector, &m_valVector);
+						m_history, IntegrationType::XXX_TO_RT, &m_keyVector, &m_valVector);
 
 				if(res == -1)
 					qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
@@ -833,7 +837,7 @@ namespace msXpSmineXpert
 
 
 	//! Integrate to a mass spectrum according to \c m_history.
-	bool 
+	bool
 		MassDataIntegrator::integrateToMz()
 		{
 			if(mp_massSpecDataSet->isStreamed())
@@ -850,7 +854,7 @@ namespace msXpSmineXpert
 				qFatal("Fatal error at %s@%d -- %s. "
 						"Cannot be that the history item  has not a single integration type."
 						"Program aborted.",
-						__FILE__, __LINE__, __FUNCTION__);	
+						__FILE__, __LINE__, __FUNCTION__);
 
 			int integrationType = integrationTypes.first();
 
@@ -880,8 +884,8 @@ namespace msXpSmineXpert
 
 			//qDebug("rtIntegration: [%.6f-%.6f] \n"
 			//"mzIntegratin: [%.6f-%.6f] \n"
-			//"dtIntegration: [%.6f-%.6f] \n", 
-			//rtStart, rtEnd, 
+			//"dtIntegration: [%.6f-%.6f] \n",
+			//rtStart, rtEnd,
 			//mzStart, mzEnd,
 			//dtStart, dtEnd);
 
@@ -903,8 +907,6 @@ namespace msXpSmineXpert
 
 			QString msg("Integrating to MZ");
 
-			emit setupFeedbackSignal(msg, 0, rtListSize, -1, LogType::LOG_TO_STATUS_BAR);
-
 			QString integrationDetails = "Integrating to MZ\n\tIntegration details:\n";
 
 			if(rtIntegration)
@@ -920,13 +922,28 @@ namespace msXpSmineXpert
 					.arg(dtStart, 0, 'f', 6)
 					.arg(dtEnd, 0, 'f', 6);
 
-			emit updateFeedbackSignal(integrationDetails, -1, LogType::LOG_TO_CONSOLE);
-
+			QList<msXpSlibmass::MassSpectrum *> massSpectrumList;
 			msXpSlibmass::MassSpectrum combinedMassSpectrum;
 
-			int iterationCount = 0;
+			// When integrating to m/z, we must take into account the bin
+			// parameters, the nominal value and the type and other parameters.
+
+			MzIntegrationParams mzIntegrationParams = m_history.mzIntegrationParams();
+			combinedMassSpectrum.setBinningType(mzIntegrationParams.m_binningType);
+			combinedMassSpectrum.setBinSize(mzIntegrationParams.m_binSize);
+			combinedMassSpectrum.setBinSizeType(mzIntegrationParams.m_binSizeType);
+			combinedMassSpectrum.applyMzShift(mzIntegrationParams.m_applyMzShift);
+			combinedMassSpectrum.setRemoveZeroValDataPoints(mzIntegrationParams.m_removeZeroValDataPoints);
+
+			// And now other data from the mass spec data stats:
+			MassSpecDataStats statistics = mp_massSpecDataSet->statistics();
+
+			combinedMassSpectrum.setMinMz(statistics.m_minMz);
+			combinedMassSpectrum.setMaxMz(statistics.m_maxMz);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "The retention time list has " << rtListSize << "time values";
 
-#pragma omp parallel for
 			for(int iter = 0; iter < rtListSize; ++iter)
 			{
 
@@ -935,9 +952,11 @@ namespace msXpSmineXpert
 
 				double rt = rtList.at(iter);
 
-				// qDebug() << __FILE__ << __LINE__ << "integrating for rt:" << rt;
+				//qDebug() << __FILE__ << __LINE__ << "integrating for rt:" << rt;
 
 				// Get a list of all the mass spectra having the same key value.
+				// For MS1 spectra, there is only one spectrum per rt for non-mobility
+				// mass spectrometry, of course.
 
 				QList<msXpSlibmass::MassSpectrum *> rtMassSpectra(rtHash.values(rt));
 
@@ -956,40 +975,188 @@ namespace msXpSmineXpert
 					}
 				}
 
-				// qDebug() << __FILE__ << __LINE__
-				// << "For rt " << rt << "the number of spectra is" <<
-				// rtMassSpectra.size();
+				// At this point we have the accounted mass spectra remaining in the
+				// rtMassSpectra list. Append these spectra to the main list.
+
+				massSpectrumList.append(rtMassSpectra);
+			}
+
+			// Now perform the integration.
+
+			int specCount = massSpectrumList.size();
+
+			emit setupFeedbackSignal(msg, 0, specCount, -1, LogType::LOG_TO_STATUS_BAR);
+			emit updateFeedbackSignal(integrationDetails, -1, LogType::LOG_TO_CONSOLE);
+
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "Size of the list of all the spectra of the integration to MZ : "
+				<< massSpectrumList.size();
+
+			int maxThreads = omp_get_max_threads();
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "Max threads: " << maxThreads;
+
+			if(specCount < 10 * maxThreads)
+			{
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Number of spectra to combine:" << specCount
+					<< "Too small a number of spectra, not using the parallel code.";
+
+				if(mzIntegration)
+					combinedMassSpectrum.combine(massSpectrumList, mzStart, mzEnd);
+				else
+					combinedMassSpectrum.combine(massSpectrumList);
+			}
+			else
+				// of
+				// if(specCount < 10 * numbThreads)
+			{
+				omp_set_num_threads(maxThreads);
+
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Number of spectra sufficient to use parallel code."
+					<< "Set number of threads: " << maxThreads;
+
+				// Allocate as many combined mass spectra as we are using threads.
+				QList<msXpSlibmass::MassSpectrum *> combinedMassSpectrumList;
+
+				// Set the various slices of massSpectra so that each thread knows what
+				// slice to handle.
+				QList<QPair<int, int> *> indexRangeList;
+
+				double specPerThread = specCount / maxThreads;
+				// Integer part of the fractional result.
+				double intPart;
+				// We won't use fracPart, but anyway.
+				double fracPart = std::modf(specPerThread, &intPart);
+				Q_UNUSED(fracPart);
+
+				for(int iter = 0; iter < maxThreads; ++iter)
+				{
+
+					// Create a new mass spectrum that is identical to the one that was
+					// configured above for the various mz integration params, so that it
+					// too benefits from the binning configuration data as determined by
+					// the call to setupBinnability above.
+
+					combinedMassSpectrumList.append(new msXpSlibmass::MassSpectrum(combinedMassSpectrum));
+
+					if(iter != maxThreads - 1)
+					{
+						if(!iter)
+							// We actually start from 0 = (iter * intPat), with iter = 0.
+							indexRangeList.append(new QPair<int, int>(iter * intPart, (iter + 1) * intPart));
+						else
+							// We need to increment by one index value the startIdx, otherwise
+							// we would combine twice the spectrum at index (iter * intPart)
+							indexRangeList.append(new QPair<int, int>((iter * intPart) + 1, (iter + 1) * intPart));
+					}
+					else
+					{
+
+						// Now is the time to complete to the fractional part, without
+						// bothering: simply set the lastIdx of QPair to
+						// specCount - 1.
+
+						indexRangeList.append(new QPair<int, int>(iter * intPart, specCount - 1));
+					}
+				}
+
+				// At this point we have setup all the slices to combine them in their
+				// own combined mass spectrum.
+
+				int count = 0;
+
+#pragma omp parallel for shared(count)
+				for(int iter = 0; iter < maxThreads; ++iter)
+				{
+					msXpSlibmass::MassSpectrum *p_combinedMassSpectrum = combinedMassSpectrumList.at(iter);
+					QPair<int, int> *p_indexRange = indexRangeList.at(iter);
+
+					count += p_combinedMassSpectrum->combineSlice(massSpectrumList,
+							p_indexRange->first, p_indexRange->second, mzStart, mzEnd);
 
 #pragma omp critical
-				{
-					if(mzIntegration)
-						combinedMassSpectrum.combine(rtMassSpectra, mzStart, mzEnd);
-					else
-						combinedMassSpectrum.combine(rtMassSpectra);
+					{
+						//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+							//<< QString::asprintf("Combined slice %d-%d\n", p_indexRange->first, p_indexRange->second);
 
-					++iterationCount;
-
-					// The feedback was setup by the master thread, so we can only
-					// update it(actually emitting the signal) from that same master
-					// thread:
-
-					if(omp_get_thread_num() == 0)
-						emit updateFeedbackSignal(msg, iterationCount, LogType::LOG_TO_STATUS_BAR);
+						//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+							//<< "with count:" << count;
+						emit updateFeedbackSignal(msg, count, LogType::LOG_TO_STATUS_BAR);
+					}
 				}
 				// End of
-				// #pragma omp critical
+				// #pragma omp parallel
+
+				// At this point we need to conclude and combine into *this mass
+				// spectrum all the spectra in the list of combined mass spectra. Take
+				// advantage of the loop to also free each combined mass spectrum in
+				// turn.
+				//
+				// At first, copy one of the mass spectra in the list right into
+				// combinedMassSpectrum, so that the bins are copied in there if there
+				// were bins. Remember, a binned combine of a spectrum cannot be
+				// performed by an empty mass spectrum.
+
+				combinedMassSpectrum = *combinedMassSpectrumList.takeFirst();
+				delete combinedMassSpectrumList.takeFirst();
+
+				// And now combine all the remaining spectra.
+				while(combinedMassSpectrumList.size())
+				{
+					msXpSlibmass::MassSpectrum *massSpectrum = combinedMassSpectrumList.takeFirst();
+
+					// No need to check mz range because we did that already above.
+					combinedMassSpectrum.combine(*massSpectrum, -1, -1);
+
+					delete massSpectrum;
+				}
+
+				// At this point, we need to free all the QPair instances
+				while(indexRangeList.size())
+					delete indexRangeList.takeFirst();
 			}
 			// End of
-			// for(int iter = 0; iter < rtListSize; ++iter)
-			// and of
-			// #pragma omp parallel for
+			// else /* of if(specCount < 10 * maxThreads) */
+
+
+			// The combination work is finished, however it has been performed.
+
+			if(combinedMassSpectrum.isRemoveZeroValDataPoints())
+			{
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "Now removing all 0-m/z data points.";
+
+				combinedMassSpectrum.removeZeroValDataPoints();
+			}
 
 			m_keyVector = QVector<double>::fromList(combinedMassSpectrum.keyList());
 			m_valVector = QVector<double>::fromList(combinedMassSpectrum.valList());
 
+			if(mzIntegrationParams.m_applySavGolFilter)
+			{
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "savGol is to be computed";
+
+				SavGolFilter savGolFilter(mzIntegrationParams.m_savGolParams);
+
+				savGolFilter.initializeData(m_keyVector, m_valVector);
+
+				savGolFilter.sgfilter();
+
+				QVector<double> filtered;
+				savGolFilter.filteredData(filtered);
+
+				m_valVector.clear();
+				m_valVector.append(filtered);
+			}
+
 			// Sanity check
 			if(m_keyVector.size() != m_valVector.size())
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "data:" << m_keyVector.size() << m_valVector.size();
 
 			// We may have gotten here because the operation was cancelled. In any case,
 			// reset the boolean value.
@@ -1001,7 +1168,7 @@ namespace msXpSmineXpert
 
 
 	//! Integrate (streamed mode) to a mass spectrum according to \c m_history.
-	bool 
+	bool
 		MassDataIntegrator::streamIntegrateToMz()
 		{
 			m_keyVector.clear();
@@ -1018,18 +1185,24 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
 				connect(this, &MassDataIntegrator::cancelOperationSignal, &loader,
 						&MassSpecDataFileLoaderSqlite3::cancelOperation);
 
+				// We are going to load data in streamed mode. For the specific case of
+				// m/z integration, and potentially using binning, we need to have a
+				// statistics knowledge of the initial data.
+
+				loader.setMassSpecDataStats(mp_massSpecDataSet->statistics());
+
 				int res = loader.streamedIntegration(
 						m_history, IntegrationType::ANY_TO_MZ, &m_keyVector, &m_valVector);
 
@@ -1043,18 +1216,24 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
 				connect(this, &MassDataIntegrator::cancelOperationSignal,
 						&loader, &MassSpecDataFileLoaderPwiz::cancelOperation);
 
+				// We are going to load data in streamed mode. For the specific case of
+				// m/z integration, and potentially using binning, we need to have a
+				// statistics knowledge of the initial data.
+
+				loader.setMassSpecDataStats(mp_massSpecDataSet->statistics());
+
 				int res = loader.streamedIntegration(
 						m_history, IntegrationType::ANY_TO_MZ, &m_keyVector, &m_valVector);
 
@@ -1068,12 +1247,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderDx::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderDx::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderDx::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderDx::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1094,12 +1273,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderMs1::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderMs1::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderMs1::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderMs1::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1122,14 +1301,14 @@ namespace msXpSmineXpert
 
 
 	//! Integrate to a drift spectrum according to \c m_history.
-	bool 
+	bool
 		MassDataIntegrator::integrateToDt()
 		{
 
 			// Only do something if the data set contains ion mobility data.
 
 			if(!mp_massSpecDataSet->isMobilityExperiment())
-				return false; 
+				return false;
 
 			if(mp_massSpecDataSet->isStreamed())
 				return streamIntegrateToDt();
@@ -1145,7 +1324,7 @@ namespace msXpSmineXpert
 				qFatal("Fatal error at %s@%d -- %s. "
 						"Cannot be that the history item  has not a single integration type."
 						"Program aborted.",
-						__FILE__, __LINE__, __FUNCTION__);	
+						__FILE__, __LINE__, __FUNCTION__);
 
 			int integrationType = integrationTypes.first();
 
@@ -1175,8 +1354,8 @@ namespace msXpSmineXpert
 
 			//qDebug("rtIntegration: [%.6f-%.6f] \n"
 			//"mzIntegration: [%.6f-%.6f] \n"
-			//"dtIntegration: [%.6f-%.6f] \n", 
-			//rtStart, rtEnd, 
+			//"dtIntegration: [%.6f-%.6f] \n",
+			//rtStart, rtEnd,
 			//mzStart, mzEnd,
 			//dtStart, dtEnd);
 
@@ -1242,7 +1421,6 @@ namespace msXpSmineXpert
 					{
 						msXpSlibmass::MassSpectrum *spectrum = rtMassSpectra.at(jter);
 
-
 						if(spectrum->dt() < dtStart || spectrum->dt() > dtEnd)
 						{
 							rtMassSpectra.removeAt(jter);
@@ -1309,7 +1487,7 @@ namespace msXpSmineXpert
 
 
 	//! Integrate (streamed mode) to a drift spectrum according to \c m_history.
-	bool	
+	bool
 		MassDataIntegrator::streamIntegrateToDt()
 		{
 			// Depending on the format of the data file from which the data were
@@ -1323,12 +1501,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1348,12 +1526,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1400,7 +1578,7 @@ namespace msXpSmineXpert
 				qFatal("Fatal error at %s@%d -- %s. "
 						"Cannot be that the history item  has not a single integration type."
 						"Program aborted.",
-						__FILE__, __LINE__, __FUNCTION__);	
+						__FILE__, __LINE__, __FUNCTION__);
 
 			int integrationType = integrationTypes.first();
 
@@ -1537,7 +1715,7 @@ namespace msXpSmineXpert
 
 
 	//! Integrate (streamed mode) to a single TIC intensity value according to \c m_history.
-	bool	
+	bool
 		MassDataIntegrator::streamIntegrateToTicIntensity()
 		{
 			if(mp_massSpecDataSet == Q_NULLPTR)
@@ -1560,16 +1738,16 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderSqlite3::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
-				connect(this, &MassDataIntegrator::cancelOperationSignal, 
+				connect(this, &MassDataIntegrator::cancelOperationSignal,
 						&loader,
 						&MassSpecDataFileLoaderSqlite3::cancelOperation);
 
@@ -1585,12 +1763,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderPwiz::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1609,12 +1787,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderDx::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderDx::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderDx::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderDx::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1622,7 +1800,7 @@ namespace msXpSmineXpert
 				connect(this, &MassDataIntegrator::cancelOperationSignal,
 						&loader, &MassSpecDataFileLoaderDx::cancelOperation);
 
-				int res = loader.streamedIntegration(m_history, 
+				int res = loader.streamedIntegration(m_history,
 						Q_NULLPTR /* keyVector */, Q_NULLPTR /* valVector */, &m_ticIntensity);
 
 				if(res == -1)
@@ -1635,12 +1813,12 @@ namespace msXpSmineXpert
 
 				// Make sure we connect the feedback signal/slot
 				connect(&loader,
-						&MassSpecDataFileLoaderMs1::setupFeedbackSignal, 
+						&MassSpecDataFileLoaderMs1::setupFeedbackSignal,
 						this,
 						&MassDataIntegrator::setupFeedbackRelay);
 
 				connect(&loader,
-						&MassSpecDataFileLoaderMs1::updateFeedbackSignal, 
+						&MassSpecDataFileLoaderMs1::updateFeedbackSignal,
 						this,
 						&MassDataIntegrator::updateFeedbackRelay);
 
@@ -1659,11 +1837,11 @@ namespace msXpSmineXpert
 			}
 
 			return true;
-		}	
+		}
 
 
 	//! React to the signal that the user cancelled the calculation. Relay the signal.
-	void 
+	void
 		MassDataIntegrator::cancelOperation()
 		{
 			m_isOperationCancelled = true;
@@ -1676,7 +1854,7 @@ namespace msXpSmineXpert
 
 
 	//! Relay the \c setupFeedback signal.
-	void 
+	void
 		MassDataIntegrator::setupFeedbackRelay(QString msg, int startValue, int endValue,
 				int currentValue, int logType)
 		{
@@ -1686,7 +1864,7 @@ namespace msXpSmineXpert
 
 
 	//! Relay the \c updateFeedback signal.
-	void 
+	void
 		MassDataIntegrator::updateFeedbackRelay(QString msg, int currentValue,
 				int logType)
 		{
