@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 /////////////////////// Qt includes
 #include <QObject>
 #include <QMainWindow>
@@ -52,10 +53,10 @@ namespace msXpSmineXpert
 
 	//! Construct a MassSpecDataFileLoader instance.
 	/*!
-	
+
 		\param Name of the file to load.
-	
-	 */
+
+*/
 	MassSpecDataFileLoader::MassSpecDataFileLoader(const QString &fileName)
 	{
 		if(!fileName.isEmpty())
@@ -69,12 +70,20 @@ namespace msXpSmineXpert
 	}
 
 
+	//! Copy \p massSpecDataStats to this instance
+	void
+		MassSpecDataFileLoader::setMassSpecDataStats(const MassSpecDataStats &massSpecDataStats)
+		{
+			m_massSpecDataStats = massSpecDataStats;
+		}
+
+
 	//! Set the name of the file to load.
 	/*!
-	
+
 		\param fileName name of the file.
-	
-	 */
+
+*/
 	void
 		MassSpecDataFileLoader::setFileName(const QString &fileName)
 		{
@@ -86,10 +95,10 @@ namespace msXpSmineXpert
 	/*!
 
 		Append a new error message string to \c m_errorMessageList.
-	
+
 		\param errorMsg new message string.
-	
-	 */
+
+*/
 	void
 		MassSpecDataFileLoader::appendErrorMessage(QString errorMsg)
 		{
@@ -99,10 +108,10 @@ namespace msXpSmineXpert
 
 	//! Cancel the running operation.
 	/*!
-	
+
 		This function can be invoked to ask the loader to stop loading mass data from the file.
-	
-	 */
+
+*/
 	void
 		MassSpecDataFileLoader::cancelOperation()
 		{
@@ -115,17 +124,17 @@ namespace msXpSmineXpert
 
 	//! Tell if the operation was cancelled.
 	/*!
-	
+
 		This function can be used to enquire if the user has cancelled the data
 		loading operation. It is typically used from inside a \e for loop. If true,
 		the loop would exit.
-	
+
 		\return true if the \c m_isOperationCancelle value is true.
-	
+
 		\sa cancelOperation().
-	
-	 */
-	bool 
+
+*/
+	bool
 		MassSpecDataFileLoader::isOperationCancelled()
 		{
 			return m_isOperationCancelled;
