@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #pragma once
 
 
@@ -47,6 +48,7 @@
 #include <minexpert/nongui/globals.hpp>
 #include <minexpert/nongui/MassSpecDataSet.hpp>
 #include <minexpert/nongui/History.hpp>
+#include <minexpert/nongui/MassSpecDataStats.hpp>
 
 namespace msXpSmineXpert
 {
@@ -81,6 +83,12 @@ namespace msXpSmineXpert
 			// stop the current operation.
 			bool m_isOperationCancelled = false;
 
+			//! Tells if the binning setup was performed already for streamed mode mz integrations
+			bool m_isBinningSetup = false;
+
+			//! Mass spectral data statistics for use while loading mass data in stream mode
+			MassSpecDataStats m_massSpecDataStats;
+
 		public:
 
 			//! Tells if the feedback should go the the standard output.
@@ -89,6 +97,8 @@ namespace msXpSmineXpert
 			MassSpecDataFileLoader(const QString &fileName = QString());
 			virtual ~MassSpecDataFileLoader();
 
+			void setMassSpecDataStats(const MassSpecDataStats &massSpecDataStats);
+
 			void appendErrorMessage(QString errorMsg);
 			void setFileName(const QString &fileName);
 			virtual QString formatDescription() const = 0;
