@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #include <omp.h>
 
 #include <QtSql>
@@ -321,9 +322,9 @@ namespace msXpSmineXpert
 		spectra loaded from file. This number is 1-based.
 
 */
-	void 
+	void
 		MassSpecDataFileLoaderPwiz::updateLoadProgression(int count)
-		{	
+		{
 			// Maintain a count of the parsed spectra, so that we can give the
 			// user a feedback each 100 spectra.
 
@@ -342,7 +343,7 @@ namespace msXpSmineXpert
 					.arg(m_spectrumIndex + 1)
 					.arg(m_totalSkippedSpectra);
 
-				emit dataLoadMessageSignal(msg, LOG_TO_BOTH_OVERWRITE);
+				// emit dataLoadMessageSignal(msg, LOG_TO_BOTH_OVERWRITE);
 				emit updateFeedbackSignal(msg, m_spectrumIndex + 1, LogType::LOG_TO_STATUS_BAR);
 
 				// We may also use this function with a terminal-only program,
@@ -362,7 +363,7 @@ namespace msXpSmineXpert
 
 		This class can load data to a number of different destinations.  In this
 		function, the destination of the data is the \c mp_massSpecDataSet
-		MassSpecDataSet pointer. 
+		MassSpecDataSet pointer.
 
 		Note that the generated MassSpectrum entities that are stored in \c
 		mp_massSpecDataSet <em>are not owned</em> by \c this instance.
@@ -381,6 +382,11 @@ namespace msXpSmineXpert
 			// each read spectrum. Or we are reading the file in streamed
 			// mode, in which case, we need to process the data a
 			// bit and then let them go.
+			//
+			// Whatever the case, the statistics about the mass data are performed in
+			// two steps: while loading data (streamed or not), each spectrum is used
+			// to increment the statistical data set. At the end of the file loading,
+			// the statistics need to be consolidated.
 
 			// We will need the mass spectrum that was parsed lastly, right before
 			// calling this function. The pointer to that spectrum cannot be
@@ -389,6 +395,18 @@ namespace msXpSmineXpert
 			if(mp_newSpectrum == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
+			// This is the moment to increment the statistics with the data in the new
+			// mass spectrum. It will be necessary to consolidate the statistics at
+			// the end of the file loading.
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Increment statistics for spectrum index" << m_spectrumIndex;
+
+			mp_massSpecDataSet->incrementStatistics(*mp_newSpectrum);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Done increment statistics for spectrum index" << m_spectrumIndex;
+
 			if(mp_massSpecDataSet->isStreamed())
 			{
 
@@ -400,8 +418,7 @@ namespace msXpSmineXpert
 
 				double newTic = mp_newSpectrum->valSum();
 
-				// Had already another spectrum have the same rt ? If so another
-				// TIC was
+				// Had already another spectrum have the same rt ? If so another TIC was
 				// already computed.
 
 				double oldTic =
@@ -453,18 +470,12 @@ namespace msXpSmineXpert
 					{
 						msXpSlibmass::MassSpectrum *newSpectrum = new msXpSlibmass::MassSpectrum;
 
-						// This is the first spectrum that we set to the hash that has
-						// the
-						// current dt value. We want to combine all the spectra that
-						// have
-						// the same dt. We need to store the spectrum so that we can
-						// later
-						// refer to it, to fill in the colormap. But remember, we want
-						// to
-						// make a calculation such that there are less MassPeak elements
-						// in
-						// the spectra. So we need to allocated a new spectrum and
-						// combine
+						// This is the first spectrum that we set to the hash that has the
+						// current dt value. We want to combine all the spectra that have
+						// the same dt. We need to store the spectrum so that we can later
+						// refer to it, to fill in the colormap. But remember, we want to
+						// make a calculation such that there are less MassPeak elements in
+						// the spectra. So we need to allocated a new spectrum and combine
 						// with rounding (0 decimals) the newSpectrum in it.
 
 						newSpectrum->rdecimalPlaces() = 0; // See comment above.
@@ -545,6 +556,9 @@ namespace msXpSmineXpert
 				}
 				// End of
 				// if(m_curDriftTime != -1)
+
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "Done loading one spectrum in stream mode.";
 			}
 			// End of
 			// if(mp_massSpecDataSet->isStreamed())
@@ -624,7 +638,7 @@ namespace msXpSmineXpert
 			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 			//<< "m_integrationType:" << m_integrationType;
 
-			if(m_integrationType == IntegrationType::ANY_TO_RT)
+			if(m_integrationType == IntegrationType::XXX_TO_RT)
 			{
 				// We want to compute a TIC chrom.
 				return integrateToRt();
@@ -632,6 +646,33 @@ namespace msXpSmineXpert
 			else if(m_integrationType == IntegrationType::ANY_TO_MZ)
 			{
 				// We want to compute a mass spectrum.
+
+				// We are performing a combination of various mass spectra (the
+				// mp_spectrum contains the spectra that are read from file) into a single
+				// mass spectrum, the combined mass spectrum m_massSpectrum.
+				// When performing combinations, the user might configure the way the
+				// binning (or no binning) should be setup.
+
+				if(!m_isBinningSetup && m_specPeakCount)
+				{
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+						<< "Setting up the bins.";
+
+					MzIntegrationParams mzIntegrationParams = m_history.mzIntegrationParams();
+
+					m_massSpectrum.setMinMz(m_massSpecDataStats.m_minMz);
+					m_massSpectrum.setMaxMz(m_massSpecDataStats.m_maxMz);
+
+					m_massSpectrum.setBinningType(mzIntegrationParams.m_binningType);
+					m_massSpectrum.setBinSize(mzIntegrationParams.m_binSize);
+					m_massSpectrum.setBinSizeType(mzIntegrationParams.m_binSizeType);
+					m_massSpectrum.applyMzShift(mzIntegrationParams.m_applyMzShift);
+					m_massSpectrum.setRemoveZeroValDataPoints(mzIntegrationParams.m_removeZeroValDataPoints);
+
+					if(m_massSpectrum.setupBinnability(*mp_newSpectrum))
+						m_isBinningSetup = true;
+				}
+
 				return integrateToMz();
 			}
 			else if(m_integrationType == IntegrationType::ANY_TO_DT)
@@ -691,7 +732,7 @@ namespace msXpSmineXpert
 					.arg(m_fileName)
 					.arg(m_spectrumIndex);
 
-				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 					<< "The xy file name is empty. Crafted file name:" << destFileName;
 			}
 			else
@@ -708,7 +749,7 @@ namespace msXpSmineXpert
 						.arg(QFileInfo(m_fileName).fileName())
 						.arg(m_spectrumIndex);
 
-					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 						<< "The xy file name is a directory. Crafted file name:" << destFileName;
 				}
 				else
@@ -716,7 +757,7 @@ namespace msXpSmineXpert
 					// The name is for a file.
 					destFileName = m_xyFileName;
 
-					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 						<< "The xy file name is a file. Using it as is." << destFileName;
 				}
 			}
@@ -741,7 +782,7 @@ namespace msXpSmineXpert
 		The data are loaded from the file in order to craft a TIC chromatogram.
 
 		When running the integration, the \c m_history History instance member is
-		checked. Any integration listed in the History is taken into account. 
+		checked. Any integration listed in the History is taken into account.
 
 		The <rt,tic> value pairs are stocked in the \c m_keyValMap, that will later
 		be used to craft two <rt> and <tic> vectors. Indeed, this function is called
@@ -872,7 +913,7 @@ namespace msXpSmineXpert
 		The data are loaded from the file in order to craft a mass spectrum.
 
 		When running the integration, the \c m_history History instance member is
-		checked. Any integration listed in the History is taken into account. 
+		checked. Any integration listed in the History is taken into account.
 
 		The <m/z,i> value pairs are stocked by combination in the \c m_massSpectrum
 		MassSpectrum instance, that will later be used to craft two <mz> and <i>
@@ -966,11 +1007,9 @@ namespace msXpSmineXpert
 			//qDebug() << __FILE__ << __LINE__
 			//<< "mp_newSpectrum has size:" << mp_newSpectrum->size();
 
-			// Now the more specific mz integration
 			if(mzIntegration)
-			{
+				// The history contains an item about mz ranges...
 				m_massSpectrum.combine(*mp_newSpectrum, mzStart, mzEnd);
-			}
 			else
 				m_massSpectrum.combine(*mp_newSpectrum);
 
@@ -987,7 +1026,7 @@ namespace msXpSmineXpert
 		The data are loaded from the file in order to craft a drift spectrum.
 
 		When running the integration, the \c m_history History instance member is
-		checked. Any integration listed in the History is taken into account. 
+		checked. Any integration listed in the History is taken into account.
 
 		The <dt,i> value pairs are stocked by combination in the \c m_keyValMap
 		member, that will later be used to craft two <dt> and <i> vectors. Indeed,
@@ -1234,7 +1273,10 @@ namespace msXpSmineXpert
 						"Program aborted.",
 						__FILE__, __LINE__);
 
-			emit setupFeedbackSignal("Parsing mzML file", 1, m_spectrumCount, -1, LogType::LOG_TO_BOTH);
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "The file contains" << m_spectrumCount << "spectra.";
+
+			emit setupFeedbackSignal("Parsing mzML/mzXML file", 1, m_spectrumCount, -1, LogType::LOG_TO_BOTH);
 
 			mp_newSpectrum = Q_NULLPTR;
 
@@ -1345,8 +1387,8 @@ namespace msXpSmineXpert
 
 							if(qIsNaN(m_curDriftTime))
 							{
-								qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
-									<< "Found a NaN drift time at index " << m_spectrumIndex << ". Ignoring it.";
+								//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+									//<< "Found a NaN drift time at index " << m_spectrumIndex << ". Ignoring it.";
 
 								wasNaN = true;
 								break;
@@ -1393,9 +1435,9 @@ namespace msXpSmineXpert
 								"Only handling the first one.";
 					}
 					//else if(scanWindowCount == 0)
-						//qDebug()
-							//<< __FILE__ << __LINE__
-							//<< "There is not a single scan window for this spectrum.";
+					//qDebug()
+					//<< __FILE__ << __LINE__
+					//<< "There is not a single scan window for this spectrum.";
 				}
 				// End of
 				// for(uint lter = 0; lter < scanList.scans.size(); ++lter)
@@ -1462,6 +1504,8 @@ namespace msXpSmineXpert
 						}
 					}
 				}
+				// End of
+				// if(!m_sqlite3Handler.m_dbFileName.isEmpty())
 
 
 				// Finally we can handle the mass spectral data!
@@ -1469,7 +1513,7 @@ namespace msXpSmineXpert
 				// This function might be run when the user wants to calculate a TIC
 				// intensity value for the data at hand during a data analysis session.
 				// In this specific case, we need to account for restricted ranges in
-				// history (we are not simply loadind data from a file, *the whole data
+				// history (we are not simply loading data from a file, *the whole data
 				// set should be loaded in this case*).
 
 				if(m_isLoadToTicIntensity)
@@ -1534,6 +1578,8 @@ namespace msXpSmineXpert
 					// For the MZ integration, we'll test it later, once the spectrum has
 					// actually been created.
 				}
+				// End of
+				// if(m_isLoadToTicIntensity)
 
 				BinaryDataArrayPtr mzArrayPtr = spectrumPtr->getMZArray();
 				BinaryDataArrayPtr iArrayPtr = spectrumPtr->getIntensityArray();
@@ -1567,6 +1613,9 @@ namespace msXpSmineXpert
 						QVector<double>::fromStdVector(mzArrayPtr->data),
 						QVector<double>::fromStdVector(iArrayPtr->data));
 
+				//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				//<< "The newly loaded spectrum has" << mp_newSpectrum->size() << "peaks";
+
 				// At this point, depending on the task at hand we have the following
 				// possibilities:
 				//
@@ -1603,17 +1652,17 @@ namespace msXpSmineXpert
 					// Do not touch the mp_newSpectrum because it now belongs to
 					// MassSpecDataSet.
 
-					// qDebug() << __FILE__ << __LINE__
+					//qDebug() << __FILE__ << __LINE__
 					//<< "Done loading spectrum to mass spec data set:"
 					//<< "m_spectrumIndex:" << m_spectrumIndex
 					//<< "m_scanStartTime:" << m_scanStartTime;
 				}
 				else if(m_isLoadToXyFilesInSequence)
 				{
-					//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ 
+					//qDebug() << __FILE__ << __LINE__ << __FUNCTION__
 					//<< "We are parsing to xy files in sequence.";
 
-					int res = loadSpectrumToXyFilesInSequence(); 
+					int res = loadSpectrumToXyFilesInSequence();
 
 					// We do not need the allocated spectrum anymore, and we are loading
 					// not to MassSpecDataSet, so delete it.
@@ -1645,7 +1694,7 @@ namespace msXpSmineXpert
 					// qDebug() << __FILE__ << __LINE__
 					//<< "We are parsing to the vectors.";
 
-					int res = loadSpectrumToVectors(); 
+					int res = loadSpectrumToVectors();
 
 					// We do not need the allocated spectrum anymore, and we are loading
 					// not to MassSpecDataSet, so delete it.
@@ -1771,7 +1820,7 @@ namespace msXpSmineXpert
 
 					m_sqlite3Handler.writeDbSpectralData(
 							m_spectrumIndex, m_spectrumTitle, m_specPeakCount, m_msLevel,
-							m_tic, m_scanStartTime, m_driftDataId, 
+							m_tic, m_scanStartTime, m_driftDataId,
 							m_scanWindowLowerLimit, m_scanWindowUpperLimit,
 							mzQByteArray, iQByteArray);
 
@@ -1852,10 +1901,18 @@ namespace msXpSmineXpert
 				// << spectrum->valSum();
 				//}
 			}
+			else if(m_isLoadToVectors)
+			{
+				// We now need to conclude the combination of all the spectra into
+				// m_spectrum and make sure we remove all the zero-val data points if
+				// that was required.
+				if(m_massSpectrum.isRemoveZeroValDataPoints())
+					m_massSpectrum.removeZeroValDataPoints();
+			}
 
-			//qWarning() << __FILE__ << __LINE__
-				//<< "File loading done with a number of stored spectra:" 
-				//<< m_storedSpectraCount;
+			qWarning() << __FILE__ << __LINE__
+				<< "File loading done with a number of stored spectra:"
+				<< m_storedSpectraCount;
 
 			return m_storedSpectraCount;
 		}
@@ -1916,7 +1973,7 @@ namespace msXpSmineXpert
 
 		When loading a mass spectrometry data file, the data are stored in various
 		structures all belonging to the MassSpecDataSet instance passed as
-		parameter.	
+		parameter.
 
 		\param massSpecDataSet pointer to a MassSpecDataSet instance in which to
 		store the mass data loaded from file.
@@ -1944,7 +2001,7 @@ namespace msXpSmineXpert
 		\return true if the format of the file can be read by \c this loader.
 
 */
-	int 
+	int
 		MassSpecDataFileLoaderPwiz::canLoadData(QString fileName,
 				MassSpecDataFileLoaderPwiz *pwiz)
 		{
@@ -2098,7 +2155,7 @@ namespace msXpSmineXpert
 		}
 
 	// FIXME doc.
-	int 
+	int
 		MassSpecDataFileLoaderPwiz::loadData(const QString &xyFileName, int startIndex, int endIndex)
 		{
 			m_xyFileName = xyFileName;
@@ -2141,7 +2198,7 @@ namespace msXpSmineXpert
 		(type and range) to be taken into account during the loading of the mass
 		spectra. At the end of the file parsing process, the result of all the
 		successive integrations is converted into a <key, value> pair in the form of
-		two same-sized vectors, \p keyVector and \p valVector. 
+		two same-sized vectors, \p keyVector and \p valVector.
 
 		For example, if an integration of the data file spectra is required such as
 		to create a TIC chromatogram, the \p keyVector will contain the retention
@@ -2252,7 +2309,7 @@ namespace msXpSmineXpert
 					valVector->append(m_keyValMap.value(key));
 				}
 			}
-			else if(m_integrationType == IntegrationType::ANY_TO_RT)
+			else if(m_integrationType == IntegrationType::XXX_TO_RT)
 			{
 				// The data were stored in m_keyValMap in the form of (rt, tic) pairs.
 
@@ -2267,7 +2324,7 @@ namespace msXpSmineXpert
 			}
 			else if(m_integrationType == IntegrationType::ANY_TO_TIC_INT)
 			{
-				// The m_ticIntensity value was updated.	
+				// The m_ticIntensity value was updated.
 				*intensity = m_ticIntensity;
 				m_ticIntensity = 0.0;
 			}
@@ -2299,7 +2356,7 @@ namespace msXpSmineXpert
 
 		The mass spectrometry data are loaded from the file. This function sets the
 		stage for the loadData() function to actually start the job, which in turn
-		will call the private loadData() function. 
+		will call the private loadData() function.
 
 		This function mimicks the loadData(const History &history, IntegrationType integrationType,
 		QVector<double> *keyVector, QVector<double> *valVector,
@@ -2344,13 +2401,11 @@ namespace msXpSmineXpert
 					<< "The History is not valid. Returning.";
 
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
-
-				return -1;
 			}
 
-			if(integrationType != IntegrationType::ANY_TO_MZ &&
-					integrationType != IntegrationType::ANY_TO_DT &&
-					integrationType != IntegrationType::ANY_TO_RT)
+			if(integrationType != IntegrationType::XXX_TO_MZ &&
+					integrationType != IntegrationType::XXX_TO_DT &&
+					integrationType != IntegrationType::XXX_TO_RT)
 			{
 				qDebug() << __FILE__ << __LINE__ << "No valid integration type is "
 					"defined for the data to be "
@@ -2379,7 +2434,7 @@ namespace msXpSmineXpert
 
 		The mass spectrometry data are loaded from the file. This function sets the
 		stage for the loadData() function to actually start the job, which in turn
-		will call the private loadData() function. 
+		will call the private loadData() function.
 
 		This function mimicks the loadData(const History &history, IntegrationType
 		integrationType, QVector<double> *keyVector, QVector<double> *valVector,
