@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include <omp.h>
 
 
@@ -140,7 +141,7 @@ namespace msXpSmineXpert
 		The SQL statement is crafted around criteria defined in the \p history
 		History instance. Note that, if \p withRecordCountStatement is true, then
 		the SQL statement includes a COUNT request. This is useful for determining
-		the number of records that match the SQL statement. 
+		the number of records that match the SQL statement.
 
 		The \p integrateToDt parameter tells if the SQL statement should include
 		drift mobility requests. This parameter is required because it might be
@@ -160,7 +161,7 @@ namespace msXpSmineXpert
 		\sa
 
 */
-	QString 
+	QString
 		MassSpecDataFileLoaderSqlite3::craftSqlSelectStatementForStreamedIntegration(const History &history,
 				bool withRecordCountStatement, bool integrateToDt)
 		{
@@ -238,11 +239,11 @@ namespace msXpSmineXpert
 			// history item. Thus, we rely on the caller to set to true the
 			// integrateToDt parameter if we require drift time data in the SQL
 			// output.
-			// 
+			//
 			// We leave that part to condFields, as this is conditional stuff (note
 			// how the comma is at the beginning now).
 
-			if(integrateToDt || dtIntegration) 
+			if(integrateToDt || dtIntegration)
 				condFields += ", driftTime ";
 
 			fromStatement += "FROM spectralData ";
@@ -326,7 +327,7 @@ namespace msXpSmineXpert
 			// Now craft the whole query depending on the needs.
 
 			// First ensure we get the actual number of records (note the
-			// countStatement element). This query string is used 
+			// countStatement element). This query string is used
 			srcDbQueryString = openingStatement + countStatement + uncondFields +
 				condFields + fromStatement + joinStatements +
 				whereStatements;
@@ -363,7 +364,7 @@ namespace msXpSmineXpert
 
 */
 	int
-		MassSpecDataFileLoaderSqlite3::loadSpectra(int firstId, 
+		MassSpecDataFileLoaderSqlite3::loadSpectra(int firstId,
 				int lastId, MassSpecDataSet *massSpecDataSet, int &dataKind)
 		{
 			// We do not want to continue doing that work if the user cancelled the
@@ -427,9 +428,9 @@ namespace msXpSmineXpert
 */
 	int
 		MassSpecDataFileLoaderSqlite3::loadSpectra(const QString &sqlQueryString,
-				MassSpecDataSet *massSpecDataSet, int &dataKind)
+				MassSpecDataSet *p_massSpecDataSet, int &dataKind)
 		{
-			if(massSpecDataSet == nullptr)
+			if(p_massSpecDataSet == nullptr)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			if(!m_sqlite3Handler.initializeDbForRead())
@@ -529,7 +530,7 @@ namespace msXpSmineXpert
 
 				int newDataKind = MassDataKind::MASS_SPECTRA;
 				newSpectraAllocated =
-					loadSpectra(sqlQueryString, massSpecDataSet, newDataKind);
+					loadSpectra(sqlQueryString, p_massSpecDataSet, newDataKind);
 
 				if(newSpectraAllocated != -1)
 				{
@@ -577,15 +578,16 @@ namespace msXpSmineXpert
 				QByteArray *p_enc64MzByteArray = mzByteArrayList.at(iter);
 				QByteArray *p_enc64IByteArray = iByteArrayList.at(iter);
 
-				msXpSlibmass::MassSpectrum *newSpectrum = new msXpSlibmass::MassSpectrum;
-				newSpectrum->initialize(p_enc64MzByteArray, p_enc64IByteArray, compressionType);
+				msXpSlibmass::MassSpectrum *p_newSpectrum = new msXpSlibmass::MassSpectrum;
 
-				if(newSpectrum == Q_NULLPTR)
+				if(p_newSpectrum == Q_NULLPTR)
 					qFatal("Fatal error at %s@%d -- %s. "
 							"Could not create a new mass spectrum from base64-encoded byte arrays."
 							"Program aborted.",
 							__FILE__, __LINE__, __FUNCTION__);
 
+				p_newSpectrum->initialize(p_enc64MzByteArray, p_enc64IByteArray, compressionType);
+
 #if 0
 				if(iter == 47)
 				{
@@ -604,7 +606,7 @@ namespace msXpSmineXpert
 
 					file1.close();
 
-					QString *text = newSpectrum->asText();
+					QString *text = p_newSpectrum->asText();
 
 					if(text != Q_NULLPTR)
 					{
@@ -623,7 +625,7 @@ namespace msXpSmineXpert
 						delete text;
 					}
 				}
-				#endif
+#endif
 
 				// qDebug() << __FILE__ << __LINE__
 				// << "Allocated new spectrum of " << mzList.size() << "peaks";
@@ -636,18 +638,24 @@ namespace msXpSmineXpert
 
 #pragma omp critical
 				{
-					if(massSpecDataSet->isStreamed())
+
+					// This is the moment to increment the statistics with the data in the new
+					// mass spectrum. It will be necessary to consolidate the statistics at
+					// the end of the file loading.
+					p_massSpecDataSet->incrementStatistics(*p_newSpectrum);
+
+					if(p_massSpecDataSet->isStreamed())
 					{
 						// This is the chance we have to calculate the TIC for the
 						// spectrum:
 
-						double newTic = newSpectrum->valSum();
+						double newTic = p_newSpectrum->valSum();
 
 						// Had already another spectrum have the same rt ? If so another
 						// TIC was already computed.
 
 						double oldTic =
-							massSpecDataSet->m_ticChromMap.value(rtList.at(iter), qSNaN());
+							p_massSpecDataSet->m_ticChromMap.value(rtList.at(iter), qSNaN());
 
 						if(!qIsNaN(oldTic))
 						{
@@ -663,9 +671,9 @@ namespace msXpSmineXpert
 						// TIC chromatogram.
 
 						// qDebug() << __FILE__ << __LINE__
-						//<< "Inserting rt/tic in massSpecDataSet->m_ticChromMap.";
+						//<< "Inserting rt/tic in p_massSpecDataSet->m_ticChromMap.";
 
-						massSpecDataSet->m_ticChromMap.insert(rtList.at(iter), newTic);
+						p_massSpecDataSet->m_ticChromMap.insert(rtList.at(iter), newTic);
 
 						// At this point, handle the case of the colorMap that relates all
 						// the dt with the corresponding (mz,i) pairs, that is, each dt
@@ -681,7 +689,7 @@ namespace msXpSmineXpert
 							// encountered (and possibly undergoing combinations).
 
 							msXpSlibmass::Trace *p_trace =
-								massSpecDataSet->m_dtHash.value(dtList.at(iter), nullptr);
+								p_massSpecDataSet->m_dtHash.value(dtList.at(iter), nullptr);
 
 							if(p_trace == Q_NULLPTR)
 							{
@@ -693,20 +701,20 @@ namespace msXpSmineXpert
 								// we want to make a calculation such that there are less
 								// MassPeak elements in the spectra. So we need to allocated a
 								// new spectrum and combine with rounding (0 decimals) the
-								// newSpectrum in it.
+								// p_newSpectrum in it.
 
 								msXpSlibmass::MassSpectrum *spectrum = new msXpSlibmass::MassSpectrum;
 								spectrum->rdecimalPlaces() = 0;
-								spectrum->combine(*newSpectrum);
-								massSpecDataSet->m_massSpectra.append(spectrum);
+								spectrum->combine(*p_newSpectrum);
+								p_massSpecDataSet->m_massSpectra.append(spectrum);
 
-								// Delete the newSpectrum that we do not use anymore.
-								delete newSpectrum;
+								// Delete the p_newSpectrum that we do not use anymore.
+								delete p_newSpectrum;
 
 								// And now make the hash association with the current
 								// drift time
 								// value.
-								massSpecDataSet->m_dtHash.insert(dtList.at(iter),
+								p_massSpecDataSet->m_dtHash.insert(dtList.at(iter),
 										spectrum);
 
 								// Finally, let's store the first mz value and the last
@@ -714,13 +722,13 @@ namespace msXpSmineXpert
 								// Only if the values need an update.
 								double mz_first = spectrum->first()->key();
 
-								if(mz_first < massSpecDataSet->m_mzRangeStart)
-									massSpecDataSet->m_mzRangeStart = mz_first;
+								if(mz_first < p_massSpecDataSet->m_mzRangeStart)
+									p_massSpecDataSet->m_mzRangeStart = mz_first;
 
 								double mz_last = spectrum->last()->key();
 
-								if(mz_last > massSpecDataSet->m_mzRangeEnd)
-									massSpecDataSet->m_mzRangeEnd = mz_last;
+								if(mz_last > p_massSpecDataSet->m_mzRangeEnd)
+									p_massSpecDataSet->m_mzRangeEnd = mz_last;
 
 								// We will need to know what is the number of cells on the
 								// mz
@@ -728,8 +736,8 @@ namespace msXpSmineXpert
 
 								int spectrumSize = spectrum->size();
 
-								if(spectrumSize > massSpecDataSet->m_mzCellCount)
-									massSpecDataSet->m_mzCellCount = spectrumSize;
+								if(spectrumSize > p_massSpecDataSet->m_mzCellCount)
+									p_massSpecDataSet->m_mzCellCount = spectrumSize;
 							}
 							else
 							{
@@ -738,44 +746,44 @@ namespace msXpSmineXpert
 								// So use it to combine the newly generated one.
 
 								p_trace->rdecimalPlaces() = 0;
-								p_trace->combine(*newSpectrum);
+								p_trace->combine(*p_newSpectrum);
 
 								// Since we do not use this spectrum anymore, we can
 								// delete it.
-								delete newSpectrum;
+								delete p_newSpectrum;
 
 								// Finally, let's store the first mz value and the last
 								// one.
 								// Only if the values need an update.
 								double mz_first = p_trace->first()->key();
 
-								if(mz_first < massSpecDataSet->m_mzRangeStart)
-									massSpecDataSet->m_mzRangeStart = mz_first;
+								if(mz_first < p_massSpecDataSet->m_mzRangeStart)
+									p_massSpecDataSet->m_mzRangeStart = mz_first;
 
 								double mz_last = p_trace->last()->key();
 
-								if(mz_last > massSpecDataSet->m_mzRangeEnd)
-									massSpecDataSet->m_mzRangeEnd = mz_last;
+								if(mz_last > p_massSpecDataSet->m_mzRangeEnd)
+									p_massSpecDataSet->m_mzRangeEnd = mz_last;
 
 								// We will need to know what is the number of cells on the mz
 								// axis of the colormap matrix.
 
 								int spectrumSize = p_trace->size();
 
-								if(spectrumSize > massSpecDataSet->m_mzCellCount)
-									massSpecDataSet->m_mzCellCount = spectrumSize;
+								if(spectrumSize > p_massSpecDataSet->m_mzCellCount)
+									p_massSpecDataSet->m_mzCellCount = spectrumSize;
 							}
 						}
 
 						// End of
 						// if(dataKind == MassDataKind::DRIFT_SPECTRA)
 					} // End if
-					// if(massSpecDataSet->isStreamed())
+					// if(p_massSpecDataSet->isStreamed())
 					else
 					{
 						// The data load mode is not streaming.
-						massSpecDataSet->m_massSpectra.append(newSpectrum);
-						newSpectrum->rrt() = rtList.at(iter);
+						p_massSpecDataSet->m_massSpectra.append(p_newSpectrum);
+						p_newSpectrum->rrt() = rtList.at(iter);
 
 						// Now that we have finished filling-in the mass spectrum, we need
 						// to
@@ -785,13 +793,13 @@ namespace msXpSmineXpert
 						// drift
 						// time (dtHash).
 
-						massSpecDataSet->m_rtHash.insert(rtList.at(iter), newSpectrum);
+						p_massSpecDataSet->m_rtHash.insert(rtList.at(iter), p_newSpectrum);
 
 						if(dataKind == MassDataKind::DRIFT_SPECTRA)
 						{
-							newSpectrum->rdt() = dtList.at(iter);
-							massSpecDataSet->m_dtHash.insert(dtList.at(iter),
-									newSpectrum);
+							p_newSpectrum->rdt() = dtList.at(iter);
+							p_massSpecDataSet->m_dtHash.insert(dtList.at(iter),
+									p_newSpectrum);
 						}
 
 						// qDebug() << __FILE__ << __LINE__
@@ -931,6 +939,10 @@ namespace msXpSmineXpert
 				msXpSlibmass::MassSpectrum *newSpectrum = new msXpSlibmass::MassSpectrum;
 				newSpectrum->initialize(p_enc64MzByteArray, p_enc64IByteArray, compressionType);
 
+				// FIXME try removing the O-m_val data points to help with Thermo
+				// Orbitrap data during combination later (binning hell).
+				//newSpectrum->removeZeroValDataPoints();
+
 				// qDebug() << __FILE__ << __LINE__
 				// << "Allocated new spectrum of " << mzList.size() << "peaks";
 
@@ -1184,7 +1196,7 @@ namespace msXpSmineXpert
 
 				// qDebug() << __FILE__ << __LINE__ << msg;
 			}
-			// End of 
+			// End of
 			// while(sqlQuery->next())
 
 			// Delete the query instance because it holds lots of stuff.
@@ -1381,8 +1393,8 @@ namespace msXpSmineXpert
 
 				// And now actually perform the data crunching
 
-				msXpSlibmass::MassSpectrum *newSpectrum = new msXpSlibmass::MassSpectrum;
-				newSpectrum->initialize(p_enc64MzByteArray, p_enc64IByteArray, compressionType);
+				msXpSlibmass::MassSpectrum *p_newSpectrum = new msXpSlibmass::MassSpectrum;
+				p_newSpectrum->initialize(p_enc64MzByteArray, p_enc64IByteArray, compressionType);
 
 				// Free the memory that we do not use anymore.
 				delete p_enc64MzByteArray;
@@ -1391,12 +1403,36 @@ namespace msXpSmineXpert
 				// qDebug() << __FILE__ << __LINE__
 				// << "Combined new spectrum of " << newSpectrum->size() << "peaks";
 
-				if(mzIntegration)
-					combinedMassSpectrum.combine(*newSpectrum, mzStart, mzEnd);
-				else
-					combinedMassSpectrum.combine(*newSpectrum);
+				// We are performing a combination of various mass spectra (the
+				// mp_spectrum contains the spectra that are read from file) into a single
+				// mass spectrum, the combined mass spectrum m_massSpectrum.
+				// When performing combinations, the user might configure the way the
+				// binning (or no binning) should be setup.
 
-				delete newSpectrum;
+				if(!m_isBinningSetup && p_newSpectrum->size())
+				{
+					MzIntegrationParams mzIntegrationParams = history.mzIntegrationParams();
+
+					combinedMassSpectrum.setMinMz(m_massSpecDataStats.m_minMz);
+					combinedMassSpectrum.setMaxMz(m_massSpecDataStats.m_maxMz);
+
+					combinedMassSpectrum.setBinningType(mzIntegrationParams.m_binningType);
+					combinedMassSpectrum.setBinSize(mzIntegrationParams.m_binSize);
+					combinedMassSpectrum.setBinSizeType(mzIntegrationParams.m_binSizeType);
+					combinedMassSpectrum.applyMzShift(mzIntegrationParams.m_applyMzShift);
+					combinedMassSpectrum.setRemoveZeroValDataPoints(mzIntegrationParams.m_removeZeroValDataPoints);
+
+					if(combinedMassSpectrum.setupBinnability(*p_newSpectrum))
+						m_isBinningSetup = true;
+				}
+
+				if(mzIntegration)
+					// The history contains at least one element that limits the m/z range.
+					combinedMassSpectrum.combine(*p_newSpectrum, mzStart, mzEnd);
+				else
+					combinedMassSpectrum.combine(*p_newSpectrum);
+
+				delete p_newSpectrum;
 
 				// At this point we can increment the number of spectra that were
 				// effectively read.
@@ -1412,6 +1448,12 @@ namespace msXpSmineXpert
 			// Delete the query instance because it holds lots of stuff.
 			delete sqlQuery;
 
+			// We now need to conclude the combination of all the spectra into
+			// m_spectrum and make sure we remove all the zero-val data points if
+			// that was required.
+			if(combinedMassSpectrum.isRemoveZeroValDataPoints())
+				combinedMassSpectrum.removeZeroValDataPoints();
+
 			// Now actually set the values to the vectors passed as parameter.
 
 			*keyVector = QVector<double>::fromList(combinedMassSpectrum.keyList());
@@ -1752,7 +1794,7 @@ namespace msXpSmineXpert
 	/*!
 
 		Reads the metaData table of the file and determines the size of the spectrum
-		list. Note that the \c m_spectrumCount member is set to that value. 
+		list. Note that the \c m_spectrumCount member is set to that value.
 
 		\return an integer containing the number of spectra in the spectrum list or
 		-1 upon error.
@@ -1868,7 +1910,7 @@ namespace msXpSmineXpert
 			emit dataLoadMessageSignal(msg, LOG_TO_BOTH);
 
 			//qDebug() << __FILE__ << __LINE__
-				//<< "Loading data: total number of spectra to load:" << m_spectrumCount;
+			//<< "Loading data: total number of spectra to load:" << m_spectrumCount;
 
 			// The way this function works is that is loads the spectra at positions
 			// firstId to lastId in the spectralData table. Each time a new spectrum
@@ -1895,7 +1937,7 @@ namespace msXpSmineXpert
 
 				msg = QString("Loading data, spectra %1 to %2\n").arg(iter).arg(lastId);
 
-				emit dataLoadMessageSignal(msg, LOG_TO_BOTH_OVERWRITE);
+				//emit dataLoadMessageSignal(msg, LOG_TO_BOTH_OVERWRITE);
 				emit dataLoadProgressSignal(lastId);
 
 				// qDebug() << __FILE__ << __LINE__
@@ -1923,7 +1965,7 @@ namespace msXpSmineXpert
 
 		The data are loaded from the database file using \p history to craft the SQL
 		statement and integrate the loaded data into the \p integrationType. Put the
-		results of the integration to \p keyVector and \p valVector.  
+		results of the integration to \p keyVector and \p valVector.
 
 		IntegrationType can be RT, MZ or DT, yielding respectively a TIC chromatogram,
 		a mass spectrum or a drift spectrum.
@@ -1959,17 +2001,17 @@ namespace msXpSmineXpert
 			{
 				qDebug() << __FILE__ << __LINE__
 					<< "history as text:" << history.asText();
-			}	
+			}
 
 			//qDebug() << __FILE__ << __LINE__
 			//<< "At this point, the m_fileName:" << m_fileName;
 
 
-			if(integrationType == IntegrationType::ANY_TO_RT)
+			if(integrationType == IntegrationType::XXX_TO_RT)
 				return streamedRtIntegration(history, keyVector, valVector);
-			else if(integrationType == IntegrationType::ANY_TO_MZ)
+			else if(integrationType == IntegrationType::XXX_TO_MZ)
 				return streamedMzIntegration(history, keyVector, valVector);
-			else if(integrationType == IntegrationType::ANY_TO_DT)
+			else if(integrationType == IntegrationType::XXX_TO_DT)
 				return streamedDtIntegration(history, keyVector, valVector);
 			else
 			{
