@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 /////////////////////// Qt includes
 #include <QDebug>
 
@@ -67,10 +68,10 @@ namespace msXpSmineXpert
 */
 	MassSpecDataSet::MassSpecDataSet(QObject *parent, const QString &name,
 			const QString &fileName, bool isStreamed)
-		: 
-			QObject{parent}, 
-		m_name{name}, 
-		m_fileName{fileName}, 
+		:
+			QObject{parent},
+		m_name{name},
+		m_fileName{fileName},
 		m_isStreamed{isStreamed}
 	{
 	}
@@ -113,22 +114,30 @@ namespace msXpSmineXpert
 	}
 
 
+	//! Get a const reference to the mass spectra data set statistics.
+	const MassSpecDataStats &
+		MassSpecDataSet::statistics()
+		{
+			return m_massSpecDataStats;
+		}
+
+
 	//! Get the name of \c this MassSpecDataSet instance.
-	QString 
+	QString
 		MassSpecDataSet::name() const
 		{
 			return m_name;
 		}
 
 	//! Get the name of the file from which the data are loaded.
-	QString 
+	QString
 		MassSpecDataSet::fileName() const
 		{
 			return m_fileName;
 		}
 
 	//! Tells if the data must be or were loaded from file in streamed mode.
-	bool 
+	bool
 		MassSpecDataSet::isStreamed() const
 		{
 			return m_isStreamed;
@@ -136,7 +145,7 @@ namespace msXpSmineXpert
 
 
 	//! Tells if the data must be or were loaded from file in streamed mode.
-	bool 
+	bool
 		MassSpecDataSet::isMobilityExperiment() const
 		{
 			return m_dtHash.size();
@@ -144,7 +153,7 @@ namespace msXpSmineXpert
 
 
 	//! Get the mass spectrometry data format of the file.
-	int 
+	int
 		MassSpecDataSet::fileFormat() const
 		{
 			return m_fileFormat;
@@ -160,7 +169,7 @@ namespace msXpSmineXpert
 
 
 	//! Get the number of map cells in the m/z axis for the m/z=f(dt) color map.
-	double 
+	double
 		MassSpecDataSet::mzCellCount() const
 		{
 			return m_mzCellCount;
@@ -168,7 +177,7 @@ namespace msXpSmineXpert
 
 
 	//! Get the smallest m/z value of the whole dataset.
-	double 
+	double
 		MassSpecDataSet::mzRangeStart() const
 		{
 			return m_mzRangeStart;
@@ -177,13 +186,51 @@ namespace msXpSmineXpert
 
 
 	//! Get the greatest m/z value of the whole dataset.
-	double 
+	double
 		MassSpecDataSet::mzRangeEnd() const
 		{
 			return m_mzRangeEnd;
 		}
 
 
+	void
+		MassSpecDataSet::resetStatistics()
+		{
+			m_massSpecDataStats.reset();
+		}
+
+
+	const MassSpecDataStats &
+		MassSpecDataSet::incrementStatistics(const msXpSlibmass::MassSpectrum &massSpectrum)
+		{
+			m_massSpecDataStats.increment(massSpectrum);
+			return m_massSpecDataStats;
+		}
+
+
+	const MassSpecDataStats &
+		MassSpecDataSet::consolidateStatistics()
+		{
+			m_massSpecDataStats.consolidate();
+			return m_massSpecDataStats;
+		}
+
+
+	const MassSpecDataStats &
+		MassSpecDataSet::makeStatistics(const History &history)
+		{
+			m_massSpecDataStats.makeStatistics(*this, history);
+			return m_massSpecDataStats;
+		}
+
+
+	QString
+		MassSpecDataSet::statisticsAsText()
+		{
+			return m_massSpecDataStats.asText();
+		}
+
+
 	//! Get a const reference to the mass spectrum list.
 	const QList<msXpSlibmass::MassSpectrum *> &
 		MassSpecDataSet::massSpectra() const
@@ -224,6 +271,124 @@ namespace msXpSmineXpert
 		}
 
 
+
+	//! Allocate a mass spectrum list according to \c m_history using \c mp_massSpecDataSet data.
+	QList<msXpSlibmass::MassSpectrum *> *
+		MassSpecDataSet::extractMassSpectra(const History &history) const
+		{
+			if(m_isStreamed)
+			{
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"At the moment this function is not implemented."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+			}
+
+			// Prepare some variables.
+
+			bool rtIntegration = false;
+			double rtStart = qSNaN();
+			double rtEnd = qSNaN();
+
+			// We do not use the mz integration variables, because in returning a list
+			// of mass spectra we cannot modify their contents based on mz range values.
+			//
+			// bool mzIntegration = false;
+			//double mzStart = qSNaN();
+			//double mzEnd = qSNaN();
+
+			bool dtIntegration = false;
+			double dtStart = qSNaN();
+			double dtEnd = qSNaN();
+
+			rtIntegration = history.innermostRtRange(&rtStart, &rtEnd);
+
+			// See above for the explanation of the commenting out.
+			//mzIntegration = m_history.innermostMzRange(&mzStart, &mzEnd);
+
+			dtIntegration = history.innermostDtRange(&dtStart, &dtEnd);
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()";
+			//qDebug("Mass spectrum list extraction:\n"
+			//"rtIntegration: [%.6f-%.6f] \n"
+			//// "mzIntegratin: [%.6f-%.6f] \n"
+			//"dtIntegration: [%.6f-%.6f] \n",
+			//rtStart, rtEnd,
+
+			// See above for the explanation of the commenting out.
+			// mzStart, mzEnd,
+
+			//dtStart, dtEnd);
+
+			// At this point we should have qualified fully the integration ranges for
+			// the three dimensions: RT, MZ and DT.
+
+			QList<double> rtList;
+
+			if(rtIntegration)
+				m_rtHash.rangedKeys(rtList, rtStart, rtEnd);
+			else
+				rtList = m_rtHash.uniqueKeys();
+
+			qSort(rtList.begin(), rtList.end());
+
+			int rtListSize = rtList.size();
+
+			QString integrationDetails = "Mass spectrum extraction:\n";
+
+			if(rtIntegration)
+				integrationDetails += QString("RT range [%1-%2]\n")
+					.arg(rtStart, 0, 'f', 6)
+					.arg(rtEnd, 0, 'f', 6);
+
+			// In this function, we cannot apply mzIntegration, because we cannot cut a
+			// single spectrum into pieces. We can only return complete spectra, we are not
+			// computing specific intra-spectra dissections.
+			//if(mzIntegration)
+			//integrationDetails += QString("MZ range [%1-%2]\n")
+			//.arg(mzStart, 0, 'f', 6)
+			//.arg(mzEnd, 0, 'f', 6);
+
+			if(dtIntegration)
+				integrationDetails += QString("DT range [%1-%2]\n")
+					.arg(dtStart, 0, 'f', 6)
+					.arg(dtEnd, 0, 'f', 6);
+
+			QList<msXpSlibmass::MassSpectrum *> *p_massSpectra = new QList<msXpSlibmass::MassSpectrum *>;
+
+			for(int iter = 0; iter < rtListSize; ++iter)
+			{
+				double rt = rtList.at(iter);
+
+				// Append into p_massSpectra all the MassSpectrum * pointers that the
+				// rtHash has for rt key.
+				p_massSpectra->append(m_rtHash.values(rt));
+
+				// And now apply the DT filtering, if history so requires.
+				if(dtIntegration)
+				{
+					for(int jter = 0; jter < p_massSpectra->size(); ++jter)
+					{
+						msXpSlibmass::MassSpectrum *spectrum = p_massSpectra->at(jter);
+
+						if(spectrum->dt() < dtStart || spectrum->dt() > dtEnd)
+						{
+							p_massSpectra->removeAt(jter);
+							--jter;
+							continue;
+						}
+					}
+				}
+			}
+			// End of for(int iter = 0; iter < rtListSize; ++iter)
+
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "Returning a list of" << p_massSpectra->size() << "spectra";
+
+			return p_massSpectra;
+		}
+
+
 	//! Return mass spectrum at index \p index.
 	const msXpSlibmass::MassSpectrum *
 		MassSpecDataSet::massSpectrum(int index) const
