@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #pragma once
 
 /////////////////////// Qt includes
@@ -39,15 +40,18 @@
 
 /////////////////////// Local includes
 #include <minexpert/nongui/globals.hpp>
+#include <minexpert/nongui/History.hpp>
+#include <globals/globals.hpp>
 #include <libmass/MassSpectrum.hpp>
 #include <libmass/MsMultiHash.hpp>
-#include <libmass/MsMultiHash.hpp>
-
+#include <minexpert/nongui/MassSpecDataStats.hpp>
 
 namespace msXpSmineXpert
 {
 
 
+	class MassSpecDataStats;
+
 	//! The MassSpecDataSet class provides a structured memory storage for mass data.
 	/*!
 
@@ -123,9 +127,8 @@ namespace msXpSmineXpert
 		//! Last (greatest) m/z value of the whole data set.
 		double m_mzRangeEnd = 0;
 
-
 		//! List of all the mass spectra in the data set.
-		/*! 
+		/*!
 
 			MassSpecDataSet owns the MassSpectrum instances allocated and stored in this list.
 			The spectra must be deleted upon destruction of \c this MassSpecDataSet instance.
@@ -135,7 +138,7 @@ namespace msXpSmineXpert
 
 		//! Hash relating retention time values with mass spectra.
 		/*!
-		
+
 			This hash is central in the working of the program: it allows to make a
 			quick relation between the retention times of a mass spectrometry
 			acquisition and the corresponding mass spectra.
@@ -148,12 +151,12 @@ namespace msXpSmineXpert
 			spectrum corresponding to a time slice of the drift cycle (that's called a
 			bin).
 
-		 */
+*/
 		msXpSlibmass::MsMultiHash m_rtHash;
-		
+
 		//! Hash relating ion mobility drift time values with mass spectra.
 		/*!
-		
+
 			This hash is central in the working of the program: it allows to make a
 			quick relation between the drift time of a mass spectrometry
 			acquisition and the corresponding mass spectra.
@@ -168,7 +171,7 @@ namespace msXpSmineXpert
 			given drift time there will be as many spectra as there were retention
 			time values.
 
-		 */
+*/
 		msXpSlibmass::MsMultiHash m_dtHash;
 
 		//! Stores the TIC chromatogram data when loading in streamed mode.
@@ -181,9 +184,11 @@ namespace msXpSmineXpert
 			start mining the data in a useful way. In this map, the key corresponds to
 			the retention time and the value to the corresponding TIC intensity value.
 
-		 */
+*/
 		QMap<double, double> m_ticChromMap;
 
+		//! Statistical data of the mass spectral data set.
+		MassSpecDataStats m_massSpecDataStats;
 
 		public:
 
@@ -193,6 +198,7 @@ namespace msXpSmineXpert
 				const QString &fileName = QString(), bool isStreamed = false);
 		~MassSpecDataSet();
 
+
 		QString name() const;
 		QString fileName() const;
 		bool isStreamed() const;
@@ -203,7 +209,20 @@ namespace msXpSmineXpert
 		double mzRangeStart() const;
 		double mzRangeEnd() const;
 
+		const MassSpecDataStats &makeStatistics(const History &history = History());
+
+		const MassSpecDataStats &incrementStatistics(const msXpSlibmass::MassSpectrum &massSpectrum);
+		const MassSpecDataStats &consolidateStatistics();
+
+		const MassSpecDataStats &statistics();
+
+		void resetStatistics();
+
+		QString statisticsAsText();
+
 		const QList<msXpSlibmass::MassSpectrum *> &massSpectra() const;
+		QList<msXpSlibmass::MassSpectrum *> *extractMassSpectra(const History &history = History()) const;
+
 		const msXpSlibmass::MsMultiHash& rtHash() const;
 		const msXpSlibmass::MsMultiHash& dtHash() const;
 		const QMap<double, double> &ticChromMap() const;
