@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include <omp.h>
 
 
@@ -58,7 +59,7 @@ namespace msXpSmineXpert
 	///////////////////////// PRIVATE ////////////////////
 	///////////////////////// PRIVATE ////////////////////
 
-	//! Fill-in the specifications to create the metaData table. 
+	//! Fill-in the specifications to create the metaData table.
 	/*!
 
 		When creating a new database, the creation of the tables is the very first
@@ -126,7 +127,7 @@ namespace msXpSmineXpert
 	/*!
 
 		\param specMetaData reference to the MassSpecFileMetaData to use to search
-		for values to INSERT in the database. 
+		for values to INSERT in the database.
 
 		\return a string containing the SQL INSERT statement to fill-in the metaData
 		table.
@@ -272,12 +273,12 @@ namespace msXpSmineXpert
 
 			QSqlQuery *sqlQuery;
 
-			if(db != nullptr)
+			if(db != Q_NULLPTR)
 				sqlQuery = new QSqlQuery(*db);
 			else
 				sqlQuery = new QSqlQuery(m_db);
 
-			if(sqlQuery == nullptr)
+			if(sqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 
@@ -331,7 +332,7 @@ namespace msXpSmineXpert
 		}
 
 
-	//! Fill-in the specifications to create the driftData table. 
+	//! Fill-in the specifications to create the driftData table.
 	/*!
 
 		When creating a new database, the creation of the tables is the very first
@@ -385,10 +386,10 @@ namespace msXpSmineXpert
 		MassSpecSqlite3Handler::writeDbDriftData(double driftTime, int driftBin,
 				int *recordId, QSqlDatabase *db)
 		{
-			if(recordId == nullptr)
+			if(recordId == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
-			// if(db != nullptr)
+			// if(db != Q_NULLPTR)
 			// qDebug() << __FILE__ << __LINE__
 			// << "The destination database has pointer: " << db
 			// << "and name:" << db->databaseName(); // Note
@@ -424,9 +425,9 @@ namespace msXpSmineXpert
 			// << "driftBin: " << driftBin;
 
 
-			QSqlQuery *sqlQuery = nullptr;
+			QSqlQuery *sqlQuery = Q_NULLPTR;
 
-			if(db != nullptr)
+			if(db != Q_NULLPTR)
 			{
 				// qDebug() << __FILE__ << __LINE__
 				// << "Destination database is not nullptr: " << db
@@ -443,7 +444,7 @@ namespace msXpSmineXpert
 				sqlQuery = new QSqlQuery(m_db);
 			}
 
-			if(sqlQuery == nullptr)
+			if(sqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			sqlQuery->prepare("SELECT driftData_id FROM driftData "
@@ -532,7 +533,7 @@ namespace msXpSmineXpert
 		}
 
 
-	//! Fill-in the specifications to create the spectralData table. 
+	//! Fill-in the specifications to create the spectralData table.
 	/*!
 
 		When creating a new database, the creation of the tables is the very first
@@ -616,7 +617,7 @@ namespace msXpSmineXpert
 			return true;
 		}
 
-	
+
 	//! Write to the database (\p db or \c m_db) the spectral data.
 	/*!
 
@@ -638,17 +639,17 @@ namespace msXpSmineXpert
 */
 	bool
 		MassSpecSqlite3Handler::writeDbSpectralData(
-				int idx, const QString &title, int peakCount, 
+				int idx, const QString &title, int peakCount,
 				int msLevel, double tic, double scanStartTime,
-				int driftDataId, double mzStart, double mzEnd, 
+				int driftDataId, double mzStart, double mzEnd,
 				const QByteArray &mzByteArray, const QByteArray &iByteArray,
 				QSqlDatabase *db)
 		{
 			// double start = omp_get_wtime();
 
-			QSqlQuery *sqlQuery = nullptr;
+			QSqlQuery *sqlQuery = Q_NULLPTR;
 
-			if(db != nullptr)
+			if(db != Q_NULLPTR)
 			{
 				// qDebug() << __FILE__ << __LINE__
 				// << "Using the other database:" << db
@@ -665,7 +666,7 @@ namespace msXpSmineXpert
 				sqlQuery = new QSqlQuery(m_db);
 			}
 
-			if(sqlQuery == nullptr)
+			if(sqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 
@@ -827,12 +828,12 @@ namespace msXpSmineXpert
 		{
 			QSqlQuery *sqlQuery;
 
-			if(db != nullptr)
+			if(db != Q_NULLPTR)
 				sqlQuery = new QSqlQuery(*db);
 			else
 				sqlQuery = new QSqlQuery(m_db);
 
-			if(sqlQuery == nullptr)
+			if(sqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			// This PRAGMA synchronous call is only really useful when the program runs
@@ -972,11 +973,17 @@ namespace msXpSmineXpert
 	MassSpecSqlite3Handler::MassSpecSqlite3Handler(const QString &fileName)
 	{
 		if(!fileName.isEmpty())
+		{
 			m_dbFileName = fileName;
-		// else
-		// qDebug() << __FILE__ << __LINE__ << "Filename for the database is empty "
-		//"in MassSpecSqlite3Handler "
-		//"constructor.";
+
+			if(!QFile(m_dbFileName).exists())
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+					<< "Attention the file does not exist:" << m_dbFileName;
+		}
+		else
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+				<< "Filename for the database is empty "
+				"in MassSpecSqlite3Handler constructor.";
 	}
 
 
@@ -1150,6 +1157,18 @@ namespace msXpSmineXpert
 		}
 
 
+	// Set the SQLite3 data file name
+	void
+		MassSpecSqlite3Handler::setFileName(const QString &fileName)
+		{
+			// Close the database because this object might be called multiple times
+			// with different file names.
+
+			closeDb();
+			m_dbFileName = fileName;
+		}
+
+
 	//! Close the database and remove the connection.
 	void
 		MassSpecSqlite3Handler::closeDb()
@@ -1247,7 +1266,7 @@ namespace msXpSmineXpert
 
 			if(!ok)
 			{
-				QString msg = 
+				QString msg =
 					QString("Failed to get proper data from the database. Error: %1. Exiting.\n")
 					.arg(sqlQuery.lastError().text());
 
@@ -1258,7 +1277,7 @@ namespace msXpSmineXpert
 
 			if(!sqlQuery.first())
 			{
-				QString msg = 
+				QString msg =
 					QString("Failed to get proper data from the database. Error: %1. Exiting.\n")
 					.arg(sqlQuery.lastError().text());
 
@@ -1432,7 +1451,7 @@ namespace msXpSmineXpert
 
 			if(!ok)
 			{
-				QString msg = 
+				QString msg =
 					QString("Failed to get proper data from the database. Error: %1. Exiting.\n")
 					.arg(sqlQuery.lastError().text());
 
@@ -1443,7 +1462,7 @@ namespace msXpSmineXpert
 
 			if(!sqlQuery.first())
 			{
-				QString msg = 
+				QString msg =
 					QString("Failed to get proper data from the database. Error: %1. Exiting.\n")
 					.arg(sqlQuery.lastError().text());
 
@@ -1548,7 +1567,7 @@ namespace msXpSmineXpert
 		MassSpecSqlite3Handler::exportData_metaData(QSqlDatabase *db,
 				int spectrumListCount)
 		{
-			if(db == nullptr)
+			if(db == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			// else
@@ -1567,14 +1586,14 @@ namespace msXpSmineXpert
 
 			QSqlQuery *srcSqlQuery = new QSqlQuery(m_db);
 
-			if(srcSqlQuery == nullptr)
+			if(srcSqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			bool ok = srcSqlQuery->exec(srcDbQueryString);
 
 			if(!ok)
 			{
-				QString msg = 
+				QString msg =
 					QString("Failed to get proper data from the database. Error: %1. Exiting.\n")
 					.arg(srcSqlQuery->lastError().text());
 
@@ -1674,7 +1693,7 @@ namespace msXpSmineXpert
 	bool
 		MassSpecSqlite3Handler::exportData_driftData(QSqlDatabase *db)
 		{
-			if(db == nullptr)
+			if(db == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			// else
@@ -1691,7 +1710,7 @@ namespace msXpSmineXpert
 
 			QSqlQuery *srcSqlQuery = new QSqlQuery(m_db);
 
-			if(srcSqlQuery == nullptr)
+			if(srcSqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			QString srcDbQueryString = "SELECT driftTime, driftBin FROM driftData ";
@@ -1700,7 +1719,7 @@ namespace msXpSmineXpert
 
 			if(!ok)
 			{
-				QString msg = 
+				QString msg =
 					QString("Failed to get proper data from the database. Error: %1. Exiting.\n")
 					.arg(srcSqlQuery->lastError().text());
 
@@ -1762,9 +1781,10 @@ namespace msXpSmineXpert
 		writeDbSpectralData()
 
 */
-	bool
+	QString
 		MassSpecSqlite3Handler::exportData(QString fileName, History history)
 		{
+			QString errors;
 
 			// This is a huge task. We need to read data from m_dbFileName (that we
 			// know from the constructor. m_dbFileName is the name of the database
@@ -1785,11 +1805,13 @@ namespace msXpSmineXpert
 
 			if(!ok)
 			{
-				qDebug()
-					<< __FILE__ << __LINE__
-					<< "Opening data export destination database: Failure. Exiting.\n";
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("Opening data export destination database: Failure\n");
 
-				exit(1);
+				return errors;
 			}
 			else
 			{
@@ -1800,11 +1822,13 @@ namespace msXpSmineXpert
 
 			if(!createDbTables(&destDb))
 			{
-				qDebug()
-					<< __FILE__ << __LINE__
-					<< "Creating tables for destination database: Failure. Exiting.\n";
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("Creating tables for destination database: Failure\n");
 
-				exit(1);
+				return errors;
 			}
 			else
 			{
@@ -1842,13 +1866,16 @@ namespace msXpSmineXpert
 			double dtEnd = -1;
 			dtIntegration = history.innermostDtRange(&dtStart, &dtEnd);
 
+			qDebug() << __FILE__ << __LINE__
+				<< "DT integration:" << dtIntegration << "[" << dtStart << "," << dtEnd << "]";
+
 			// We need to craft a SqlQuery string that is specific of the experimental
 			// situation: are we dealing with drift data or not.
 
 			bool wasDriftExperiment = isDriftExperiment();
 
-			// qDebug() << __FILE__ << __LINE__
-			// << "wasDriftExperiment: " << wasDriftExperiment;
+			qDebug() << __FILE__ << __LINE__
+				<< "wasDriftExperiment: " << wasDriftExperiment;
 
 			if(wasDriftExperiment)
 			{
@@ -1934,6 +1961,10 @@ namespace msXpSmineXpert
 
 				if(dtIntegration)
 				{
+					qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+						<< "dtIntegration is true with dtStart:" << dtStart
+						<< "dtEnd:" << dtEnd;
+
 					if(oneAlready)
 					{
 						whereStatements +=
@@ -1944,7 +1975,7 @@ namespace msXpSmineXpert
 					else
 					{
 						whereStatements +=
-							QString("WHERE (driftTime >= %1) AND (driftTime <= %2) ")
+							QString(" (driftTime >= %1) AND (driftTime <= %2) ")
 							.arg(dtStart, 0, 'f', 12)
 							.arg(dtEnd, 0, 'f', 12);
 
@@ -1963,7 +1994,7 @@ namespace msXpSmineXpert
 
 			QSqlQuery *srcSqlQuery = new QSqlQuery(m_db);
 
-			if(srcSqlQuery == nullptr)
+			if(srcSqlQuery == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			// Now craft the whole query depending on the needs.
@@ -1973,40 +2004,52 @@ namespace msXpSmineXpert
 				condFields + fromStatement + joinStatements +
 				whereStatements;
 
-			// qDebug() << __FILE__ << __LINE__
-			// << "For record counting, the query string is: " << srcDbQueryString;
+			qDebug() << __FILE__ << __LINE__
+				<< "For record counting, the query string is: " << srcDbQueryString;
 
 			ok = srcSqlQuery->exec(srcDbQueryString);
 
 			if(!ok)
 			{
-				qDebug() << __FILE__ << __LINE__
-					<< "Failed to get proper data from the database.\n"
-					<< srcSqlQuery->lastError() << "Exiting.\n";
+				errors += QString("%1 %2 %3 %4 %5")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("Failed to get proper data from the database with errors:\n")
+					.arg(srcSqlQuery->lastError().text());
 
-				exit(1);
+				return errors;
 			}
 
 			if(!srcSqlQuery->isActive())
 			{
-				qFatal(
-						"Fatal error at %s@%d. The query is not active. Program aborted.",
-						__FILE__, __LINE__);
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("The query is not active.");
+
+				return errors;
 			}
 
 			// Get the number of records that were retrieved, because we'll need this
 			// to provide feedback to the caller.
 			if(!srcSqlQuery->first())
 			{
-				qDebug() << __FILE__ << __LINE__
-					<< "Failed to get the number of records. Returning.";
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("Failed to get the number of records.");
+
+				return errors;
 			}
 
 			int recordCount =
 				srcSqlQuery->value(srcSqlQuery->record().indexOf("COUNT(*)")).toInt();
 
-			// qDebug() << __FILE__ << __LINE__
-			// << "The recordCount:" << recordCount;
+			qDebug() << __FILE__ << __LINE__
+				<< "The recordCount:" << recordCount;
 
 			// Now reset the query
 			srcSqlQuery->finish();
@@ -2016,25 +2059,31 @@ namespace msXpSmineXpert
 			srcDbQueryString = openingStatement + uncondFields + condFields +
 				fromStatement + joinStatements + whereStatements;
 
-			// qDebug() << __FILE__ << __LINE__
-			// << "For data export, now, the query string is: " << srcDbQueryString;
+			qDebug() << __FILE__ << __LINE__
+				<< "For data export, now, the query string is: " << srcDbQueryString;
 
 			ok = srcSqlQuery->exec(srcDbQueryString);
 
 			if(!ok)
 			{
-				qDebug() << __FILE__ << __LINE__
-					<< "Failed to get proper data from the database.\n"
-					<< srcSqlQuery->lastError() << "Exiting.\n";
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("Failed to get proper data from the database.");
 
-				exit(1);
+				return errors;
 			}
 
 			if(!srcSqlQuery->isActive())
 			{
-				qFatal(
-						"Fatal error at %s@%d. The query is not active. Program aborted.",
-						__FILE__, __LINE__);
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("The query is not active.");
+
+				return errors;
 			}
 
 			// Commented out because it crashes the program below ? FIXME: Why ?
@@ -2082,9 +2131,15 @@ namespace msXpSmineXpert
 					fieldNo = srcSqlQuery->record().indexOf("driftData_id");
 
 					if(fieldNo == -1)
-						qFatal("Fatal error at %s@%d. Program aborted.", __FILE__,
-								__LINE__);
+					{
+						errors += QString("%1 %2 %3 %4")
+							.arg(__FILE__)
+							.arg(__LINE__)
+							.arg(__FUNCTION__)
+							.arg("Drift experiment, but no driftData_id record found in the database.");
 
+						return errors;
+					}
 					driftData_id = srcSqlQuery->value(fieldNo).toInt();
 				}
 				else
@@ -2140,15 +2195,17 @@ namespace msXpSmineXpert
 				// the spectralData table of the destination database.
 
 				if(!writeDbSpectralData(idx, title, peakCount, msLevel, tic,
-							scanStartTime, driftData_id, 
+							scanStartTime, driftData_id,
 							mzStart, mzEnd, mzByteArray,
 							iByteArray, &destDb))
 				{
-					qDebug() << __FILE__ << __LINE__
-						<< "Failed to write the mass spectral data to the "
-						"spectralData table of the destination database.";
+					errors += QString("%1 %2 %3 %4")
+						.arg(__FILE__)
+						.arg(__LINE__)
+						.arg(__FUNCTION__)
+						.arg("Failed to write the mass data to the spectralData table.");
 
-					return false;
+					return errors;
 				}
 
 				++spectrumListCount;
@@ -2161,14 +2218,19 @@ namespace msXpSmineXpert
 
 			// Finally, we can export the metaData stuff:
 			if(!exportData_metaData(&destDb, spectrumListCount))
-				qDebug() << __FILE__ << __LINE__ << "Failed to export the meta data to "
-					"the metaData table of the "
-					"destination database.";
+			{
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("Failed to write the meta data to the metaData table.");
 
+				return errors;
+			}
 			// And now close the destination database.
 			destDb.close();
 
-			return true;
+			return errors;
 		}
 
 
