@@ -1,36 +1,37 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
+
 #pragma once
 
 
@@ -148,6 +149,8 @@ namespace msXpSmineXpert
 		bool initializeDbForRead(const QString &fileName = QString());
 		bool initializeDbForWrite();
 
+		void setFileName(const QString &fileName);
+
 		void closeDb();
 
 		MassSpecFileMetaData getMassSpecFileMetaDataFromDb();
@@ -157,13 +160,13 @@ namespace msXpSmineXpert
 		QList<double> rtList();
 
 		void mzRange(double &mzStart, double &mzEnd);
-		
+
 		bool isDriftExperiment();
 		QList<double> *driftTimeList();
 
 		bool exportData_metaData(QSqlDatabase *db, int spectrumListCount);
 		bool exportData_driftData(QSqlDatabase *db);
-		bool exportData(QString fileName, History history);
+		QString exportData(QString fileName, History history);
 
 signals:
 		void dataLoadMessageSignal(QString msg, int logType);
