@@ -1,37 +1,38 @@
-/*  BEGIN software license
+/* BEGIN software license
  *
- *  msXpertSuite - mass spectrometry software suite
- *  -----------------------------------------------
- *  Copyright(C) 2009, 2017 Filippo Rusconi
+ * msXpertSuite - mass spectrometry software suite
+ * -----------------------------------------------
+ * Copyright(C) 2009,...,2018 Filippo Rusconi
  *
- *  http://www.msxpertsuite.org
+ * http://www.msxpertsuite.org
  *
- *  This file is part of the msXpertSuite project.
+ * This file is part of the msXpertSuite project.
  *
- *  The msXpertSuite project is the successor of the massXpert project. This
- *  project now includes various independent modules:
- *  
- *  - massXpert, model polymer chemistries and simulate mass spectrometric data;
- *  - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
+ * The msXpertSuite project is the successor of the massXpert project. This
+ * project now includes various independent modules:
  *
- *  This program is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * - massXpert, model polymer chemistries and simulate mass spectrometric data;
+ * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  * END software license
  */
 
 
 
+
 #include "qmath.h"
 
 /////////////////////// Qt includes
@@ -47,10 +48,239 @@
 namespace msXpSmineXpert
 {
 
+	int sliceSpecifMetaTypeId =
+		qRegisterMetaType<msXpSmineXpert::SliceSpecif>("msXpSmineXpert::SliceSpecif");
+
+	int sqlMassDataSlicerMetaTypeId =
+		qRegisterMetaType<msXpSmineXpert::SqlMassDataSlicer>("msXpSmineXpert::SqlMassDataSlicer");
+	int sqlMassDataSlicerPtrMetaTypeId =
+		qRegisterMetaType<msXpSmineXpert::SqlMassDataSlicer*>("msXpSmineXpert::SqlMassDataSlicer*");
+
+	QMap<int, QString> sliceTypeMap = {
+		{SliceType::SLICE_NOT_SET, "NOT_SET"},
+		{SliceType::SLICE_ZOOMED_REGION, "ZOOMED_REGION"},
+		{SliceType::SLICE_BY_COUNT, "BY_COUNT"},
+		{SliceType::SLICE_BY_SIZE, "BY_SIZE"} };
+
+
 	///////////////////////// PRIVATE ////////////////////
 	///////////////////////// PRIVATE ////////////////////
 	///////////////////////// PRIVATE ////////////////////
 
+	//! Deletes from memory all the SliceSpecif instances.
+	/*!
+
+		The \c m_sliceSpecifList is emptied and each instance is deleted.
+
+*/
+	void
+		SqlMassDataSlicer::clearSliceSpecifications()
+		{
+			while(!m_sliceSpecifList.isEmpty())
+				delete m_sliceSpecifList.takeFirst();
+		}
+
+
+	///////////////////////// PUBLIC ////////////////////
+	///////////////////////// PUBLIC ////////////////////
+	///////////////////////// PUBLIC ////////////////////
+
+
+	//! Construct a SqlMassDataSlicer instance
+	SqlMassDataSlicer::SqlMassDataSlicer()
+	{
+	}
+
+
+	//! Construct a SqlMassDataSlicer instance on the basis of \p other.
+	/*!
+
+		The copying of the \p other data into \c this instance is deep with a copy
+		of all the \c m_sliceSpecifList items.
+
+		Note that the value of \p other.mp_sqlHandler cannot be nullptr and that
+		pointer is copied as is in \c this instance.
+
+		\sa SqlMassDataSlicer().
+
+*/
+	SqlMassDataSlicer::SqlMassDataSlicer(const SqlMassDataSlicer &other)
+		:
+			m_history{other.m_history},
+		m_srcFileName{other.m_srcFileName},
+		m_formatString{other.m_formatString},
+		m_sliceType{other.m_sliceType},
+		m_rangeStart{other.m_rangeStart},
+		m_rangeEnd{other.m_rangeEnd},
+		m_sliceCount{other.m_sliceCount},
+		m_sliceSize{other.m_sliceSize}
+	{
+		for(int iter = 0; iter < other.m_sliceSpecifList.size(); ++iter)
+		{
+			SliceSpecif *specif = new SliceSpecif;
+
+			specif->fileName = other.m_sliceSpecifList.at(iter)->fileName;
+			specif->start = other.m_sliceSpecifList.at(iter)->start;
+			specif->end = other.m_sliceSpecifList.at(iter)->end;
+			specif->number = other.m_sliceSpecifList.at(iter)->number;
+
+			m_sliceSpecifList.append(specif);
+		}
+	}
+
+
+
+	//! Construct a SqlMassDataSlicer instance.
+	/*!
+
+		The construction involves the initialization of member data based on the
+		parameters given by the function caller.
+
+		\param history History instance to be used to characterize the data that
+		should be sliced.
+
+		\param rangeStart value representing the left border of the initial data
+		range to be sliced.
+
+		\param rangeEnd value representing the right border of the initial data
+		range to be sliced.
+
+		\sa SqlMassDataSlicer().
+
+*/
+	SqlMassDataSlicer::SqlMassDataSlicer(History history, double rangeStart, double rangeEnd)
+		:
+			m_history{history},
+		m_rangeStart{rangeStart},
+		m_rangeEnd{rangeEnd}
+	{
+	}
+
+
+	//! Destruct \c this SqlMassDataSlicer instance.
+	/*!
+
+		The SliceSpecif instances allocated and stored in \c m_sliceSpecifList are
+		deleted.
+
+*/
+	SqlMassDataSlicer::~SqlMassDataSlicer() {
+		// We do not own mp_sqlHandler.
+
+		clearSliceSpecifications();
+	}
+
+
+	//! Initialize the initial member data
+	SqlMassDataSlicer &
+		SqlMassDataSlicer::initialize(QString fileName,
+				double rangeStart,
+				double rangeEnd,
+				const History &history)
+		{
+			if(!QFileInfo(fileName).isAbsolute())
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"The file path name must be absolute."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
+			setSrcFileName(fileName);
+			setRangeStart(rangeStart);
+			setRangeEnd(rangeEnd);
+			setHistory(history);
+
+			return *this;
+		}
+
+
+	//! Initialize \c this Assignment operator.
+	/*!
+
+		\c this instance is made identical to \p other. The copying of the \p other
+		instance data into \c this instance is deep.
+
+		\param other reference to another OASqlMassDataSlicer instance to be used
+		for the initialization of \c this instance.
+
+		\return a reference to \c this instance.
+
+		\sa SqlMassDataSlicer(const SqlMassDataSlicer &other).
+
+*/
+	SqlMassDataSlicer &
+		SqlMassDataSlicer::initialize(const SqlMassDataSlicer &other)
+		{
+			if(&other == this)
+				return *this;
+
+			// Do not copy the pointer to the MassSpecSqlite3Handler!
+
+			m_history = other.m_history;
+			m_rangeStart = other.m_rangeStart;
+			m_rangeEnd = other.m_rangeEnd;
+			m_srcFileName = other.m_srcFileName;
+			m_formatString = other.m_formatString;
+			m_sliceType = other.m_sliceType;
+			m_rangeStart = other.m_rangeStart;
+			m_rangeEnd = other.m_rangeEnd;
+			m_sliceCount = other.m_sliceCount;
+			m_sliceSize = other.m_sliceSize;
+
+			for(int iter = 0; iter < other.m_sliceSpecifList.size(); ++iter)
+			{
+				SliceSpecif *specif = new SliceSpecif;
+
+				specif->fileName = other.m_sliceSpecifList.at(iter)->fileName;
+				specif->start = other.m_sliceSpecifList.at(iter)->start;
+				specif->end = other.m_sliceSpecifList.at(iter)->end;
+				specif->number = other.m_sliceSpecifList.at(iter)->number;
+
+				m_sliceSpecifList.append(specif);
+			}
+
+			return *this;
+		}
+
+
+	//! Assignment operator.
+	/*!
+
+		\c this instance is made identical to \p other. The copying of the \p other
+		instance data into \c this instance is deep.
+
+		\param other reference to another OASqlMassDataSlicer instance to be used
+		for the initialization of \c this instance.
+
+		\return a reference to \c this instance.
+
+		\sa SqlMassDataSlicer(const SqlMassDataSlicer &other).
+		\sa SqlMassDataSlicer::initialize(const SqlMassDataSlicer &other).
+
+*/
+	SqlMassDataSlicer &
+		SqlMassDataSlicer::operator=(const SqlMassDataSlicer &other)
+		{
+			return initialize(other);
+		}
+
+
+	//! Reset \c this SqlMassDataSlicer instance.
+	/*!
+
+		Empties this instance (See also) and resets variables to default values.
+
+		\sa clearSliceSpecifications().
+		*/
+	void
+		SqlMassDataSlicer::reset()
+		{
+			m_sliceType = SliceType::SLICE_NOT_SET;
+			m_formatString.clear();
+			clearSliceSpecifications();
+			m_sliceCount = 0;
+			m_sliceSize = 0;
+		}
+
 
 	//! Configure a number of the slicing operation parameters.
 	/*!
@@ -67,7 +297,7 @@ namespace msXpSmineXpert
 	bool
 		SqlMassDataSlicer::configureSpecifs(QString *errors)
 		{
-			if(errors == nullptr)
+			if(errors == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			QString errorString;
@@ -102,10 +332,10 @@ namespace msXpSmineXpert
 				// We want a round number of slices
 				m_sliceCount = ceil(origDataRange / m_sliceSize);
 
-				// qDebug() << __FILE__ << __LINE__
-				// << "m_sliceSize is" << m_sliceSize
-				// << "Setting m_sliceCount to" << m_sliceCount
-				// << "because SliceType::SLICE_BY_SIZE";
+				//qDebug() << __FILE__ << __LINE__
+				//<< "m_sliceSize is" << m_sliceSize
+				//<< "Setting m_sliceCount to" << m_sliceCount
+				//<< "because SliceType::SLICE_BY_SIZE";
 			}
 			else
 			{
@@ -124,12 +354,12 @@ namespace msXpSmineXpert
 				// We have the [start-end] data range to slice by a slice count
 				m_sliceSize = origDataRange / m_sliceCount;
 
-				// qDebug() << __FILE__ << __LINE__
-				// << "Setting m_sliceSize to" << m_sliceSize;
+				//qDebug() << __FILE__ << __LINE__
+				//<< "Setting m_sliceSize to" << m_sliceSize;
 			}
 
-			// qDebug() << __FILE__ << __LINE__
-			// << "Before making the slicing:" << asText();
+			qDebug() << __FILE__ << __LINE__
+				<< "Before making the slicing, SqlMassDataSlicer.asText():" << asText();
 
 			// At this point, m_sliceCount and m_sliceRange will have been filled-up
 			// with proper values.
@@ -182,182 +412,128 @@ namespace msXpSmineXpert
 		}
 
 
-	//! Deletes from memory all the SliceSpecif instances.
-	/*!
-
-		The \c m_sliceSpecifList is emptied and each instance is deleted.
-
-*/
+	//! Set the smallest value of the range to be sliced off
 	void
-		SqlMassDataSlicer::clearSliceSpecifications()
+		SqlMassDataSlicer::setRangeStart(double value)
 		{
-			while(!m_sliceSpecifList.isEmpty())
-				delete m_sliceSpecifList.takeFirst();
+			m_rangeStart = value;
 		}
 
 
-	//! Reset \c this SqlMassDataSlicer instance.
-	/*!
+	//! Return the smallest value of the range to be sliced off
+	double
+		SqlMassDataSlicer::rangeStart() const
+		{
+			return m_rangeStart;
+		}
 
-		Empties this instance (See also) and resets variables to default values.
 
-		\sa clearSliceSpecifications().
-		*/
+	//! Set the greatest value of the range to be sliced off
 	void
-		SqlMassDataSlicer::reset()
+		SqlMassDataSlicer::setRangeEnd(double value)
 		{
-			m_sliceType = SliceType::SLICE_NOT_SET;
-			m_formatString.clear();
-			clearSliceSpecifications();
-			m_sliceCount = 0;
-			m_sliceSize = 0;
+			m_rangeEnd = value;
 		}
 
 
-
-	///////////////////////// PUBLIC ////////////////////
-	///////////////////////// PUBLIC ////////////////////
-	///////////////////////// PUBLIC ////////////////////
-
-
-	//! Construct a SqlMassDataSlicer instance on the basis of \p other.
-	/*!
-
-		The copying of the \p other data into \c this instance is deep with a copy
-		of all the \c m_sliceSpecifList items.
-
-		Note that the value of \p other.mp_sqlHandler cannot be nullptr and that
-		pointer is copied as is in \c this instance.
-
-		\sa SqlMassDataSlicer().
-
-*/
-	SqlMassDataSlicer::SqlMassDataSlicer(const SqlMassDataSlicer &other)
-		: 
-			mp_sqlHandler{other.mp_sqlHandler},
-		m_history{other.m_history},
-		m_srcFileName{other.m_srcFileName},
-		m_formatString{other.m_formatString},
-		m_sliceType{other.m_sliceType},
-		m_rangeStart{other.m_rangeStart},
-		m_rangeEnd{other.m_rangeEnd},
-		m_sliceCount{other.m_sliceCount},
-		m_sliceSize{other.m_sliceSize}
-	{
-		if(mp_sqlHandler == nullptr)
-			qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
-
-		for(int iter = 0; iter < other.m_sliceSpecifList.size(); ++iter)
+	//! Return the greatest value of the range to be sliced off
+	double
+		SqlMassDataSlicer::rangeEnd() const
 		{
-			SliceSpecif *specif = new SliceSpecif;
-
-			specif->fileName = other.m_sliceSpecifList.at(iter)->fileName;
-			specif->start = other.m_sliceSpecifList.at(iter)->start;
-			specif->end = other.m_sliceSpecifList.at(iter)->end;
-			specif->number = other.m_sliceSpecifList.at(iter)->number;
-
-			m_sliceSpecifList.append(specif);
+			return m_rangeEnd;
 		}
-	}
 
 
-
-	//! Construct a SqlMassDataSlicer instance.
-	/*!
-
-		The construction involves the initialization of member data based on the
-		parameters given by the function caller.
-
-		\param sqlHandler pointer to the MassSpecSqlite3Handler instance responsible
-		for handling the SQLite3-based database format file. Cannot be nullptr.
-
-		\param history History instance to be used to characterize the data that
-		should be sliced.
-
-		\param rangeStart value representing the left border of the initial data
-		range to be sliced.
-
-		\param rangeEnd value representing the right border of the initial data
-		range to be sliced.
-
-		\sa SqlMassDataSlicer().
-
-*/
-	SqlMassDataSlicer::SqlMassDataSlicer(MassSpecSqlite3Handler *sqlHandler,
-			History history, double rangeStart, double rangeEnd)
-		:
-			mp_sqlHandler{sqlHandler},
-		m_history{history},
-		m_rangeStart{rangeStart},
-		m_rangeEnd{rangeEnd}
-	{
-		if(mp_sqlHandler == nullptr)
-			qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
-	}
-
-
-	//! Destruct \c this SqlMassDataSlicer instance.
-	/*!
-
-		The SliceSpecif instances allocated and stored in \c m_sliceSpecifList are
-		deleted.
-
-*/
-	SqlMassDataSlicer::~SqlMassDataSlicer() {
-		// We do not own mp_sqlHandler.
-
-		clearSliceSpecifications();
-	}
-
-
-	//! Assignment operator.
-	/*!
-
-		\c this instance is made identical to \p other. The copying of the \p other
-		instance data into \c this instance is deep. 
-
-		\param other reference to another OASqlMassDataSlicer instance to be used
-		for the initialization of \c this instance.
-
-		\return a reference to \c this instance.
-
-		\sa SqlMassDataSlicer(const SqlMassDataSlicer &other).
-
-*/
-	SqlMassDataSlicer &
-		SqlMassDataSlicer::operator=(const SqlMassDataSlicer &other)
+	//! Set the values of the range to be sliced off
+	void
+		SqlMassDataSlicer::setRange(double start, double end)
 		{
-			if(&other == this)
-				return *this;
-
-			mp_sqlHandler = other.mp_sqlHandler;
-			m_history = other.m_history;
-			m_rangeStart = other.m_rangeStart;
-			m_rangeEnd = other.m_rangeEnd;
-			m_srcFileName = other.m_srcFileName;
-			m_formatString = other.m_formatString;
-			m_sliceType = other.m_sliceType;
-			m_rangeStart = other.m_rangeStart;
-			m_rangeEnd = other.m_rangeEnd;
-			m_sliceCount = other.m_sliceCount;
-			m_sliceSize = other.m_sliceSize;
-
-			for(int iter = 0; iter < other.m_sliceSpecifList.size(); ++iter)
-			{
-				SliceSpecif *specif = new SliceSpecif;
-
-				specif->fileName = other.m_sliceSpecifList.at(iter)->fileName;
-				specif->start = other.m_sliceSpecifList.at(iter)->start;
-				specif->end = other.m_sliceSpecifList.at(iter)->end;
-				specif->number = other.m_sliceSpecifList.at(iter)->number;
-
-				m_sliceSpecifList.append(specif);
-			}
-
-			return *this;
+			setRangeStart(start);
+			setRangeEnd(end);
 		}
 
 
+	//! Set the caller widget's History
+	void
+		SqlMassDataSlicer::setHistory(const History &history)
+		{
+			m_history = history;
+		}
+
+
+	//! Set the type of slicing method
+	void
+		SqlMassDataSlicer::setSliceType(int type)
+		{
+			m_sliceType = type;
+		}
+
+
+	//! Set the size of the slices
+	void
+		SqlMassDataSlicer::setSliceSize(double size)
+		{
+			m_sliceSize = size;
+		}
+
+	//! Set the number of slices
+	void
+		SqlMassDataSlicer::setSliceCount(int count)
+		{
+			m_sliceCount = count;
+		}
+
+
+	//! Set the directory where the files have to be created
+	void
+		SqlMassDataSlicer::setDestDirectory(QString dir)
+		{
+			m_destDirectory = dir;
+		}
+
+
+	//! Return the directory where the files have to be created
+	QString
+		SqlMassDataSlicer::destDirectory() const
+		{
+			return m_destDirectory;
+		}
+
+
+	//! Set the source data file name
+	void
+		SqlMassDataSlicer::setSrcFileName(QString fileName)
+		{
+			m_srcFileName = fileName;
+			//qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+			//<< "m_srcFileName:" << m_srcFileName;
+		}
+
+
+	//! Return the source data file name
+	QString
+		SqlMassDataSlicer::srcFileName() const
+		{
+			return m_srcFileName;
+		}
+
+
+	//! Set the format string to be used to craft filenames
+	void
+		SqlMassDataSlicer::setFormatString(QString format)
+		{
+			m_formatString = format;
+		}
+
+	//! Return the format string to be used to craft filenames
+	QString
+		SqlMassDataSlicer::formatString() const
+		{
+			return m_formatString;
+		}
+
+
+
 	//! Craft the name of the slice file according to the \p sliceSpecif.
 	/*!
 
@@ -381,7 +557,7 @@ namespace msXpSmineXpert
 			if(m_formatString.isEmpty())
 				return QString();
 
-			if(sliceSpecif == nullptr)
+			if(sliceSpecif == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			QString fileName;
@@ -541,37 +717,58 @@ namespace msXpSmineXpert
 		already, the file is "silently" overwritten with only a debugging message
 		sent to the terminal window.
 
+		\param mp_sqlHandler pointer to the MassSpecSqlite3Handler to use for the
+		slicing operation.
+
 		\return true if the slicing operation succeeded, false otherwise.
 
 */
-	bool
-		SqlMassDataSlicer::slice()
+	QString
+		SqlMassDataSlicer::slice(MassSpecSqlite3Handler *mp_sqlHandler)
 		{
+			if(mp_sqlHandler == Q_NULLPTR)
+				qFatal("Fatal error at %s@%d -- %s(). "
+						"Cannot be that the pointer to the SQLite3 data handler is nullptr."
+						"Program aborted.",
+						__FILE__, __LINE__, __FUNCTION__);
+
 			int sliceSpecifListSize = m_sliceSpecifList.size();
 
+			// Set aside a string to collect the errors and return them.
+			QString errors;
+
 			if(sliceSpecifListSize == 0)
 			{
-				qDebug() << __FILE__ << __LINE__
-					<< "There are no slice specifications in the list.";
+				errors += QString("%1 %2 %3 %4")
+					.arg(__FILE__)
+					.arg(__LINE__)
+					.arg(__FUNCTION__)
+					.arg("There are no slice specifications in the list.");
 
-				return false;
+				return errors;
 			}
 
+			qDebug() << __FILE__ << __LINE__ << __FUNCTION__ << "()"
+				<< "allSliceSpecifsAsText:" << allSliceSpecifsAsText();
+
 			for(int iter = 0; iter < sliceSpecifListSize; ++iter)
 			{
 				SliceSpecif *sliceSpecif = m_sliceSpecifList.at(iter);
 
 				if(sliceSpecif->fileName.isEmpty())
 				{
-					qDebug() << __FILE__ << __LINE__ << "The file name is empty.";
+					errors += QString("%1 %2 %3 %4")
+						.arg(__FILE__)
+						.arg(__LINE__)
+						.arg(__FUNCTION__)
+						.arg("The file name is empty.");
 
-					return false;
+					return errors;
 				}
 
 				QFile file(sliceSpecif->fileName);
 				if(file.exists())
 				{
-
 					qDebug() << __FILE__ << __LINE__ << "File" << sliceSpecif->fileName
 						<< "exists already, overwriting it.";
 
@@ -581,60 +778,65 @@ namespace msXpSmineXpert
 
 				emit updateSliceFileName(sliceSpecif->fileName);
 
-				// Get the last history item that is of any ARBITRARY IntegrationType.
-				// We'll set the range limits to the proper values.
-				HistoryItem *item = nullptr;
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+					<< "Now handling slice:" << sliceSpecifAsText(sliceSpecif);
 
-				IntegrationRange *integrationRange = nullptr;
+				// We need to update *INSIDE, that is IN PLACE* the member History the
+				// last item such that the slicer will have the sliceSpecif range values
+				// for current slice.
 
-				item = m_history.lastHistoryItem(IntegrationType::ANY_TO_RT,
+				HistoryItem *item = Q_NULLPTR;
+
+				IntegrationRange *integrationRange = Q_NULLPTR;
+
+				item = m_history.lastHistoryItem(IntegrationType::RT_TO_XXX,
 						&integrationRange);
-				if(item == nullptr)
-					item = m_history.lastHistoryItem(IntegrationType::ANY_TO_MZ,
+				if(item == Q_NULLPTR)
+					item = m_history.lastHistoryItem(IntegrationType::MZ_TO_XXX,
 							&integrationRange);
-				if(item == nullptr)
-					item = m_history.lastHistoryItem(IntegrationType::ANY_TO_DT,
+				if(item == Q_NULLPTR)
+					item = m_history.lastHistoryItem(IntegrationType::DT_TO_XXX,
 							&integrationRange);
-				if(item == nullptr)
+				if(item == Q_NULLPTR)
 				{
-					qDebug()
-						<< __FILE__ << __LINE__
-						<< "There is not XX_ARBITRARY IntegrationType history item.";
+					errors += QString("%1 %2 %3 %4")
+						.arg(__FILE__)
+						.arg(__LINE__)
+						.arg(__FUNCTION__)
+						.arg("There is no XXX_TO_YY IntegrationType history item.");
 
-					return false;
+					return errors;
 				}
 
-				//qDebug() << __FILE__ << __LINE__
-					//<< "Last history item:" << item->asText(false);
+				// Modify *IN PLACE* the integration ranges to match the current slice limits.
 
-				// Modify the integration ranges to match the current slice limits.
 				integrationRange->start = sliceSpecif->start;
 				integrationRange->end = sliceSpecif->end;
 
-				//qDebug() << __FILE__ << __LINE__
-					//<< "History:" << m_history.asText();
+				qDebug() << __FILE__ << __LINE__ << __FUNCTION__
+					<< "m_history as transmitted to the sql handler:" << m_history.asText();
 
-				qDebug() << __FILE__ << __LINE__
-					<< "Handling slice:" << sliceSpecifAsText(sliceSpecif);
+				mp_sqlHandler->setFileName(m_srcFileName);
+				errors = mp_sqlHandler->exportData(sliceSpecif->fileName, m_history);
 
-				// At this point we are all set to start making the export:
-				if(!mp_sqlHandler->exportData(sliceSpecif->fileName, m_history))
-					return false;
+				// If there were errors, then return immediately.
+				if(!errors.isEmpty())
+					return errors;
 			}
 			// End of
 			// for(int iter = 0; iter < sliceSpecifListSize; ++iter)
 
-			return true;
+			return errors;
 		}
 
 
 	//! Create a textual representation of \c this SqlMassDataSlicer instance contents.
 	/*!
-	 
+
 		\return a string containing a textual representation of \c this
 		SqlMassDataSlicer instance contents.
-	
-	 */
+
+*/
 	QString
 		SqlMassDataSlicer::asText() const
 		{
@@ -660,18 +862,17 @@ namespace msXpSmineXpert
 		}
 
 
-
 	//! Create a textual representation of \c specif.
 	/*!
-	 
+
 		\return a string containing a textual representation of \c this
 		SqlMassDataSlicer instance contents.
-	
-	 */
+
+*/
 	QString
 		SqlMassDataSlicer::sliceSpecifAsText(SliceSpecif *specif) const
 		{
-			if(specif == nullptr)
+			if(specif == Q_NULLPTR)
 				qFatal("Fatal error at %s@%d. Program aborted.", __FILE__, __LINE__);
 
 			QString text;
@@ -691,11 +892,12 @@ namespace msXpSmineXpert
 		The SliceSpecif instances stored in \c m_sliceSpecifList are iterated over
 		and for each one a string is created (See also) and concatenated to a local
 		string. That local string is then returned.
-	 
+
 		\return a string containing a textual representation of all the SliceSpecif
 		instances stored in \c m_sliceSpecifList.
-	
-	 */	QString
+
+*/
+	QString
 		SqlMassDataSlicer::allSliceSpecifsAsText() const
 		{
 			QString text;
